@@ -35,6 +35,7 @@ namespace MonoDevelop.Components.Commands
 	{
 		ArrayList list = new ArrayList ();
 		CommandInfo defaultInfo;
+		bool bypass;
 		
 		internal CommandArrayInfo (CommandInfo defaultInfo)
 		{
@@ -71,5 +72,12 @@ namespace MonoDevelop.Components.Commands
 		{
 			return list.GetEnumerator ();
 		}
+		
+		// When set in an update handler, the command manager will ignore this handler method
+		// and will keep looking in the command route.
+		public bool Bypass {
+			get { return bypass; }
+			set { bypass = value; }
+		}
 	}
 }
