@@ -42,11 +42,13 @@ namespace MonoDevelop.Components.Commands
 		ArrayList toolbars = new ArrayList ();
 		ArrayList globalHandlers = new ArrayList ();
 		ArrayList commandUpdateErrors = new ArrayList ();
+		ArrayList visitors = new ArrayList ();
 		Hashtable overloadedAccelCommands = new Hashtable ();
 		Stack delegatorStack = new Stack ();
 		bool disposed;
 		bool toolbarUpdaterRunning;
 		bool enableToolbarUpdate;
+		int guiLock;
 		
 		Gtk.AccelGroup accelGroup;
 		
@@ -65,10 +67,7 @@ namespace MonoDevelop.Components.Commands
 		public void SetRootWindow (Gtk.Window root)
 		{
 			rootWidget = root;
-			if (rootWidget.IsRealized)
-				rootWidget.AddAccelGroup (AccelGroup);
-			else
-				rootWidget.Realized += new EventHandler (RootRealized);
+			rootWidget.AddAccelGroup (AccelGroup);
 		}
 		
 		public void Dispose ()
@@ -76,12 +75,32 @@ namespace MonoDevelop.Components.Commands
 			disposed = true;
 		}
 		
+		public void LockAll ()
+		{
+			guiLock++;
+			if (guiLock == 1) {
+				foreach (CommandToolbar toolbar in toolbars)
+					toolbar.SetEnabled (false);
+			}
+		}
+		
+		public void UnlockAll ()
+		{
+			if (guiLock == 1) {
+				foreach (CommandToolbar toolbar in toolbars)
+					toolbar.SetEnabled (true);
+			}
+			
+			if (guiLock > 0)
+				guiLock--;
+		}
+		
 		public bool EnableIdleUpdate {
 			get { return enableToolbarUpdate; }
 			set {
 				if (enableToolbarUpdate != value) {
 					if (value) {
-						if (toolbars.Count > 0) {
+						if (toolbars.Count > 0 || visitors.Count > 0) {
 							if (!toolbarUpdaterRunning) {
 								GLib.Timeout.Add (500, new GLib.TimeoutHandler (UpdateStatus));
 								toolbarUpdaterRunning = true;
@@ -122,6 +141,20 @@ namespace MonoDevelop.Components.Commands
 			}
 		}
 		
+		public void UnregisterCommand (Command cmd)
+		{
+			if (cmd.AccelKey != null && cmd.AccelKey != "") {
+				// Unregister overloaded key bindings
+				ArrayList list = (ArrayList) overloadedAccelCommands [cmd.AccelKey];
+				if (list != null) {
+					list.Remove (cmd.Id);
+					if (list.Count < 2)
+						overloadedAccelCommands.Remove (cmd.AccelKey);
+				}
+			}
+			cmds.Remove (cmd.Id);
+		}
+		
 		public void RegisterGlobalHandler (object handler)
 		{
 			if (!globalHandlers.Contains (handler))
@@ -133,25 +166,28 @@ namespace MonoDevelop.Components.Commands
 			globalHandlers.Remove (handler);
 		}
 		
-		public Command GetCommand (object cmdId)
+		public void RegisterCommandTargetVisitor (ICommandTargetVisitor visitor)
 		{
-			Command cmd = cmds [cmdId] as Command;
-			if (cmd == null)
-				throw new InvalidOperationException ("Invalid command id: " + cmdId);
-			return cmd;
+			visitors.Add (visitor);
+			if (enableToolbarUpdate && !toolbarUpdaterRunning) {
+				GLib.Timeout.Add (500, new GLib.TimeoutHandler (UpdateStatus));
+				toolbarUpdaterRunning = true;
+			}
 		}
 		
-		internal Command FindCommand (object cmdId)
+		public void UnregisterCommandTargetVisitor (ICommandTargetVisitor visitor)
+		{
+			visitors.Remove (visitor);
+		}
+		
+		public Command GetCommand (object cmdId)
 		{
 			return cmds [cmdId] as Command;
 		}
 		
 		public ActionCommand GetActionCommand (object cmdId)
 		{
-			ActionCommand cmd = cmds [cmdId] as ActionCommand;
-			if (cmd == null)
-				throw new InvalidOperationException ("Invalid action command id: " + cmdId);
-			return cmd;
+			return cmds [cmdId] as ActionCommand;
 		}
 		
 		public Gtk.MenuBar CreateMenuBar (string name, CommandEntrySet entrySet)
@@ -204,6 +240,11 @@ namespace MonoDevelop.Components.Commands
 		public void ShowContextMenu (Gtk.Menu menu)
 		{
 			menu.Popup (null, null, null, 0, Gtk.Global.CurrentEventTime);
+			menu.Hidden += delegate {
+				// Needed to make sure the menu entries are destroyed and
+				// can't handle accelerators
+				menu.Destroy ();
+			};
 		}
 		
 		public Gtk.Toolbar CreateToolbar (string id, CommandEntrySet entrySet)
@@ -226,9 +267,14 @@ namespace MonoDevelop.Components.Commands
 
 		public bool DispatchCommand (object commandId, object dataItem, object initialTarget)
 		{
+			if (guiLock > 0)
+				return false;
+
 			ActionCommand cmd = null;
 			try {
 				cmd = GetActionCommand (commandId);
+				if (cmd == null)
+					return false;
 				
 				int globalPos = -1;
 				object cmdTarget = initialTarget != null ? initialTarget : GetFirstCommandTarget (out globalPos);
@@ -238,36 +284,44 @@ namespace MonoDevelop.Components.Commands
 				{
 					HandlerTypeInfo typeInfo = GetTypeHandlerInfo (cmdTarget);
 					
+					bool bypass = false;
+					
 					CommandUpdaterInfo cui = typeInfo.GetCommandUpdater (commandId);
 					if (cui != null) {
 						if (cmd.CommandArray) {
 							// Make sure that the option is still active
 							CommandArrayInfo ainfo = new CommandArrayInfo (info);
 							cui.Run (cmdTarget, ainfo);
-							bool found = false;
-							foreach (CommandInfo ci in ainfo) {
-								if (Object.Equals (dataItem, ci.DataItem)) {
-									found = true;
-									break;
+							if (!ainfo.Bypass) {
+								bool found = false;
+								foreach (CommandInfo ci in ainfo) {
+									if (Object.Equals (dataItem, ci.DataItem)) {
+										found = true;
+										break;
+									}
 								}
-							}
-							if (!found) return false;
+								if (!found) return false;
+							} else
+								bypass = true;
 						} else {
+							info.Bypass = false;
 							cui.Run (cmdTarget, info);
-							if (!info.Enabled || !info.Visible) return false;
+							bypass = info.Bypass;
+							if (!bypass && (!info.Enabled || !info.Visible)) return false;
 						}
 					}
 					
-					CommandHandlerInfo chi = typeInfo.GetCommandHandler (commandId);
-					if (chi != null) {
-						if (cmd.CommandArray)
-							chi.Run (cmdTarget, dataItem);
-						else
-							chi.Run (cmdTarget);
-						UpdateToolbars ();
-						return true;
+					if (!bypass) {
+						CommandHandlerInfo chi = typeInfo.GetCommandHandler (commandId);
+						if (chi != null) {
+							if (cmd.CommandArray)
+								chi.Run (cmdTarget, dataItem);
+							else
+								chi.Run (cmdTarget);
+							UpdateToolbars ();
+							return true;
+						}
 					}
-					
 					cmdTarget = GetNextCommandTarget (cmdTarget, ref globalPos);
 				}
 	
@@ -284,6 +338,9 @@ namespace MonoDevelop.Components.Commands
 		internal CommandInfo GetCommandInfo (object commandId, object initialTarget)
 		{
 			ActionCommand cmd = GetActionCommand (commandId);
+			if (cmd == null)
+				throw new InvalidOperationException ("Invalid action command id: " + commandId);
+			
 			CommandInfo info = new CommandInfo (cmd);
 			
 			try {
@@ -295,20 +352,31 @@ namespace MonoDevelop.Components.Commands
 					HandlerTypeInfo typeInfo = GetTypeHandlerInfo (cmdTarget);
 					CommandUpdaterInfo cui = typeInfo.GetCommandUpdater (commandId);
 					
+					bool bypass = false;
 					if (cui != null) {
 						if (cmd.CommandArray) {
 							info.ArrayInfo = new CommandArrayInfo (info);
 							cui.Run (cmdTarget, info.ArrayInfo);
-							return info;
+							if (!info.ArrayInfo.Bypass) {
+								if (guiLock > 0)
+									info.Enabled = false;
+								return info;
+							}
 						}
 						else {
+							info.Bypass = false;
 							cui.Run (cmdTarget, info);
-							return info;
+							if (!info.Bypass) {
+								if (guiLock > 0)
+									info.Enabled = false;
+								return info;
+							}
 						}
+						bypass = true;
 					}
 					
-					if (typeInfo.GetCommandHandler (commandId) != null) {
-						info.Enabled = true;
+					if (!bypass && typeInfo.GetCommandHandler (commandId) != null) {
+						info.Enabled = guiLock == 0;
 						info.Visible = true;
 						return info;
 					}
@@ -326,9 +394,28 @@ namespace MonoDevelop.Components.Commands
 				info.Enabled = false;
 				info.Visible = true;
 			}
+			if (guiLock > 0)
+				info.Enabled = false;
 			return info;
 		}
 		
+		public object VisitCommandTargets (ICommandTargetVisitor visitor, object initialTarget)
+		{
+			int globalPos = -1;
+			object cmdTarget = initialTarget != null ? initialTarget : GetFirstCommandTarget (out globalPos);
+			
+			while (cmdTarget != null)
+			{
+				if (visitor.Visit (cmdTarget))
+					return cmdTarget;
+
+				cmdTarget = GetNextCommandTarget (cmdTarget, ref globalPos);
+			}
+			
+			visitor.Visit (null);
+			return null;
+		}
+		
 		internal ArrayList FindAlternateAccelCommands (object commandId)
 		{
 			return (ArrayList) overloadedAccelCommands [commandId];
@@ -346,6 +433,9 @@ namespace MonoDevelop.Components.Commands
 			// but checking all possible commands would be too slow.
 			
 			Command cmd = GetCommand (commandId);
+			if (cmd == null)
+				return false;
+			
 			string accel = cmd.AccelKey;
 			if (accel == null || accel == "")
 				return DispatchCommand (commandId, dataItem, initialTarget);
@@ -401,11 +491,6 @@ namespace MonoDevelop.Components.Commands
 			return path;
 		}
 		
-		void RootRealized (object sender, EventArgs e)
-		{
-			rootWidget.AddAccelGroup (AccelGroup);
-		}
-		
 		internal Gtk.AccelGroup AccelGroup {
 			get {
 				if (accelGroup == null) {
@@ -498,7 +583,7 @@ namespace MonoDevelop.Components.Commands
 			Gtk.Window[] wins = Gtk.Window.ListToplevels ();
 			
 			foreach (Gtk.Window w in wins) {
-				if (w.IsActive && w.Visible && w.Type == Gtk.WindowType.Toplevel) {
+				if (w.IsActive && w.Visible && w.Type == Gtk.WindowType.Toplevel && !(w is Gtk.Dialog)) {
 					win = w;
 					break;
 				}
@@ -516,16 +601,6 @@ namespace MonoDevelop.Components.Commands
 				return widget;
 			}
 			return win;
-			
-/*			Gtk.Widget widget = win;
-			while (widget is Gtk.Container) {
-				Gtk.Widget child = ((Gtk.Container)widget).FocusChild;
-				if (child != null)
-					widget = child;
-				else
-					break;
-			}
-			return widget;*/
 		}
 		
 		bool UpdateStatus ()
@@ -545,6 +620,8 @@ namespace MonoDevelop.Components.Commands
 				GLib.Timeout.Add (500, new GLib.TimeoutHandler (UpdateStatus));
 				toolbarUpdaterRunning = true;
 			}
+			if (guiLock > 0)
+				toolbar.SetEnabled (false);
 		}
 		
 		void UpdateToolbars ()
@@ -554,6 +631,9 @@ namespace MonoDevelop.Components.Commands
 				if (toolbar.Visible)
 					toolbar.Update (activeWidget);
 			}
+			foreach (ICommandTargetVisitor v in visitors)
+				VisitCommandTargets (v, null);
+				
 		}
 		
 		public void ReportError (object commandId, string message, Exception ex)
