@@ -12,23 +12,10 @@ namespace MonoDevelop.Components.HtmlControl
 		
 		void GoHome();
 		
-		void GoSearch();
-		
 		void Navigate(string Url, ref object Flags, ref object targetFrame, ref object postData, ref object headers);
 		
 		void Refresh();
 		
-		void Refresh2();
-		
 		void Stop();
-		
-		void GetApplication();
-		
-		void GetParent();
-		
-		void GetContainer();
-		
-		[return: MarshalAs(UnmanagedType.Interface)]
-		IHTMLDocument2 GetDocument();
 	}
 }
