@@ -32,24 +32,20 @@
 using System;
 using System.Collections;
 using System.ComponentModel;
-using MonoDevelop.Core.AddIns;
+using Mono.Addins;
 
 namespace MonoDevelop.Core.Gui.Codons
 {
-	[Description ("A stock icon. It is possible to register several icons with the same 'id' and different sizes.")]
-	[CodonNameAttribute ("StockIcon")]
-	internal class StockIconCodon : AbstractCodon
+	[ExtensionNode (Description="A stock icon. It is possible to register several icons with the same 'id' and different sizes.")]
+	internal class StockIconCodon : ExtensionNode
 	{
-		[Description ("Id of the stock icon.")]
-		[XmlMemberAttribute ("stockid", IsRequired = true)]
+		[NodeAttribute ("stockid", true, "Id of the stock icon.")]
 		string stockid;
 		
-		[Description ("Size of the icon.")]
-		[XmlMemberAttribute ("size")]
+		[NodeAttribute ("size", "Size of the icon.")]
 		Gtk.IconSize size = Gtk.IconSize.Invalid;
 		
-		[Description ("Name of the resource where the icon is stored.")]
-		[XmlMemberAttribute ("resource")]
+		[NodeAttribute ("resource", "Name of the resource where the icon is stored.")]
 		string resource;
 		
 		public string StockId {
@@ -63,10 +59,5 @@ namespace MonoDevelop.Core.Gui.Codons
 		public string Resource {
 			get { return resource; }
 		}
-		
-		public override object BuildItem (object owner, ArrayList subItems, ConditionCollection conditions)
-		{
-			return this;
-		}
 	}
 }
