@@ -27,7 +27,8 @@ namespace MonoDevelop.Ide.Commands
 		FindNextSelection,
 		FindPreviousSelection,
 		FindBox,
-		ReplaceInFiles
+		ReplaceInFiles,
+		GotoType
 	}
 
 	internal class FindInFilesHandler : CommandHandler
@@ -46,4 +47,16 @@ namespace MonoDevelop.Ide.Commands
 		}
 	}
 	
+	internal class GotoTypeHandler : CommandHandler
+	{
+	    protected override void Run()
+		{
+			OpenFileInSolutionDialog.Show (false);
+	    }
+	    
+	    protected override void Update(CommandInfo info)
+	    {
+	    	info.Enabled = (IdeApp.ProjectOperations.CurrentOpenCombine != null);
+	    }
+	}
 }
