@@ -57,7 +57,7 @@ namespace MonoDevelop.Ide.Gui.BrowserDisplayBinding
 				return false;
 			string mimetype = Gnome.Vfs.MimeType.GetMimeTypeForUri (filename);
 			if (mimetype == "text/html")
-				return (parent is MonoDevelop.Ide.Gui.Content.ITextBuffer);
+				return parent.GetContent (typeof(MonoDevelop.Ide.Gui.Content.ITextBuffer)) != null;
 			return false;
 		}
 
