@@ -91,6 +91,7 @@ namespace MonoDevelop.Ide.Gui.Pads.ProjectPad
 				} else
 					dir = file.Name;
 				
+				// add the directory if it isn't already present
 				if (dir.StartsWith (folderPrefix)) {
 					int i = dir.IndexOf (Path.DirectorySeparatorChar, folderPrefix.Length);
 					if (i != -1) dir = dir.Substring (0,i);
@@ -196,14 +197,14 @@ namespace MonoDevelop.Ide.Gui.Pads.ProjectPad
 				
 				if (operation == DragOperation.Move) {
 					if (filesToSave.Count == 1)
-						question = string.Format (GettextCatalog.GetString ("Do you want to save the file '{0}' before the move operation?"), sb.ToString ());
+						question = GettextCatalog.GetString ("Do you want to save the file '{0}' before the move operation?", sb.ToString ());
 					else
-						question = string.Format (GettextCatalog.GetString ("Do you want to save the following files before the move operation?\n\n{0}"), sb.ToString ());
+						question = GettextCatalog.GetString ("Do you want to save the following files before the move operation?\n\n{0}", sb.ToString ());
 				} else {
 					if (filesToSave.Count == 1)
-						question = string.Format (GettextCatalog.GetString ("Do you want to save the file '{0}' before the copy operation?"), sb.ToString ());
+						question = GettextCatalog.GetString ("Do you want to save the file '{0}' before the copy operation?", sb.ToString ());
 					else
-						question = string.Format (GettextCatalog.GetString ("Do you want to save the following files before the copy operation?\n\n{0}"), sb.ToString ());
+						question = GettextCatalog.GetString ("Do you want to save the following files before the copy operation?\n\n{0}", sb.ToString ());
 				}
 				
 				switch (Services.MessageService.AskQuestionWithCancel (question)) {
@@ -222,7 +223,7 @@ namespace MonoDevelop.Ide.Gui.Pads.ProjectPad
 				}
 			}
 			
-			using (IProgressMonitor monitor = IdeApp.Workbench.ProgressMonitors.GetStatusProgressMonitor (GettextCatalog.GetString("Copying files ..."), MonoDevelop.Core.Gui.Stock.CopyIcon, true))
+			using (IProgressMonitor monitor = IdeApp.Workbench.ProgressMonitors.GetStatusProgressMonitor (GettextCatalog.GetString("Copying files..."), MonoDevelop.Core.Gui.Stock.CopyIcon, true))
 			{
 				bool move = operation == DragOperation.Move;
 				IdeApp.ProjectOperations.TransferFiles (monitor, sourceProject, source, targetProject, targetPath, move, false);
@@ -235,106 +236,30 @@ namespace MonoDevelop.Ide.Gui.Pads.ProjectPad
 		{
 			Project project = CurrentNode.GetParentDataItem (typeof(Project), true) as Project;
 			
-			using (FileSelector fdiag  = new FileSelector (GettextCatalog.GetString ("Add files"))) {
-				fdiag.SetCurrentFolder (GetFolderPath (CurrentNode.DataItem));
-				fdiag.SelectMultiple = true;
-				
-				IProgressMonitor monitor = null;
-				int result = fdiag.Run ();
-				try {
-					if (result != (int) ResponseType.Ok)
-						return;
-					
-					int action = -1;
-					
-					if (fdiag.Filenames.Length > 10) {
-						monitor = new MonoDevelop.Core.Gui.ProgressMonitoring.MessageDialogProgressMonitor (true);
-						monitor.BeginTask (GettextCatalog.GetString("Adding files..."), fdiag.Filenames.Length);
-					}
-					
-					foreach (string file in fdiag.Filenames) {
-						if (monitor != null)
-							monitor.Log.WriteLine (file);
-						if (file.StartsWith (project.BaseDirectory)) {
-							MoveCopyFile (project, CurrentNode, file, true, true);
-						} else {
-							using (MessageDialog md = new MessageDialog (
-								 IdeApp.Workbench.RootWindow,
-								 DialogFlags.Modal | DialogFlags.DestroyWithParent,
-								 MessageType.Question, ButtonsType.None,
-								 String.Format (GettextCatalog.GetString ("{0} is outside the project directory, what should I do?"), file)))
-							{
-								CheckButton remember = null;
-								if (fdiag.Filenames.Length > 1) {
-									remember = new CheckButton (GettextCatalog.GetString ("Use the same action for all selected files."));
-									md.VBox.PackStart (remember, false, false, 0);
-								}
-								md.AddButton (Gtk.Stock.Copy, 1);
-								md.AddButton (GettextCatalog.GetString ("_Move"), 2);
-								md.AddButton (Gtk.Stock.Cancel, ResponseType.Cancel);
-								md.VBox.ShowAll ();
-								
-								int ret = -1;
-								if (action < 0) {
-									ret = md.Run ();
-									md.Hide ();
-									if (ret < 0) {
-										IdeApp.ProjectOperations.SaveCombine();
-										return;
-									}
-									if (remember != null && remember.Active) action = ret;
-								} else {
-									ret = action;
-								}
-
-								try {
-									MoveCopyFile (project, CurrentNode, file, ret == 2, false);
-								}
-								catch (Exception ex) {
-									Services.MessageService.ShowError (ex, GettextCatalog.GetString ("An error occurred while attempt to move/copy that file. Please check your permissions."));
-								}
-							}
-						}
-						if (monitor != null)
-							monitor.Step (1);
-					}
-					IdeApp.ProjectOperations.SaveCombine();
-				} finally {
-					fdiag.Hide ();
-					if (monitor != null)
-						monitor.Dispose ();
-				}
+			FileSelector fdiag  = new FileSelector (GettextCatalog.GetString ("Add files"));
+			fdiag.SetCurrentFolder (GetFolderPath (CurrentNode.DataItem));
+			fdiag.SelectMultiple = true;
+			
+			int result;
+			string[] files;
+			IProgressMonitor monitor = null;
+			
+			try {
+				result = fdiag.Run ();
+				files = fdiag.Filenames;
+				if (result != (int) ResponseType.Ok)
+					return;
+			} finally {
+				fdiag.Destroy ();
 			}
+			
+			ProjectFolder folder = CurrentNode.GetParentDataItem (typeof(ProjectFolder), true) as ProjectFolder;
+			string baseDirectory = folder != null ? folder.Path : project.BaseDirectory;
+			
+			IdeApp.ProjectOperations.AddFilesToProject (project, files, baseDirectory);
+			IdeApp.ProjectOperations.SaveProject (project);
 		}
 		
-		static void MoveCopyFile (Project project, ITreeNavigator nav, string filename, bool move, bool alreadyInPlace)
-		{
-			if (Runtime.FileUtilityService.IsDirectory (filename))
-			    return;
-
-			ProjectFolder folder = nav.GetParentDataItem (typeof(ProjectFolder), true) as ProjectFolder;
-			
-			string name = System.IO.Path.GetFileName (filename);
-			string baseDirectory = folder != null ? folder.Path : project.BaseDirectory;
-			string newfilename = alreadyInPlace ? filename : Path.Combine (baseDirectory, name);
-
-			if (filename != newfilename) {
-				if (File.Exists (newfilename)) {
-					if (!Services.MessageService.AskQuestion (string.Format (GettextCatalog.GetString ("The file '{0}' already exists. Do you want to replace it?"), newfilename), "MonoDevelop"))
-						return;
-				}
-				File.Copy (filename, newfilename, true);
-				if (move)
-					Services.FileService.RemoveFile (filename);
-			}
-			
-			if (project.IsCompileable (newfilename)) {
-				project.AddFile (newfilename, BuildAction.Compile);
-			} else {
-				project.AddFile (newfilename, BuildAction.Nothing);
-			}
-		}		
-
 		[CommandHandler (ProjectCommands.AddNewFiles)]
 		public void AddNewFileToProject()
 		{
