@@ -39,6 +39,8 @@ using MonoDevelop.Core.Gui;
 using MonoDevelop.Ide.Codons;
 using MonoDevelop.Ide.Gui.Content;
 using MonoDevelop.Core.Gui.Utils;
+using MonoDevelop.Core.Gui.Dialogs;
+using Mono.Addins;
 
 namespace MonoDevelop.Ide.Gui
 {
@@ -55,6 +57,7 @@ namespace MonoDevelop.Ide.Gui
 		IStatusBarService statusBarService;
 		
 		public event EventHandler ActiveDocumentChanged;
+		public event EventHandler LayoutChanged;
 		
 		internal void Initialize (IProgressMonitor monitor)
 		{
@@ -77,8 +80,8 @@ namespace MonoDevelop.Ide.Gui
 					Services.DebuggingService.PausedEvent += (EventHandler) Services.DispatchService.GuiDispatch (new EventHandler (OnDebuggerPaused));
 				}
 				
-				Services.FileService.FileRemoved += (FileEventHandler) Services.DispatchService.GuiDispatch (new FileEventHandler (IdeApp.Workbench.RecentOpen.FileRemoved));
-				Services.FileService.FileRenamed += (FileEventHandler) Services.DispatchService.GuiDispatch (new FileEventHandler (IdeApp.Workbench.RecentOpen.FileRenamed));
+				Runtime.FileService.FileRemoved += (FileEventHandler) Services.DispatchService.GuiDispatch (new FileEventHandler (IdeApp.Workbench.RecentOpen.FileRemoved));
+				Runtime.FileService.FileRenamed += (FileEventHandler) Services.DispatchService.GuiDispatch (new FileEventHandler (IdeApp.Workbench.RecentOpen.FileRenamed));
 				
 				pads = null;	// Make sure we get an up to date pad list.
 				monitor.Step (1);
@@ -108,6 +111,11 @@ namespace MonoDevelop.Ide.Gui
 			workbench.SetMemento ((IXmlConvertable)Runtime.Properties.GetProperty (workbenchMemento, workbench.CreateMemento()));
 			RootWindow.Visible = true;
 			workbench.Context = WorkbenchContext.Edit;
+			
+			// now we have an layout set notify it
+			if (LayoutChanged != null)
+				LayoutChanged (this, EventArgs.Empty);
+
 			workbench.RedrawAllComponents ();
 			RootWindow.Present ();
 		}
@@ -137,7 +145,7 @@ namespace MonoDevelop.Ide.Gui
 			get {
 				if (pads == null) {
 					pads = new PadCollection ();
-					foreach (IPadContent pc in workbench.PadContentCollection)
+					foreach (PadCodon pc in workbench.ActivePadContentCollection)
 						WrapPad (pc);
 				}
 				return pads;
@@ -167,7 +175,11 @@ namespace MonoDevelop.Ide.Gui
 			get { return workbench.WorkbenchLayout != null ? workbench.WorkbenchLayout.CurrentLayout : ""; }
 			set {
 				if (value != workbench.WorkbenchLayout.CurrentLayout)
-					workbench.WorkbenchLayout.CurrentLayout = value; 
+				{
+					workbench.WorkbenchLayout.CurrentLayout = value;
+					if (LayoutChanged != null)
+						LayoutChanged (this, EventArgs.Empty);
+				}
 			}
 		}
 
@@ -196,6 +208,16 @@ namespace MonoDevelop.Ide.Gui
 			workbench.WorkbenchLayout.DeleteLayout (name);
 		}
 		
+		public void LockGui ()
+		{
+			IdeApp.CommandService.CommandManager.LockAll ();
+		}
+		
+		public void UnlockGui ()
+		{
+			IdeApp.CommandService.CommandManager.UnlockAll ();
+		}
+		
 		public void SaveAll ()
 		{
 			foreach (Document doc in Documents)
@@ -218,12 +240,17 @@ namespace MonoDevelop.Ide.Gui
 				ActiveDocument.Close ();
 		}
 
-		public Pad ShowPad (IPadContent content)
+		public Pad ShowPad (PadCodon content)
 		{
 			workbench.ShowPad (content);
 			return WrapPad (content);
 		}
 		
+		public Pad ShowPad (IPadContent padContent, string id, string label, string defaultPlacement, string icon)
+		{
+			return ShowPad (new PadCodon (padContent, id, label, defaultPlacement, icon));
+		}
+		
 		public FileViewer[] GetFileViewers (string fileName)
 		{
 			ArrayList list = new ArrayList ();
@@ -295,15 +322,17 @@ namespace MonoDevelop.Ide.Gui
 						doc.Select ();
 						doc.Window.SwitchView (vcIndex);
 					}
-					if (line != -1 && vcFound is IPositionable) {
-						((IPositionable) vcFound).JumpTo (line, column != -1 ? column : 0);
+					
+					IPositionable ipos = (IPositionable) vcFound.GetContent (typeof(IPositionable));
+					if (line >= 1 && ipos != null) {
+						ipos.JumpTo (line, column >= 1 ? column : 1);
 					}
 					
 					return doc;
 				}
 			}
-
-			IProgressMonitor pm = ProgressMonitors.GetStatusProgressMonitor (string.Format (GettextCatalog.GetString ("Opening {0}"), fileName), Stock.OpenFileIcon, true);
+			
+			IProgressMonitor pm = ProgressMonitors.GetStatusProgressMonitor (GettextCatalog.GetString ("Opening {0}", fileName), Stock.OpenFileIcon, true);
 			FileInformation openFileInfo = new FileInformation();
 			openFileInfo.ProgressMonitor = pm;
 			openFileInfo.FileName = fileName;
@@ -365,6 +394,26 @@ namespace MonoDevelop.Ide.Gui
 			return WrapDocument (newContent.WorkbenchWindow);
 		}
 		
+		public void ShowGlobalPreferencesDialog (Gtk.Window parentWindow)
+		{
+			ShowGlobalPreferencesDialog (null, null);
+		}
+		
+		public void ShowGlobalPreferencesDialog (Gtk.Window parentWindow, string panelId)
+		{
+			if (parentWindow == null)
+				parentWindow = IdeApp.Workbench.RootWindow;
+
+			TreeViewOptions ops = new TreeViewOptions (
+				parentWindow,
+				(IProperties)Runtime.Properties.GetProperty("MonoDevelop.TextEditor.Document.Document.DefaultDocumentAggregatorProperties", new DefaultProperties()),
+				AddinManager.GetExtensionNode ("/SharpDevelop/Dialogs/OptionsDialog"));
+
+			if (panelId != null)
+				ops.SelectPanel (panelId);
+			ops.Run ();
+		}
+		
 		void OnDocumentChanged (object s, EventArgs a)
 		{
 			if (ActiveDocumentChanged != null)
@@ -380,10 +429,13 @@ namespace MonoDevelop.Ide.Gui
 			window.Closing += new WorkbenchWindowEventHandler (OnWindowClosing);
 			window.Closed += new EventHandler (OnWindowClosed);
 			Documents.Add (doc);
+			
+			doc.OnDocumentAttached ();
+			
 			return doc;
 		}
 		
-		Pad WrapPad (IPadContent padContent)
+		Pad WrapPad (PadCodon padContent)
 		{
 			Pad pad = new Pad (workbench, padContent);
 			Pads.Add (pad);
@@ -407,7 +459,7 @@ namespace MonoDevelop.Ide.Gui
 						while (true) {
 							FindDocument (window).Save ();
 							if (window.ViewContent.IsDirty) {
-								if (Services.MessageService.AskQuestion(GettextCatalog.GetString ("Do you really want to discard your changes ?"))) {
+								if (Services.MessageService.AskQuestion(GettextCatalog.GetString ("Do you really want to discard your changes?"))) {
 									break;
 								}
 							} else {
@@ -416,7 +468,10 @@ namespace MonoDevelop.Ide.Gui
 						}
 						
 					} else {
-						window.ViewContent.Save (window.ViewContent.ContentName);
+						if (window.ViewContent.IsFile)
+							window.ViewContent.Save (window.ViewContent.ContentName);
+						else
+							window.ViewContent.Save ();
 					}
 				}
 			}
@@ -445,21 +500,15 @@ namespace MonoDevelop.Ide.Gui
 	
 				string origName = fileName;
 	
-				if (fileName.StartsWith ("file://")) {
-					fileName = fileName.Substring (7);
-					// FIXME: this is lame
-					fileName = fileName.Replace ("%3f", "?");
-					fileName = fileName.Replace ("%3F", "?");
-					fileName = fileName.Replace ("%23", "#");
-					fileName = fileName.Replace ("%25", "%");
-				}
+				if (fileName.StartsWith ("file://"))
+					fileName = new Uri(fileName).LocalPath;
 	
 				if (!fileName.StartsWith ("http://"))
 					fileName = System.IO.Path.GetFullPath (fileName);
 				
-				//Debug.Assert(Runtime.FileUtilityService.IsValidFileName(fileName));
-				if (Runtime.FileUtilityService.IsDirectory (fileName)) {
-					monitor.ReportError (string.Format (GettextCatalog.GetString ("{0} is a directory"), fileName), null);
+				//Debug.Assert(Runtime.FileService.IsValidFileName(fileName));
+				if (Runtime.FileService.IsDirectory (fileName)) {
+					monitor.ReportError (GettextCatalog.GetString ("{0} is a directory", fileName), null);
 					return;
 				}
 				// test, if file fileName exists
@@ -475,7 +524,7 @@ namespace MonoDevelop.Ide.Gui
 						}
 					}
 					if (!File.Exists (fileName)) {
-						monitor.ReportError (string.Format (GettextCatalog.GetString ("File not found: {0}"), fileName), null);
+						monitor.ReportError (GettextCatalog.GetString ("File not found: {0}", fileName), null);
 						return;
 					}
 				}
@@ -484,8 +533,9 @@ namespace MonoDevelop.Ide.Gui
 					if (doc.FileName == fileName) {
 						if (oFileInfo.BringToFront) {
 							doc.Select ();
-							if (oFileInfo.Line != -1 && doc.Window.ViewContent is IPositionable) {
-								((IPositionable)doc.Window.ViewContent).JumpTo (oFileInfo.Line, oFileInfo.Column != -1 ? oFileInfo.Column : 0);
+							IPositionable ipos = doc.GetContent <IPositionable> ();
+							if (oFileInfo.Line != -1 && ipos != null) {
+								ipos.JumpTo (oFileInfo.Line, oFileInfo.Column != -1 ? oFileInfo.Column : 0);
 							}
 						}
 						oFileInfo.NewContent = doc.Window.ViewContent;
@@ -619,8 +669,9 @@ namespace MonoDevelop.Ide.Gui
 					return;
 				}
 
-				if (fileInfo.Encoding != null && newContent is IEncodedTextContent)
-					((IEncodedTextContent)newContent).Load (fileName, fileInfo.Encoding);
+				IEncodedTextContent etc = (IEncodedTextContent) newContent.GetContent (typeof(IEncodedTextContent));
+				if (fileInfo.Encoding != null && etc != null)
+					etc.Load (fileName, fileInfo.Encoding);
 				else
 					newContent.Load (fileName);
 			} catch (Exception ex) {
@@ -634,7 +685,8 @@ namespace MonoDevelop.Ide.Gui
 			workbench.ShowView (newContent, fileInfo.BringToFront);
 			Services.DisplayBindings.AttachSubWindows(newContent.WorkbenchWindow);
 			
-			if (fileInfo.Line != -1 && newContent is IPositionable) {
+			IPositionable ipos = (IPositionable) newContent.GetContent (typeof(IPositionable));
+			if (fileInfo.Line != -1 && ipos != null) {
 				GLib.Timeout.Add (10, new GLib.TimeoutHandler (JumpToLine));
 			}
 			fileInfo.NewContent = newContent;
@@ -642,7 +694,8 @@ namespace MonoDevelop.Ide.Gui
 		
 		public bool JumpToLine ()
 		{
-			((IPositionable)newContent).JumpTo (Math.Max(1, fileInfo.Line), Math.Max(1, fileInfo.Column));
+			IPositionable ipos = (IPositionable) newContent.GetContent (typeof(IPositionable));
+			ipos.JumpTo (Math.Max(1, fileInfo.Line), Math.Max(1, fileInfo.Column));
 			return false;
 		}
 	}
