@@ -27,8 +27,6 @@ namespace MonoDevelop.Projects.Gui.Completion
 		{
 			get;
 		}
-		
-		void InsertAction(ICompletionWidget widget);
 	}
 	
 	public interface ICompletionDataWithMarkup : ICompletionData
@@ -37,4 +35,9 @@ namespace MonoDevelop.Projects.Gui.Completion
 			get;
 		}
 	}
+	
+	public interface IActionCompletionData : ICompletionData
+	{
+		void InsertAction (ICompletionWidget widget, ICodeCompletionContext context);
+	}
 }
