@@ -27,10 +27,11 @@
 //
 
 using System;
+using System.IO;
 using System.CodeDom;
 using System.CodeDom.Compiler;
 using Microsoft.CSharp;
-using System.IO;
+
 using MonoDevelop.Projects.Parser;
 using MonoDevelop.Projects.Text;
 using MonoDevelop.Projects.CodeGeneration;
@@ -56,7 +57,7 @@ namespace MonoDevelop.Projects.CodeGeneration
 			StreamWriter sw = new StreamWriter (file);
 			
 			ICodeGenerator gen = GetGenerator ();
-			gen.GenerateCodeFromCompileUnit (unit, sw, GetOpions ());
+			gen.GenerateCodeFromCompileUnit (unit, sw, GetOptions (false));
 			
 			sw.Close ();
 			
@@ -97,10 +98,95 @@ namespace MonoDevelop.Projects.CodeGeneration
 			return FindGeneratedMember (ctx, buffer, cls, member);
 		}
 		
+		protected CodeTypeReference ReturnTypeToDom (IReturnType rtype)
+		{
+			CodeTypeReference [] argTypes = null;
+			CodeTypeReference typeRef = null;
+			int i;
+			
+			ReturnTypeList genericArgs = rtype.GenericArguments;
+			if (genericArgs != null && genericArgs.Count > 0) {
+				argTypes = new CodeTypeReference [genericArgs.Count];
+				for (i = 0; i < genericArgs.Count; i++)
+					argTypes[i] = ReturnTypeToDom (genericArgs[i]);
+			}
+			
+			if (argTypes != null)
+				typeRef = new CodeTypeReference (rtype.FullyQualifiedName, argTypes);
+			else
+				typeRef = new CodeTypeReference (rtype.FullyQualifiedName);
+			
+			if (rtype.ArrayCount == 0)
+				return typeRef;
+			
+			int [] dim = rtype.ArrayDimensions;
+			for (i = 0; i < dim.Length; i++)
+				typeRef = new CodeTypeReference (typeRef, dim[i]);
+			
+			return typeRef;
+		}
+		
+		public virtual IMember ImplementMember (RefactorerContext ctx, IClass cls, string prefix, IMember member)
+		{
+			CodeTypeMember m;
+			
+			if (member is IEvent) {
+				CodeMemberEvent mEvent = new CodeMemberEvent ();
+				m = (CodeTypeMember) mEvent;
+				
+				mEvent.Type = ReturnTypeToDom (member.ReturnType);
+			} else if (member is IMethod) {
+				CodeMemberMethod mMethod = new CodeMemberMethod ();
+				IMethod method = (IMethod) member;
+				m = (CodeMemberMethod) mMethod;
+				
+				mMethod.ReturnType = ReturnTypeToDom (member.ReturnType);
+				
+				foreach (IParameter param in method.Parameters) {
+					CodeParameterDeclarationExpression par;
+					par = new CodeParameterDeclarationExpression (ReturnTypeToDom (param.ReturnType), param.Name);
+					mMethod.Parameters.Add (par);
+				}
+			} else if (member is IProperty) {
+				CodeMemberProperty mProperty = new CodeMemberProperty ();
+				IProperty property = (IProperty) member;
+				m = (CodeMemberProperty) mProperty;
+				
+				mProperty.HasGet = property.CanGet;
+				mProperty.HasSet = property.CanSet;
+				
+				mProperty.Type = ReturnTypeToDom (member.ReturnType);
+			} else {
+				return null;
+			}
+			
+			if (prefix != null)
+				m.Name = prefix + member.Name;
+			else
+				m.Name = member.Name;
+			
+			m.Attributes = MemberAttributes.Public;
+			
+			return AddMember (ctx, cls, m);
+		}
+		
 		public virtual void RemoveMember (RefactorerContext ctx, IClass cls, IMember member)
 		{
-			IEditableTextFile buffer = ctx.GetFile (cls.Region.FileName);
-			IRegion reg = GetMemberBounds (member);
+			IEditableTextFile buffer = null;
+			int pos = -1;
+			
+			for (int i = 0; i < cls.Parts.Length; i++) {
+				if ((buffer = ctx.GetFile (cls.Parts[i].Region.FileName)) == null)
+					continue;
+				
+				if ((pos = GetMemberNamePosition (buffer, member)) != -1)
+					break;
+			}
+			
+			if (pos == -1)
+				return;
+			
+			IRegion reg = GetMemberBounds (buffer, member);
 			int sp = buffer.GetPositionFromLineColumn (reg.BeginLine, reg.BeginColumn);
 			int ep = buffer.GetPositionFromLineColumn (reg.EndLine, reg.EndColumn);
 			buffer.DeleteText (sp, ep - sp);
@@ -108,8 +194,21 @@ namespace MonoDevelop.Projects.CodeGeneration
 		
 		public virtual IMember ReplaceMember (RefactorerContext ctx, IClass cls, IMember oldMember, CodeTypeMember memberInfo)
 		{
-			IEditableTextFile buffer = ctx.GetFile (cls.Region.FileName);
-			IRegion reg = GetMemberBounds (oldMember);
+			IEditableTextFile buffer = null;
+			int pos = -1;
+			
+			for (int i = 0; i < cls.Parts.Length; i++) {
+				if ((buffer = ctx.GetFile (cls.Parts[i].Region.FileName)) == null)
+					continue;
+				
+				if ((pos = GetMemberNamePosition (buffer, oldMember)) != -1)
+					break;
+			}
+			
+			if (pos == -1)
+				return null;
+			
+			IRegion reg = GetMemberBounds (buffer, oldMember);
 			int sp = buffer.GetPositionFromLineColumn (reg.BeginLine, reg.BeginColumn);
 			int ep = buffer.GetPositionFromLineColumn (reg.EndLine, reg.EndColumn);
 			buffer.DeleteText (sp, ep - sp);
@@ -121,21 +220,32 @@ namespace MonoDevelop.Projects.CodeGeneration
 			buffer.InsertText (sp, code);
 			
 			return FindGeneratedMember (ctx, buffer, cls, memberInfo);
-				
 		}
 		
 		public virtual IMember RenameMember (RefactorerContext ctx, IClass cls, IMember member, string newName)
 		{
-			IEditableTextFile file = ctx.GetFile (cls.Region.FileName);
-			if (file == null)
-				return null;
-
-			int pos = GetMemberNamePosition (file, member);
+			IEditableTextFile file = null;
+			int pos = -1;
+			
+			for (int i = 0; i < cls.Parts.Length; i++) {
+				if ((file = ctx.GetFile (cls.Parts[i].Region.FileName)) == null)
+					continue;
+				
+				if ((pos = GetMemberNamePosition (file, member)) != -1)
+					break;
+			}
+			
 			if (pos == -1)
 				return null;
-
-			string txt = file.GetText (pos, pos + member.Name.Length);
-			if (txt != member.Name)
+			
+			string name;
+			if (member is IMethod && ((IMethod) member).IsConstructor)
+				name = cls.Name;
+			else
+				name = member.Name;
+			
+			string txt = file.GetText (pos, pos + name.Length);
+			if (txt != name)
 				return null;
 			
 			file.DeleteText (pos, txt.Length);
@@ -171,6 +281,77 @@ namespace MonoDevelop.Projects.CodeGeneration
 				return null;
 		}
 		
+		///
+		/// EncapsulateFieldImpGetSet:
+		///
+		/// Override this method for each language to fill-in the Get/SetStatements
+		///
+		protected virtual void EncapsulateFieldImpGetSet (RefactorerContext ctx, IClass cls, IField field, CodeMemberProperty  prop)
+		{
+			
+		}
+		
+		public virtual IMember EncapsulateField (RefactorerContext ctx, IClass cls, IField field, string propName)
+		{
+			// If the field isn't already private/protected/internal, we'll need to fix it to be
+			if (true || field.IsPublic || (!field.IsPrivate && !field.IsProtectedOrInternal)) {
+				IEditableTextFile file = null;
+				int pos = -1;
+				
+				// Find the file the field is contained in
+				for (int i = 0; i < cls.Parts.Length; i++) {
+					if ((file = ctx.GetFile (cls.Parts[i].Region.FileName)) == null)
+						continue;
+					
+					if ((pos = GetMemberNamePosition (file, field)) != -1)
+						break;
+				}
+				
+				if (pos != -1) {
+					// FIXME: need a way to get the CodeMemberField fieldInfo as a parsed object
+					// (so we don't lose initialization state nor custom attributes, etc).
+//					CodeMemberField fieldInfo = new CodeMemberField ();
+//					
+//					fieldInfo.Attributes = fieldInfo.Attributes & ~MemberAttributes.Public;
+//					fieldInfo.Attributes |= MemberAttributes.Private;
+//					
+//					RemoveMember (ctx, cls, field);
+//					AddMember (ctx, cls, fieldInfo);
+					
+					//int begin = file.GetPositionFromLineColumn (field.Region.BeginLine, field.Region.BeginColumn);
+					//int end = file.GetPositionFromLineColumn (field.Region.EndLine, field.Region.EndColumn);
+					//
+					//string snippet = file.GetText (begin, end);
+					//
+					//Console.WriteLine ("field declaration: {0}", snippet);
+					//
+					//IRegion region = GetMemberBounds (file, field);
+					//
+					//begin = file.GetPositionFromLineColumn (region.BeginLine, region.BeginColumn);
+					//end = file.GetPositionFromLineColumn (region.EndLine, region.EndColumn);
+					//
+					//snippet = file.GetText (begin, end);
+					//
+					//Console.WriteLine ("delete '{0}'", snippet);
+				}
+			}
+			
+			CodeMemberProperty prop = new CodeMemberProperty ();
+			prop.Name = propName;
+			
+			prop.Type = ReturnTypeToDom (field.ReturnType);
+			prop.Attributes = MemberAttributes.Public;
+			
+			prop.HasGet = true;
+			
+			// if the field was public before, then we provide a 'set', else we don't
+			prop.HasSet = (field.IsPublic || (!field.IsPrivate && !field.IsProtectedOrInternal));
+			
+			EncapsulateFieldImpGetSet (ctx, cls, field, prop);
+			
+			return AddMember (ctx, cls, prop);
+		}
+		
 
 		/// Method overridables ////////////////////////////
 		
@@ -187,7 +368,6 @@ namespace MonoDevelop.Projects.CodeGeneration
 
 		/// Field overridables ////////////////////////////
 		
-		
 		protected virtual IField RenameField (RefactorerContext ctx, IClass cls, IField field, string newName)
 		{
 			return null;
@@ -209,9 +389,8 @@ namespace MonoDevelop.Projects.CodeGeneration
 		protected virtual MemberReferenceCollection FindPropertyReferences (RefactorerContext ctx, string fileName, IClass cls, IProperty property)
 		{
 			return null;
-		}		
+		}
 
-		
 		/// Event overridables ////////////////////////////		
 		
 		protected virtual IEvent RenameEvent (RefactorerContext ctx, IClass cls, IEvent evnt, string newName)
@@ -222,7 +401,89 @@ namespace MonoDevelop.Projects.CodeGeneration
 		protected virtual MemberReferenceCollection FindEventReferences (RefactorerContext ctx, string fileName, IClass cls, IEvent evnt)
 		{
 			return null;
-		}		
+		}
+
+
+		/// LocalVariable overridables /////////////////////
+		
+		public virtual bool RenameVariable (RefactorerContext ctx, LocalVariable var, string newName)
+		{
+			IEditableTextFile file = ctx.GetFile (var.Region.FileName);
+			if (file == null)
+				return false;
+			
+			int pos = GetVariableNamePosition (file, var);
+			if (pos == -1)
+				return false;
+			
+			string txt = file.GetText (pos, pos + var.Name.Length);
+			if (txt != var.Name)
+				return false;
+			
+			file.DeleteText (pos, txt.Length);
+			file.InsertText (pos, newName);
+			
+			ctx.ParserContext.ParserDatabase.UpdateFile (file.Name, file.Text);
+			
+			return true;
+		}
+
+		public virtual MemberReferenceCollection FindVariableReferences (RefactorerContext ctx, string fileName, LocalVariable var)
+		{
+			return null;
+		}
+
+
+		/// Parameter overridables /////////////////////
+		
+		public virtual bool RenameParameter (RefactorerContext ctx, IParameter param, string newName)
+		{
+			IMember member = param.DeclaringMember;
+			IEditableTextFile file = null;
+			int pos = -1;
+			
+			// It'd be nice if we didn't have to worry about this being null
+			if (member.Region.FileName != null) {
+				if ((file = ctx.GetFile (member.Region.FileName)) != null)
+					pos = GetParameterNamePosition (file, param);
+			}
+			
+			// Plan B. - fallback to searching all partial class files for this parameter's parent member
+			if (pos == -1) {
+				IClass cls = member.DeclaringType;
+				
+				for (int i = 0; i < cls.Parts.Length; i++) {
+					if ((file = ctx.GetFile (cls.Parts[i].Region.FileName)) == null)
+						continue;
+					
+					// sanity check, if the parent member isn't here then neither is the param
+					//if ((pos = GetMemberNamePosition (file, member)) == -1)
+					//	continue;
+					
+					if ((pos = GetParameterNamePosition (file, param)) != -1)
+						break;
+				}
+				
+				if (pos == -1)
+					return false;
+			}
+			
+			string txt = file.GetText (pos, pos + param.Name.Length);
+			if (txt != param.Name)
+				return false;
+			
+			file.DeleteText (pos, txt.Length);
+			file.InsertText (pos, newName);
+			
+			ctx.ParserContext.ParserDatabase.UpdateFile (file.Name, file.Text);
+			
+			return true;
+		}
+
+		public virtual MemberReferenceCollection FindParameterReferences (RefactorerContext ctx, string fileName, IParameter param)
+		{
+			return null;
+		}
 
 		/// Helper overridables ////////////////////////////
 
@@ -231,7 +492,17 @@ namespace MonoDevelop.Projects.CodeGeneration
 			return -1;
 		}
 
-		protected virtual IRegion GetMemberBounds (IMember member)
+		protected virtual int GetVariableNamePosition (IEditableTextFile file, LocalVariable var)
+		{
+			return -1;
+		}
+		
+		protected virtual int GetParameterNamePosition (IEditableTextFile file, IParameter param)
+		{
+			return -1;
+		}
+
+		protected virtual IRegion GetMemberBounds (IEditableTextFile file, IMember member)
 		{
 			int minLin = member.Region.BeginLine;
 			int minCol = member.Region.BeginColumn;
@@ -255,18 +526,21 @@ namespace MonoDevelop.Projects.CodeGeneration
 			}
 			return new DefaultRegion (minLin, minCol, maxLin, maxCol);
 		}
-				
+		
 		protected virtual string GenerateCodeFromMember (CodeTypeMember member)
 		{
 			CodeTypeDeclaration type = new CodeTypeDeclaration ("temp");
 			type.Members.Add (member);
 			ICodeGenerator gen = GetGenerator ();
 			StringWriter sw = new StringWriter ();
-			gen.GenerateCodeFromType (type, sw, GetOpions ());
+			gen.GenerateCodeFromType (type, sw, GetOptions (member is CodeMemberMethod));
 			string code = sw.ToString ();
 			int i = code.IndexOf ('{');
 			int j = code.LastIndexOf ('}');
 			code = code.Substring (i+1, j-i-1);
+			if (member is CodeMemberMethod)
+				if ((i = code.IndexOf ('(')) != -1)
+					code = code.Insert (i, " ");
 			return RemoveIndent (code);
 		}
 		
@@ -277,7 +551,7 @@ namespace MonoDevelop.Projects.CodeGeneration
 		protected IClass GetGeneratedClass (RefactorerContext ctx, IEditableTextFile buffer, IClass cls)
 		{
 			IParseInformation pi = ctx.ParserContext.ParserDatabase.UpdateFile (buffer.Name, buffer.Text);
-			foreach (IClass rclass in ((ICompilationUnit)pi.BestCompilationUnit).Classes) {
+			foreach (IClass rclass in ((ICompilationUnit) pi.MostRecentCompilationUnit).Classes) {
 				if (cls.Name == rclass.Name)
 					return rclass;
 			}
@@ -406,17 +680,17 @@ namespace MonoDevelop.Projects.CodeGeneration
 				pos = GetNextLine (buffer, pos);
 				buffer.InsertText (pos, ind);
 				return pos + ind.Length;
-			}
-			else {
+			} else {
 				IMethod m = cls.Methods [cls.Methods.Count - 1];
 				
 				int pos;
-				if (m.BodyRegion != null && m.BodyRegion.EndLine > 0)
+				if (m.BodyRegion != null && m.BodyRegion.EndLine > 0) {
 					pos = buffer.GetPositionFromLineColumn (m.BodyRegion.EndLine, m.BodyRegion.EndColumn);
-				else
+				} else {
 					// Abstract or P/Inboke methods don't have a body
 					pos = buffer.GetPositionFromLineColumn (m.Region.EndLine, m.Region.EndColumn);
-
+				}
+				
 				pos = GetNextLine (buffer, pos);
 				pos = GetNextLine (buffer, pos);
 				string ind = GetLineIndent (buffer, m.Region.EndLine);
@@ -428,37 +702,35 @@ namespace MonoDevelop.Projects.CodeGeneration
 		protected virtual int GetNewPropertyPosition (IEditableTextFile buffer, IClass cls)
 		{
 			if (cls.Properties.Count == 0) {
-				int pos = GetNewEventPosition (buffer, cls);
+				int pos = GetNewFieldPosition (buffer, cls);
 				int line, col;
 				buffer.GetLineColumnFromPosition (pos, out line, out col);
-				string ind = GetLineIndent (buffer, line);
+				string indent = GetLineIndent (buffer, line);
 				pos = GetNextLine (buffer, pos);
-				buffer.InsertText (pos, ind);
-				return pos + ind.Length;
-			}
-			else {
+				buffer.InsertText (pos, indent);
+				return pos + indent.Length;
+			} else {
 				IProperty m = cls.Properties [cls.Properties.Count - 1];
 				int pos = buffer.GetPositionFromLineColumn (m.BodyRegion.EndLine, m.BodyRegion.EndColumn);
 				pos = GetNextLine (buffer, pos);
 				pos = GetNextLine (buffer, pos);
-				string ind = GetLineIndent (buffer, m.Region.EndLine);
-				buffer.InsertText (pos, ind);
-				return pos + ind.Length;
+				string indent = GetLineIndent (buffer, m.Region.EndLine);
+				buffer.InsertText (pos, indent);
+				return pos + indent.Length;
 			}
 		}
 		
 		protected virtual int GetNewEventPosition (IEditableTextFile buffer, IClass cls)
 		{
 			if (cls.Events.Count == 0) {
-				int pos = GetNewFieldPosition (buffer, cls);
+				int pos = GetNewMethodPosition (buffer, cls);
 				int line, col;
 				buffer.GetLineColumnFromPosition (pos, out line, out col);
 				string ind = GetLineIndent (buffer, line);
 				pos = GetNextLine (buffer, pos);
 				buffer.InsertText (pos, ind);
 				return pos + ind.Length;
-			}
-			else {
+			} else {
 				IEvent m = cls.Events [cls.Events.Count - 1];
 				int pos = buffer.GetPositionFromLineColumn (m.BodyRegion.EndLine, m.BodyRegion.EndColumn);
 				pos = GetNextLine (buffer, pos);
@@ -498,11 +770,12 @@ namespace MonoDevelop.Projects.CodeGeneration
 			return buffer.GetText (ipos, pos);
 		}
 		
-		CodeGeneratorOptions GetOpions ()
+		protected virtual CodeGeneratorOptions GetOptions (bool isMethod)
 		{
 			CodeGeneratorOptions ops = new CodeGeneratorOptions ();
 			ops.IndentString = "\t";
-			ops.BracingStyle = "C";
+			if (isMethod)
+				ops.BracingStyle = "C";
 			return ops;
 		}
 	}
