@@ -39,6 +39,9 @@ namespace MonoDevelop.Projects.CodeGeneration
 		AddField = 0x08,
 		RemoveField = 0x10,
 		RenameField = 0x20,
-		All = 0xff
+		RenameVariable = 0x40,
+		RenameParameter = 0x80,
+		EncapsulateField = 0x100,
+		All = 0xffff
 	}
 }
