@@ -26,17 +26,21 @@
 // WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 //
 
+//#define CHECK_STRINGS
 
 using System;
+using System.Text;
 using System.Threading;
 using System.IO;
 using System.Collections;
+using System.Collections.Specialized;
+using System.Collections.Generic;
 using System.Runtime.Serialization;
 using System.Runtime.Serialization.Formatters.Binary;
 
 using MonoDevelop.Core.Properties;
 using MonoDevelop.Core;
-using MonoDevelop.Core.AddIns;
+using Mono.Addins;
 using MonoDevelop.Projects;
 using MonoDevelop.Projects.Parser;
 using System.Reflection;
@@ -47,7 +51,7 @@ namespace MonoDevelop.Projects.Parser
 	{
 		static readonly int MAX_ACTIVE_COUNT = 100;
 		static readonly int MIN_ACTIVE_COUNT = 50;
-		static protected readonly int FORMAT_VERSION = 13;
+		static protected readonly int FORMAT_VERSION = 23;
 		
 		NamespaceEntry rootNamespace;
 		protected ArrayList references;
@@ -64,19 +68,36 @@ namespace MonoDevelop.Projects.Parser
 		string basePath;
 		string dataFile;
 		
+		// This table is a cache of instantiated generic types. It is not stored
+		// in disk, it's created under demand when a specific type is requested.
+		Hashtable instantiatedGenericTypes = new Hashtable ();
+		
+		// This table stores type->subclasses relations for types which are not
+		// known in this database. For example, types declared in other databases.
+		// For known types, the type->subclasses relation is stored in the corresponding
+		// ClassEntry object, not here. Inner classes don't have a class entry, so their
+		// relations are also stored here.
+		// The key of the hashtable is the full name of a type. The value is an ArrayList
+		// which can contain ClassEntry objects, or other full type names (this second case
+		// is only used for inner classes).
+		Hashtable unresolvedSubclassTable = new Hashtable ();
+		
 		protected Object rwlock = new Object ();
 		
 		public CodeCompletionDatabase (ParserDatabase parserDatabase)
 		{
 			this.parserDatabase = parserDatabase;
-			rootNamespace = new NamespaceEntry ();
+			rootNamespace = new NamespaceEntry (null, null);
 			files = new Hashtable ();
 			references = new ArrayList ();
 			headers = new Hashtable ();
+			
+			Runtime.Properties.PropertyChanged += new PropertyEventHandler (OnPropertyUpdated);	
 		}
 		
 		public virtual void Dispose ()
 		{
+			Runtime.Properties.PropertyChanged -= new PropertyEventHandler (OnPropertyUpdated);
 			disposed = true;
 		}
 		
@@ -94,6 +115,10 @@ namespace MonoDevelop.Projects.Parser
 			get { return disposed; }
 		}
 		
+		public virtual CombineEntry SourceEntry {
+			get { return null; }
+		}
+		
 		protected void SetLocation (string basePath, string name)
 		{
 			dataFile = Path.Combine (basePath, name + ".pidb");
@@ -111,7 +136,7 @@ namespace MonoDevelop.Projects.Parser
 				CloseReader ();
 				
 				if (File.Exists (oldDataFile))
-					File.Move (oldDataFile, dataFile);
+					Runtime.FileService.MoveFile (oldDataFile, dataFile);
 			}
 		}
 		
@@ -152,6 +177,7 @@ namespace MonoDevelop.Projects.Parser
 					references = (ArrayList) dataQueue.Dequeue ();
 					rootNamespace = (NamespaceEntry)  dataQueue.Dequeue ();
 					files = (Hashtable)  dataQueue.Dequeue ();
+					unresolvedSubclassTable = (Hashtable) dataQueue.Dequeue ();
 					DeserializeData (dataQueue);
 
 					ifile.Close ();
@@ -160,12 +186,24 @@ namespace MonoDevelop.Projects.Parser
 				{
 					if (ifile != null) ifile.Close ();
 					Runtime.LoggingService.Error ("PIDB file '" + dataFile + "' couldn not be loaded: '" + ex.Message + "'. The file will be recreated");
-					rootNamespace = new NamespaceEntry ();
+					rootNamespace = new NamespaceEntry (null, null);
 					files = new Hashtable ();
 					references = new ArrayList ();
 					headers = new Hashtable ();
+					unresolvedSubclassTable = new Hashtable ();
 				}
 			}
+			
+			// Notify read comments
+			foreach (FileEntry fe in files.Values)
+			{
+				if (! fe.IsAssembly && fe.CommentTasks != null)
+					parserDatabase.UpdatedCommentTasks (fe);
+			}
+			
+			// Update comments if needed...
+			PropertyEventArgs args = new PropertyEventArgs (null, "Monodevelop.TaskListTokens", LastValidTaskListTokens, Runtime.Properties.GetProperty ("Monodevelop.TaskListTokens", ""));
+			this.OnPropertyUpdated (null, args);
 		}
 		
 		public static Hashtable ReadHeaders (string baseDir, string name)
@@ -186,7 +224,8 @@ namespace MonoDevelop.Projects.Parser
 				
 				modified = false;
 				headers["Version"] = FORMAT_VERSION;
-							
+				headers["LastValidTaskListTokens"] = (string)Runtime.Properties.GetProperty ("Monodevelop.TaskListTokens", "");
+
 				Runtime.LoggingService.Debug ("Writing " + dataFile);
 				
 				string tmpDataFile = dataFile + ".tmp";
@@ -208,45 +247,40 @@ namespace MonoDevelop.Projects.Parser
 					BinaryWriter bufWriter = new BinaryWriter (buffer);
 					
 					// Write all class data
-					foreach (FileEntry fe in files.Values) 
+					foreach (ClassEntry ce in GetAllClasses ()) 
 					{
-						ClassEntry ce = fe.FirstClass;
-						while (ce != null)
-						{
-							IClass c = ce.Class;
-							byte[] data;
-							int len;
+						IClass c = ce.Class;
+						byte[] data;
+						int len;
+						
+						if (c == null) {
+							// Copy the data from the source file
+							if (datareader == null) {
+								datafile = new FileStream (dataFile, FileMode.Open, FileAccess.Read, FileShare.Read);
+								datareader = new BinaryReader (datafile);
+							}
+							datafile.Position = ce.Position;
+							len = datareader.ReadInt32 ();
 							
-							if (c == null) {
-								// Copy the data from the source file
-								if (datareader == null) {
-									datafile = new FileStream (dataFile, FileMode.Open, FileAccess.Read, FileShare.Read);
-									datareader = new BinaryReader (datafile);
-								}
-								datafile.Position = ce.Position;
-								len = datareader.ReadInt32 ();
+							// Sanity check to avoid allocating huge byte arrays if something
+							// goes wrong when reading the file contents
+							if (len > 1024*1024*10 || len < 0)
+								throw new InvalidOperationException ("pidb file corrupted: " + dataFile);
 								
-								// Sanity check to avoid allocating huge byte arrays if something
-								// goes wrong when reading the file contents
-								if (len > 1024*1024*10 || len < 0)
-									throw new InvalidOperationException ("pidb file corrupted: " + dataFile);
-									
-								data = new byte[len];
-								datafile.Read (data, 0, len);
-							}
-							else {
-								buffer.Position = 0;
-								PersistentClass.WriteTo (c, bufWriter, parserDatabase.DefaultNameEncoder);
-								bufWriter.Flush ();
-								data = buffer.GetBuffer ();
-								len = (int)buffer.Position;
-							}
-							
-							ce.Position = dfile.Position;
-							bw.Write (len);
-							bw.Write (data, 0, len);
-							ce = ce.NextInFile;
+							data = new byte[len];
+							datafile.Read (data, 0, len);
 						}
+						else {
+							buffer.Position = 0;
+							PersistentClass.WriteTo (c, bufWriter, parserDatabase.DefaultNameEncoder);
+							bufWriter.Flush ();
+							data = buffer.GetBuffer ();
+							len = (int)buffer.Position;
+						}
+						
+						ce.Position = dfile.Position;
+						bw.Write (len);
+						bw.Write (data, 0, len);
 					}
 					
 					bw.Flush ();
@@ -258,6 +292,7 @@ namespace MonoDevelop.Projects.Parser
 					dataQueue.Enqueue (references);
 					dataQueue.Enqueue (rootNamespace);
 					dataQueue.Enqueue (files);
+					dataQueue.Enqueue (unresolvedSubclassTable);
 					SerializeData (dataQueue);
 					bf.Serialize (dfile, dataQueue.ToArray ());
 					
@@ -271,16 +306,21 @@ namespace MonoDevelop.Projects.Parser
 					CloseReader ();
 					
 					if (File.Exists (dataFile))
-						File.Delete (dataFile);
+						Runtime.FileService.DeleteFile (dataFile);
 						
-					File.Move (tmpDataFile, dataFile);
-				} catch {
+					Runtime.FileService.MoveFile (tmpDataFile, dataFile);
+				} catch (Exception ex) {
+					Console.WriteLine (ex);
 					if (dfile != null)
 						dfile.Close ();
 					if (File.Exists (tmpDataFile))
-						File.Delete (tmpDataFile);
+						Runtime.FileService.DeleteFile (tmpDataFile);
 				}
 			}
+			
+#if CHECK_STRINGS
+			StringNameTable.PrintTop100 ();
+#endif
 		}
 		
 		protected virtual void SerializeData (Queue dataQueue)
@@ -295,7 +335,12 @@ namespace MonoDevelop.Projects.Parser
 		{
 			return files [name] as FileEntry;
 		}
-				
+
+		internal IEnumerable<ClassEntry> GetAllClasses ()
+		{
+			return rootNamespace.GetAllClasses ();
+		}
+		
 		public void Flush ()
 		{
 			// Saves the database if it has too much information
@@ -305,29 +350,22 @@ namespace MonoDevelop.Projects.Parser
 
 			int activeCount = 0;
 			
-			foreach (FileEntry fe in files.Values) {
-				ClassEntry ce = fe.FirstClass;
-				while (ce != null) { 
-					if (ce.Class != null) activeCount++;
-					ce = ce.NextInFile;
-				}
+			foreach (ClassEntry ce in GetAllClasses ()) {
+				if (ce.Class != null)
+					activeCount++;
 			}
 			
 			if (activeCount <= MAX_ACTIVE_COUNT) return;
 			
 			Write ();
 			
-			foreach (FileEntry fe in files.Values) {
-				ClassEntry ce = fe.FirstClass;
-				while (ce != null) { 
-					if (ce.LastGetTime < currentGetTime - MIN_ACTIVE_COUNT)
-						ce.Class = null;
-					ce = ce.NextInFile;
-				}
+			foreach (ClassEntry ce in GetAllClasses ()) {
+				if (ce.LastGetTime < currentGetTime - MIN_ACTIVE_COUNT)
+					ce.Class = null;
 			}
 		}
 		
-		IClass ReadClass (ClassEntry ce)
+		internal IClass ReadClass (ClassEntry ce)
 		{
 			if (datareader == null) {
 				datafile = new FileStream (dataFile, FileMode.Open, FileAccess.Read, FileShare.Read);
@@ -335,7 +373,9 @@ namespace MonoDevelop.Projects.Parser
 			}
 			datafile.Position = ce.Position;
 			datareader.ReadInt32 ();	// Length of data
-			return PersistentClass.Read (datareader, parserDatabase.DefaultNameDecoder);
+			DefaultClass cls = PersistentClass.Read (datareader, parserDatabase.DefaultNameDecoder);
+			cls.SourceProject = SourceEntry;
+			return cls;
 		}
 		
 		void CloseReader ()
@@ -348,16 +388,33 @@ namespace MonoDevelop.Projects.Parser
 		
 		public void Clear ()
 		{
-			rootNamespace = new NamespaceEntry ();
+			rootNamespace = new NamespaceEntry (null, null);
 			files = new Hashtable ();
 			references = new ArrayList ();
 			headers = new Hashtable ();
 		}
 		
-		public IClass GetClass (string typeName, bool caseSensitive)
+		public IClass GetClass (string typeName, ReturnTypeList genericArguments, bool caseSensitive)
 		{
 			lock (rwlock)
 			{
+				if (genericArguments != null && genericArguments.Count > 0) {
+					IClass templateClass = GetClass (typeName, null, caseSensitive);
+					if (templateClass == null)
+						return null;
+
+					if (templateClass.GenericParameters == null || (templateClass.GenericParameters.Count != genericArguments.Count))
+						return null;
+			
+					string tname = DefaultClass.GetInstantiatedTypeName (templateClass.FullyQualifiedName, genericArguments);
+					IClass res = (IClass) instantiatedGenericTypes [tname];
+					if (res == null) {
+						res = DefaultClass.CreateInstantiatedGenericType (templateClass, genericArguments);
+						instantiatedGenericTypes [tname] = res;
+					}
+					return res;
+				}
+				
 //				Runtime.LoggingService.Debug ("GET CLASS " + typeName + " in " + dataFile);
 				string[] path = typeName.Split ('.');
 				int len = path.Length - 1;
@@ -396,15 +453,157 @@ namespace MonoDevelop.Projects.Parser
 			}
 		}
 		
-		IClass GetClass (ClassEntry ce)
+		internal IClass GetClass (ClassEntry ce)
 		{
 			ce.LastGetTime = currentGetTime++;
-			if (ce.Class != null) return ce.Class;
+			if (ce.Class != null)
+				return ce.Class;
+			else
+				return new ClassWrapper (this, ce);
+		}
+		
+		public IEnumerable GetSubclasses (string fullName, string[] namespaces)
+		{
+			ArrayList nsubs = (ArrayList) unresolvedSubclassTable [fullName];
+			ArrayList csubs = null;
+			IList nsList = namespaces;
 			
-			// Read the class from the file
-			
-			ce.Class = ReadClass (ce);
-			return ce.Class;
+			ClassEntry ce = FindClassEntry (fullName);
+			if (ce != null)
+				csubs = ce.Subclasses;
+
+			foreach (ArrayList subs in new object[] { nsubs, csubs }) {
+				if (subs == null)
+					continue;
+				foreach (object ob in subs) {
+					if (ob is ClassEntry) {
+						string ns = ((ClassEntry) ob).NamespaceRef.FullName;
+						if (namespaces == null || nsList.Contains (ns))
+							yield return GetClass ((ClassEntry)ob);
+					}
+					else {
+						// It's a full class name
+						IClass cls = this.GetClass ((string)ob, null, true);
+						if (cls != null && (namespaces == null || nsList.Contains (cls.Namespace)))
+							yield return cls;
+					}
+				}
+			}
+		}
+		
+		void OnPropertyUpdated (object sender, PropertyEventArgs e)
+		{
+			if (e.Key == "Monodevelop.TaskListTokens")
+			{
+				// Update LastValidTagComments
+				headers["LastValidTagComments"] = (string)e.NewValue;
+				
+				List<string> oldTokensList = new List<string> ();
+				if (e.OldValue != null)
+				{
+					string[] tokens = ((string)e.OldValue).Split (';');
+					foreach (string token in tokens)
+					{
+						int pos = token.IndexOf (':');
+						if (pos != -1)
+							oldTokensList.Add (token.Substring (0, pos));
+					}
+				}
+				List<string> newTokensList = new List<string> ();
+				if (e.NewValue != null)
+				{
+					string[] tokens = ((string)e.NewValue).Split (';');
+					foreach (string token in tokens)
+					{
+						int pos = token.IndexOf (':');
+						if (pos != -1)
+							newTokensList.Add (token.Substring (0, pos));
+					}
+				}
+				
+				// Check if tokens just reordered or are the same
+				if (oldTokensList.Count == newTokensList.Count)
+				{
+					bool tokensFound = true;
+					foreach (string token in newTokensList)
+					{	
+						if (oldTokensList.Contains (token)) continue;
+						tokensFound = false;
+						break;
+					}
+					if (tokensFound) return;
+				}
+				
+				// Check if some token(s) just removed
+				if (oldTokensList.Count >= newTokensList.Count)
+				{
+					bool newTokenFound = false;
+					foreach (string token in newTokensList)
+					{	
+						if (oldTokensList.Contains (token)) continue;
+						newTokenFound = true;
+						break;
+					}
+					if (!newTokenFound)
+					{
+						List<string> removedTokensList = new List<string> ();
+						foreach (string token in oldTokensList)
+						{	
+							if (!newTokensList.Contains (token))
+								removedTokensList.Add (token);
+						}
+						
+						// Remove them from FileEntry data
+						foreach (string token in removedTokensList)
+							RemoveSpecialCommentTag (token);
+	                	return;
+					}
+				}
+				
+				QueueAllFilesForParse ();
+			}
+		}
+	
+		public TagCollection GetSpecialComments (string fileName)
+		{
+			lock (rwlock)
+			{
+				FileEntry fe = files[fileName] as FileEntry;
+				return fe != null ? fe.CommentTasks : null;
+			}
+		}
+		
+		public void UpdateTagComments (TagCollection tags, string fileName)
+		{
+			lock (rwlock)
+			{
+				FileEntry fe = files[fileName] as FileEntry;
+				if (fe != null)
+					fe.CommentTasks = tags;
+			}
+		}
+		
+		void RemoveSpecialCommentTag (string token)
+		{
+			foreach (FileEntry fe in files.Values)
+			{
+				if (fe.CommentTasks != null) {
+					List<Tag> markedTags = new List<Tag> ();
+					foreach (Tag tag in fe.CommentTasks)
+						if (tag.Key == token) markedTags.Add (tag);
+					foreach (Tag tag in markedTags)
+						fe.CommentTasks.Remove (tag);
+					parserDatabase.UpdatedCommentTasks (fe);
+				}
+			}
+		}
+		
+		string LastValidTaskListTokens
+		{
+			get
+			{
+				return (string)headers["LastValidTaskListTokens"];
+			}
 		}
 		
 		public void UpdateDatabase ()
@@ -413,7 +612,7 @@ namespace MonoDevelop.Projects.Parser
 			foreach (FileEntry file in list)
 				ParseFile (file.FileName, null);
 		}
-		
+
 		public virtual void CheckModifiedFiles ()
 		{
 			ArrayList list = GetModifiedFileEntries ();
@@ -451,6 +650,16 @@ namespace MonoDevelop.Projects.Parser
 			parserDatabase.QueueParseJob (this, new JobCallback (ParseCallback), file.FileName);
 		}
 		
+		protected void QueueAllFilesForParse ()
+		{
+			lock (rwlock)
+			{
+				foreach (FileEntry file in files.Values)
+					file.LastParseTime = DateTime.MinValue;
+			}
+			CheckModifiedFiles ();
+		}
+		
 		void ParseCallback (object ob, IProgressMonitor monitor)
 		{
 			string fileName = (string) ob;
@@ -528,20 +737,34 @@ namespace MonoDevelop.Projects.Parser
 		{
 			lock (rwlock)
 			{
+				ClassUpdateInformation classInfo = new ClassUpdateInformation ();
+				
 				FileEntry fe = files [fileName] as FileEntry;
 				if (fe == null) return;
 				
-				ClassEntry ce = fe.FirstClass;
-				while (ce != null) {
-					ce.NamespaceRef.Remove (ce.Name);
-					ce = ce.NextInFile;
+				foreach (ClassEntry ce in fe.ClassEntries) {
+					if (ce.Class == null) ce.Class = ReadClass (ce);
+					IClass c = CompoundClass.RemoveFile (ce.Class, fileName);
+					if (c == null) {
+						classInfo.Removed.Add (ce.Class);
+						RemoveSubclassReferences (ce);
+						UnresolveSubclasses (ce);
+						ce.NamespaceRef.Remove (ce.Name);
+					} else
+						ce.Class = c;
 				}
 				
 				files.Remove (fileName);
 				modified = true;
+
+				OnFileRemoved (fileName, classInfo);
 			}
 		}
 		
+		protected virtual void OnFileRemoved (string fileName, ClassUpdateInformation classInfo)
+		{
+		}
+		
 		public ClassUpdateInformation UpdateClassInformation (ClassCollection newClasses, string fileName)
 		{
 			lock (rwlock)
@@ -557,6 +780,7 @@ namespace MonoDevelop.Projects.Parser
 				NamespaceEntry[] newNss = new NamespaceEntry [newClasses.Count];
 				for (int n=0; n<newClasses.Count; n++) {
 					string[] path = newClasses[n].Namespace.Split ('.');
+					((DefaultClass)newClasses[n]).SourceProject = SourceEntry;
 					newNss[n] = GetNamespaceEntry (path, path.Length, true, true);
 				}
 				
@@ -564,8 +788,7 @@ namespace MonoDevelop.Projects.Parser
 				
 				if (fe != null)
 				{
-					ClassEntry ce = fe.FirstClass;
-					while (ce != null)
+					foreach (ClassEntry ce in fe.ClassEntries)
 					{
 						IClass newClass = null;
 						for (int n=0; n<newClasses.Count && newClass == null; n++) {
@@ -577,23 +800,37 @@ namespace MonoDevelop.Projects.Parser
 						}
 						
 						if (newClass != null) {
-							// Class found, replace it
-							ce.Class = CopyClass (newClass);
+							// Class already in the database, update it
+							if (ce.Class == null) ce.Class = ReadClass (ce);
+							RemoveSubclassReferences (ce);
+							ce.Class = CompoundClass.MergeClass (ce.Class, CopyClass (newClass));
+							AddSubclassReferences (ce);							
+							
 							ce.LastGetTime = currentGetTime++;
 							newFileClasses.Add (ce);
 							res.Modified.Add (ce.Class);
 						}
 						else {
-							// Class not found, it has to be deleted, unless it has
-							// been added in another file
-							if (ce.FileEntry == fe) {
-								IClass c = ce.Class;
-								if (c == null) c = ReadClass (ce);
+							// Database class not found in the new class list, it has to be deleted
+							IClass c = ce.Class;
+							if (c == null) {
+								ce.Class = ReadClass (ce);
+								c = ce.Class;
+							}
+							IClass removed = CompoundClass.RemoveFile (c, fileName);
+							if (removed != null) {
+								// It's still a compound class
+								ce.Class = removed;
+								AddSubclassReferences (ce);							
+								res.Modified.Add (removed);
+							} else {
+								// It's not a compoudnd class. Remove it.
+								RemoveSubclassReferences (ce);
+								UnresolveSubclasses (ce);
 								res.Removed.Add (c);
 								ce.NamespaceRef.Remove (ce.Name);
 							}
 						}
-						ce = ce.NextInFile;
 					}
 				}
 				
@@ -605,11 +842,25 @@ namespace MonoDevelop.Projects.Parser
 				for (int n=0; n<newClasses.Count; n++) {
 					if (!added[n]) {
 						IClass c = CopyClass (newClasses[n]);
-						ClassEntry ce = new ClassEntry (c, fe, newNss[n]);
-						ce.LastGetTime = currentGetTime++;
-						newNss[n].Add (c.Name, ce);
+						
+						// A ClassEntry may already exist if part of the class is defined in another file
+						ClassEntry ce = newNss[n].GetClass (c.Name, true);
+						if (ce != null) {
+							// The entry exists, just update it
+							if (ce.Class == null) ce.Class = ReadClass (ce);
+							RemoveSubclassReferences (ce);
+							ce.Class = CompoundClass.MergeClass (ce.Class, c);
+							res.Modified.Add (ce.Class);
+						} else {
+							// It's a new class
+							ce = new ClassEntry (c, fe, newNss[n]);
+							newNss[n].Add (c.Name, ce);
+							res.Added.Add (c);
+							ResolveSubclasses (ce);
+						}
+						AddSubclassReferences (ce);							
 						newFileClasses.Add (ce);
-						res.Added.Add (c);
+						ce.LastGetTime = currentGetTime++;
 					}
 				}
 				
@@ -622,6 +873,109 @@ namespace MonoDevelop.Projects.Parser
 			}
 		}
 		
+		void ResolveSubclasses (ClassEntry ce)
+		{
+			// If this type is registered in the unresolved subclass table, now those subclasses
+			// can properly be assigned.
+			ArrayList subs = (ArrayList) unresolvedSubclassTable [ce.Class.FullyQualifiedName];
+			if (subs != null) {
+				ce.Subclasses = subs;
+				unresolvedSubclassTable.Remove (ce.Class.FullyQualifiedName);
+			}
+		}
+		
+		void UnresolveSubclasses (ClassEntry ce)
+		{
+			// Called when a ClassEntry is removed. If there are registered subclass, add them
+			// to the unresolved subclass table
+			if (ce.Subclasses != null)
+				unresolvedSubclassTable [ce.Class.FullyQualifiedName] = ce.Subclasses;
+		}
+		
+		void AddSubclassReferences (ClassEntry ce)
+		{
+			foreach (IReturnType type in ce.Class.BaseTypes) {
+				string bt = type.FullyQualifiedName;
+				if (bt == "System.Object")
+					continue;
+				ClassEntry sup = FindClassEntry (bt);
+				if (sup != null)
+					sup.RegisterSubclass (ce);
+				else {
+					ArrayList subs = (ArrayList) unresolvedSubclassTable [bt];
+					if (subs == null) {
+						subs = new ArrayList ();
+						unresolvedSubclassTable [bt] = subs;
+					}
+					subs.Add (ce);
+				}
+			}
+			foreach (IClass cls in ce.Class.InnerClasses)
+				AddInnerSubclassReferences (cls);
+		}
+		
+		void AddInnerSubclassReferences (IClass cls)
+		{
+			foreach (IReturnType type in cls.BaseTypes) {
+				string bt = type.FullyQualifiedName;
+				if (bt == "System.Object")
+					continue;
+				ArrayList subs = (ArrayList) unresolvedSubclassTable [bt];
+				if (subs == null) {
+					subs = new ArrayList ();
+					unresolvedSubclassTable [bt] = subs;
+				}
+				subs.Add (cls.FullyQualifiedName);
+			}
+			foreach (IClass ic in cls.InnerClasses)
+				AddInnerSubclassReferences (ic);
+		}
+		
+		void RemoveSubclassReferences (ClassEntry ce)
+		{
+			foreach (IReturnType type in ce.Class.BaseTypes) {
+				ClassEntry sup = FindClassEntry (type.FullyQualifiedName);
+				if (sup != null)
+					sup.UnregisterSubclass (ce);
+					
+				ArrayList subs = (ArrayList) unresolvedSubclassTable [type.FullyQualifiedName];
+				if (subs != null) {
+					subs.Remove (ce);
+					if (subs.Count == 0)
+						unresolvedSubclassTable.Remove (type.FullyQualifiedName);
+				}
+			}
+			foreach (IClass cls in ce.Class.InnerClasses)
+				RemoveInnerSubclassReferences (cls);
+		}
+		
+		void RemoveInnerSubclassReferences (IClass cls)
+		{
+			foreach (IReturnType type in cls.BaseTypes) {
+				ArrayList subs = (ArrayList) unresolvedSubclassTable [type.FullyQualifiedName];
+				if (subs != null)
+					subs.Remove (type.FullyQualifiedName);
+			}
+			foreach (IClass ic in cls.InnerClasses)
+				RemoveInnerSubclassReferences (ic);
+		}
+		
+		ClassEntry FindClassEntry (string fullName)
+		{
+			string[] path = fullName.Split ('.');
+			int len = path.Length - 1;
+			NamespaceEntry nst;
+			int nextPos;
+			
+			if (GetBestNamespaceEntry (path, len, false, true, out nst, out nextPos)) 
+			{
+				ClassEntry ce = nst.GetClass (path[len], true);
+				if (ce == null) return null;
+				return ce;
+			}
+			return null;
+		}
+		
 		public void GetNamespaceContents (LanguageItemCollection list, string subNameSpace, bool caseSensitive)
 		{
 			lock (rwlock)
@@ -659,17 +1013,39 @@ namespace MonoDevelop.Projects.Parser
 			lock (rwlock)
 			{
 				ArrayList list = new ArrayList ();
-				foreach (FileEntry fe in files.Values) {
-					ClassEntry ce = fe.FirstClass;
-					while (ce != null) {
-						list.Add (GetClass (ce));
-						ce = ce.NextInFile;
-					}
+				foreach (ClassEntry ce in GetAllClasses ()) {
+					list.Add (GetClass (ce));
 				}
 				return (IClass[]) list.ToArray (typeof(IClass));
 			}
 		}
 		
+		public IEnumerable<IClass> GetClassList (bool includeInner, string[] namespaces)
+		{
+			lock (rwlock)
+			{
+				IList nsList = namespaces;
+				ArrayList list = new ArrayList ();
+				foreach (ClassEntry ce in GetAllClasses ()) {
+					IClass cls = GetClass (ce);
+					if (nsList != null && !nsList.Contains (cls.Namespace))
+						continue;
+					list.Add (cls);
+					if (includeInner && ((ce.ContentFlags & ContentFlags.HasInnerClasses) != 0))
+						GetAllInnerClassesRec (list, cls);
+				}
+				return (IClass[]) list.ToArray (typeof(IClass));
+			}
+		}
+		
+		void GetAllInnerClassesRec (ArrayList list, IClass cls)
+		{
+			foreach (IClass ic in cls.InnerClasses) {
+				list.Add (ic);
+				GetAllInnerClassesRec (list, ic);
+			}
+		}
+		
 		public void GetNamespaceList (ArrayList list, string subNameSpace, bool caseSensitive)
 		{
 			lock (rwlock)
@@ -706,10 +1082,8 @@ namespace MonoDevelop.Projects.Parser
 			if (fe == null) return new IClass [0];
 
 			ArrayList classes = new ArrayList ();
-			ClassEntry ce = fe.FirstClass;
-			while (ce != null) {
+			foreach (ClassEntry ce in fe.ClassEntries) {
 				classes.Add (GetClass (ce));
-				ce = ce.NextInFile;
 			}
 			return (IClass[]) classes.ToArray (typeof(IClass));
 		}
@@ -722,7 +1096,9 @@ namespace MonoDevelop.Projects.Parser
 			bw.Flush ();
 			ms.Position = 0;
 			BinaryReader br = new BinaryReader (ms);
-			return PersistentClass.Read (br, parserDatabase.DefaultNameDecoder);
+			DefaultClass ret = PersistentClass.Read (br, parserDatabase.DefaultNameDecoder);
+			ret.SourceProject = cls.SourceProject;
+			return ret;
 		}
 		
 		bool GetBestNamespaceEntry (string[] path, int length, bool createPath, bool caseSensitive, out NamespaceEntry lastEntry, out int numMatched)
@@ -743,7 +1119,7 @@ namespace MonoDevelop.Projects.Parser
 							return false;
 						}
 						
-						nh = new NamespaceEntry ();
+						nh = new NamespaceEntry (lastEntry, path[n]);
 						lastEntry.Add (path[n], nh);
 					}
 					lastEntry = nh;
@@ -796,9 +1172,42 @@ namespace MonoDevelop.Projects.Parser
 		
 		public int GetStringId (string text)
 		{
+#if CHECK_STRINGS
+			count++;
+			object ob = all [text];
+			if (ob != null)
+				all [text] = ((int)ob) + 1;
+			else
+				all [text] = 1;
+#endif
+
 			int i = Array.BinarySearch (table, text);
 			if (i >= 0) return i;
 			else return -1;
 		}
+
+#if CHECK_STRINGS
+		static Hashtable all = new Hashtable ();
+		static int count;
+		
+		public static void PrintTop100 ()
+		{
+			string[] ss = new string [all.Count];
+			int[] nn = new int [all.Count];
+			int n = 0;
+			foreach (DictionaryEntry e in all) {
+				ss [n] = (string) e.Key;
+				nn [n] = (int) e.Value;
+				n++;
+			}
+			Array.Sort (nn, ss);
+			n=0;
+			Console.WriteLine ("{0} total strings", count);
+			Console.WriteLine ("{0} unique strings", nn.Length);
+			for (int i = nn.Length - 1; i > nn.Length - 101 && i >= 0; i--) {
+				Console.WriteLine ("\"{1}\", // {2}", n, ss[i], nn[i]);
+			}
+		}
+#endif
 	}
 }
