@@ -36,5 +36,16 @@ namespace MonoDevelop.Projects.Parser
 			get;
 		}
 		
+		bool ByRef {
+			get;
+		}
+		 		
+		/// <summary>
+		/// Contains values (types) of actual parameters (arguments) to a
+		/// generic type.
+		/// </summary>
+		ReturnTypeList GenericArguments {
+			get;
+		}
 	}
 }
