@@ -12,7 +12,7 @@ using MonoDevelop.Core;
 namespace MonoDevelop.Projects.Parser
 {
 	[Serializable]
-	internal sealed class PersistentProperty : AbstractProperty
+	internal sealed class PersistentProperty : DefaultProperty
 	{
 		const uint canGetFlag = (uint)(1 << 29);
 		const uint canSetFlag = (uint)(1 << 30);
@@ -35,13 +35,14 @@ namespace MonoDevelop.Projects.Parser
 		public static PersistentProperty Resolve (IProperty source, ITypeResolver typeResolver)
 		{
 			PersistentProperty pro = new PersistentProperty ();
-			pro.FullyQualifiedName = source.FullyQualifiedName;
+			pro.Name = source.Name;
 			pro.Documentation = source.Documentation;
 			pro.modifiers = source.Modifiers;
 			pro.returnType = PersistentReturnType.Resolve (source.ReturnType, typeResolver);
 			pro.canGet = source.CanGet;
 			pro.canSet = source.CanSet;
 			pro.region = source.Region;
+			pro.bodyRegion = source.BodyRegion;
 			pro.attributes = PersistentAttributeSectionCollection.Resolve (source.Attributes, typeResolver);
 			return pro;
 		}
@@ -49,7 +50,7 @@ namespace MonoDevelop.Projects.Parser
 		public static PersistentProperty Read (BinaryReader reader, INameDecoder nameTable)
 		{
 			PersistentProperty pro = new PersistentProperty ();
-			pro.FullyQualifiedName = PersistentHelper.ReadString (reader, nameTable);
+			pro.Name = PersistentHelper.ReadString (reader, nameTable);
 			pro.Documentation = PersistentHelper.ReadString (reader, nameTable);
 			uint m = reader.ReadUInt32();
 			pro.modifiers = (ModifierEnum)(m & (canGetFlag - 1));
@@ -57,17 +58,19 @@ namespace MonoDevelop.Projects.Parser
 			pro.canSet = (m & canSetFlag) == canSetFlag;
 			pro.returnType = PersistentReturnType.Read (reader, nameTable);
 			pro.region = PersistentRegion.Read (reader, nameTable);
+			pro.bodyRegion = PersistentRegion.Read (reader, nameTable);
 			pro.attributes = PersistentAttributeSectionCollection.Read (reader, nameTable);
 			return pro;
 		}
 		
 		public static void WriteTo (IProperty p, BinaryWriter writer, INameEncoder nameTable)
 		{
-			PersistentHelper.WriteString (p.FullyQualifiedName, writer, nameTable);
+			PersistentHelper.WriteString (p.Name, writer, nameTable);
 			PersistentHelper.WriteString (p.Documentation, writer, nameTable);
 			writer.Write((uint)p.Modifiers + (p.CanGet ? canGetFlag : 0) + (p.CanSet ? canSetFlag : 0));
 			PersistentReturnType.WriteTo (p.ReturnType, writer, nameTable);
 			PersistentRegion.WriteTo (p.Region, writer, nameTable);
+			PersistentRegion.WriteTo (p.BodyRegion, writer, nameTable);
 			PersistentAttributeSectionCollection.WriteTo (p.Attributes, writer, nameTable);
 		}
 	}
