@@ -41,22 +41,15 @@ namespace MonoDevelop.Projects.Parser
 	{
 		Project project;
 		bool initialFileCheck;
-		ClrVersion lastVersion;
+		ClrVersion lastVersion = ClrVersion.Default;
 		
 		public ProjectCodeCompletionDatabase (Project project, ParserDatabase parserDatabase)
 		: base (parserDatabase)
 		{
-			initialFileCheck = true;
-			
 			SetLocation (project.BaseDirectory, project.Name);
 			
 			this.project = project;
 			
-			// Store the project target runtime
-			DotNetProject prj = project as DotNetProject;
-			if (prj != null)
-				lastVersion = prj.ClrVersion;
-			
 			Read ();
 			
 			UpdateFromProject ();
@@ -66,12 +59,18 @@ namespace MonoDevelop.Projects.Parser
 			project.FileRemovedFromProject += new ProjectFileEventHandler (OnFileRemoved);
 			project.FileRenamedInProject += new ProjectFileRenamedEventHandler (OnFileRenamed);
 			project.Modified += new CombineEntryEventHandler (OnProjectModified);
+
+			initialFileCheck = true;
 		}
 		
 		public Project Project {
 			get { return project; }
 		}
 		
+		public override CombineEntry SourceEntry {
+			get { return project; }
+		}
+		
 		public override void Dispose ()
 		{
 			base.Dispose ();
@@ -222,7 +221,8 @@ namespace MonoDevelop.Projects.Parser
 					return new string[] { "Project:" + pr.Reference };
 				case ReferenceType.Gac:
 					string refId = pr.Reference;
-					if (refId.ToLower().EndsWith (".dll"))
+					string ext = Path.GetExtension (refId).ToLower ();
+					if (ext == ".dll" || ext == ".exe")
 						refId = refId.Substring (0, refId.Length - 4);
 					return new string[] { "Assembly:" + refId };
 				default:
@@ -261,6 +261,14 @@ namespace MonoDevelop.Projects.Parser
 			
 			FileEntry file = files [fileName] as FileEntry;
 			if (file != null) {
+				
+				if (file.CommentTasks != ((ICompilationUnit)parserInfo.BestCompilationUnit).TagComments)
+				{
+					file.CommentTasks = ((ICompilationUnit)parserInfo.BestCompilationUnit).TagComments;
+					parserDatabase.UpdatedCommentTasks (file);
+				}
+				
+				
 				if (!allResolved) {
 					if (file.ParseErrorRetries > 0) {
 						file.ParseErrorRetries--;
@@ -275,6 +283,13 @@ namespace MonoDevelop.Projects.Parser
 			Flush ();
 			return res;
 		}
+		
+		protected override void OnFileRemoved (string fileName, ClassUpdateInformation classInfo)
+		{
+			if (classInfo.Removed.Count > 0)
+				parserDatabase.NotifyParseInfoChange (fileName, classInfo, project);
+		}
+
 	}
 }
 
