@@ -17,16 +17,10 @@ using MDGenericParameter = MonoDevelop.Projects.Parser.GenericParameter;
 namespace MonoDevelop.Projects.Parser
 {
 	[Serializable]
-	internal class ReflectionClass : AbstractClass
+	internal class ReflectionClass : DefaultClass
 	{
-		/// <value>
-		/// A reflection class doesn't have a compilation unit (because
-		/// it is not parsed the information is gathered using reflection)
-		/// </value>
-		public override ICompilationUnit CompilationUnit {
-			get {
-				return null;
-			}
+		public ReflectionClass (): base (null)
+		{
 		}
 		
 		public static bool IsDelegate (TypeReference type)
@@ -46,11 +40,14 @@ namespace MonoDevelop.Projects.Parser
 			return false;
 		}
 		
-		public ReflectionClass(TypeDefinition type)
+		public ReflectionClass (TypeDefinition type): base (null)
 		{
-			FullyQualifiedName = type.FullName.Replace ('/','+');
+			string fqname = type.FullName.Replace ('/','+');
+			Name = type.Name.Replace ('/','.');
+			Name = Name.Replace ('+', '.');
+			Namespace = type.Namespace;
 
-			XmlDocument docs = Services.DocumentationService != null ? Services.DocumentationService.GetHelpXml (FullyQualifiedName) : null;
+			XmlDocument docs = Services.DocumentationService != null ? Services.DocumentationService.GetHelpXml (fqname) : null;
 			if (docs != null) {
 				XmlNode node = docs.SelectSingleNode ("/Type/Docs/summary");
 				if (node != null) {
@@ -58,13 +55,11 @@ namespace MonoDevelop.Projects.Parser
 				}
 			}
 			
-			FullyQualifiedName = FullyQualifiedName.Replace('+', '.');
-			
 			// set classtype
 			if (IsDelegate(type)) {
 				classType = ClassType.Delegate;
 				MethodDefinition invoke = type.Methods.GetMethod ("Invoke")[0];
-				ReflectionMethod newMethod = new ReflectionMethod(invoke, null);
+				ReflectionMethod newMethod = new ReflectionMethod (invoke, null);
 				methods.Add(newMethod);
 			} else if (type.IsInterface) {
 				classType = ClassType.Interface;
@@ -118,12 +113,14 @@ namespace MonoDevelop.Projects.Parser
 				}
 				
 				foreach (TypeDefinition nestedType in type.NestedTypes) {
-					innerClasses.Add (new ReflectionClass(nestedType));
+					TypeAttributes vis = nestedType.Attributes & TypeAttributes.VisibilityMask;
+					if (vis == TypeAttributes.Public || vis == TypeAttributes.NestedPublic)
+						innerClasses.Add (new ReflectionClass(nestedType));
 				}
 				
 				foreach (FieldDefinition field in type.Fields) {
 //					if (!field.IsSpecialName) {
-					IField newField = new ReflectionField(field, docs);
+					IField newField = new ReflectionField (field, docs);
 					if (!newField.IsInternal) {
 						fields.Add(newField);
 					}
@@ -131,43 +128,47 @@ namespace MonoDevelop.Projects.Parser
 				}
 				
 				foreach (PropertyDefinition propertyInfo in type.Properties) {
-//					if (!propertyInfo.IsSpecialName) {
 					ParameterDefinitionCollection p = propertyInfo.Parameters;
 					
 					if (p == null || p.Count == 0) {
-						IProperty newProperty = new ReflectionProperty(propertyInfo, docs);
+						IProperty newProperty = new ReflectionProperty (propertyInfo, docs);
 						if (!newProperty.IsInternal) {
 							properties.Add(newProperty);
+							UpdateClassRegion (newProperty.Region);
 						}
 					} else {
-						IIndexer newIndexer = new ReflectionIndexer(propertyInfo, docs);
+						IIndexer newIndexer = new ReflectionIndexer (propertyInfo, docs);
 						if (!newIndexer.IsInternal) {
-							indexer.Add(newIndexer);
+							indexer.Add (newIndexer);
+							UpdateClassRegion (newIndexer.Region);
 						}
 					}
-//					}
 				}
 				
 				foreach (MethodDefinition methodInfo in type.Methods) {
-					if (!methodInfo.IsSpecialName) {
-						IMethod newMethod = new ReflectionMethod(methodInfo, docs);
-						
-						if (!newMethod.IsInternal) {
-							methods.Add(newMethod);
-						}
+					// Include overloaded operators
+					if (methodInfo.IsSpecialName && !methodInfo.Name.StartsWith ("op_"))
+						continue;
+					
+					IMethod newMethod = new ReflectionMethod (methodInfo, docs);
+					
+					if (!newMethod.IsInternal) {
+						methods.Add (newMethod);
+						UpdateClassRegion (newMethod.Region);
 					}
 				}
 				
 				foreach (MethodDefinition constructorInfo in type.Constructors) {
-					IMethod newMethod = new ReflectionMethod(constructorInfo, docs);
+					IMethod newMethod = new ReflectionMethod (constructorInfo, docs);
 					if (!newMethod.IsInternal) {
-						methods.Add(newMethod);
+						methods.Add (newMethod);
+						UpdateClassRegion (newMethod.Region);
 					}
 				}
 				
 				foreach (EventDefinition eventInfo in type.Events) {
 //					if (!eventInfo.IsSpecialName) {
-					IEvent newEvent = new ReflectionEvent(eventInfo, docs);
+					IEvent newEvent = new ReflectionEvent (eventInfo, docs);
 					
 					if (!newEvent.IsInternal) {
 						events.Add(newEvent);
@@ -177,6 +178,27 @@ namespace MonoDevelop.Projects.Parser
 			}
 		}
 		
+		void UpdateClassRegion (IRegion memberRegion)
+		{
+			DefaultRegion reg = memberRegion as DefaultRegion;
+			if (reg == null)
+				return;
+			
+			if (region == null)
+				region = new DefaultRegion (0, 0);
+			
+			DefaultRegion cregion = (DefaultRegion) region;
+			
+			if (cregion.FileName == null || cregion.FileName == reg.FileName) {
+				cregion.FileName = reg.FileName;
+				reg.FileName = null;
+				if (cregion.BeginLine == 0 || cregion.BeginLine > reg.BeginLine) {
+					cregion.BeginLine = reg.BeginLine;
+					cregion.BeginColumn = reg.BeginColumn;
+				}
+			}
+		}
+		
 		public static ModifierEnum GetModifiers (TypeAttributes attributes)
 		{
 			TypeAttributes visibility = attributes & TypeAttributes.VisibilityMask;
