@@ -13,12 +13,12 @@ using Mono.Cecil;
 namespace MonoDevelop.Projects.Parser
 {
 	[Serializable]
-	internal class ReflectionField : AbstractField
+	internal class ReflectionField : DefaultField
 	{
-		public ReflectionField(FieldDefinition fieldInfo, XmlDocument docs)
+		public ReflectionField (FieldDefinition fieldInfo, XmlDocument docs)
 		{
 			System.Diagnostics.Debug.Assert(fieldInfo != null);
-			FullyQualifiedName = String.Concat(fieldInfo.DeclaringType.FullName, ".", fieldInfo.Name);
+			Name = fieldInfo.Name;
 
 			if (docs != null) {
 				XmlNode node = docs.SelectSingleNode ("/Type/Members/Member[@MemberName='" + fieldInfo.Name + "']/Docs/summary");
