@@ -78,6 +78,8 @@ namespace MonoDevelop.Projects
 			}
 			set {
 				startupEntry = value;
+				if (value == null)
+					startEntryName = null;
 				NotifyModified ();
 				OnStartupPropertyChanged(null);
 			}
@@ -131,15 +133,16 @@ namespace MonoDevelop.Projects
 			}
 		}
 		
-		public override bool NeedsBuilding {
-			get {
-				foreach (CombineEntry entry in Entries)
-					if (entry.NeedsBuilding) return true;
-				return false;
-			}
-			set {
-				// Ignore
-			}
+		protected internal override bool OnGetNeedsBuilding ()
+		{
+			foreach (CombineEntry entry in Entries)
+				if (entry.NeedsBuilding) return true;
+			return false;
+		}
+		
+		protected internal override void OnSetNeedsBuilding (bool value)
+		{
+			// Ignore
 		}
 		
 		public Combine()
@@ -187,6 +190,59 @@ namespace MonoDevelop.Projects
 			}
 		}
 		
+		public CombineEntry ReloadEntry (IProgressMonitor monitor, CombineEntry entry)
+		{
+			if (Entries.IndexOf (entry) == -1)
+				throw new InvalidOperationException ("Combine entry '" + entry.Name + "' does not belong to combine '" + Name + "'");
+
+			// Load the new entry
+			
+			CombineEntry newEntry;
+			try {
+				newEntry = Services.ProjectService.ReadCombineEntry (entry.FileName, monitor);
+			} catch (Exception ex) {
+				UnknownCombineEntry e = new UnknownCombineEntry ();
+				e.LoadError = ex.Message;
+				e.FileName = entry.FileName;
+				newEntry = e;
+			}
+			
+			// Replace the old entry by the new entry in the execute definitions
+			
+			foreach (CombineExecuteDefinition exDef in CombineExecuteDefinitions) {
+				if (exDef.Entry == entry) {
+					exDef.Entry = newEntry;
+					break;
+				}
+			}
+
+			// Replace in the configurations as well
+			
+			foreach (CombineConfiguration dentry in Configurations) {
+				foreach (CombineConfigurationEntry cce in dentry.Entries) {
+					if (cce.Entry == entry)
+						cce.Entry = newEntry;
+				}
+			}
+
+			if (StartupEntry == entry)
+				StartupEntry = newEntry;
+			
+			// Replace in the file list
+			Entries.Replace (entry, newEntry);
+			
+			DisconnectChildEntryEvents (entry);
+			ConnectChildEntryEvents (newEntry);
+
+			NotifyModified ();
+			OnEntryRemoved (new CombineEntryChangeEventArgs (entry, true));
+			OnEntryAdded (new CombineEntryChangeEventArgs (newEntry, true));
+			
+			entry.Dispose ();
+			
+			return newEntry;
+		}
+		
 		internal void NotifyEntryAdded (CombineEntry entry)
 		{
 			if (StartupEntry == null)
@@ -227,7 +283,15 @@ namespace MonoDevelop.Projects
 					entry.Configurations.Add(new CombineConfiguration(conf.Name));			
 			
 			combineExecuteDefinitions.Add (new CombineExecuteDefinition (entry, EntryExecuteType.None));
+			
+			ConnectChildEntryEvents (entry);
 
+			NotifyModified ();
+			OnEntryAdded (new CombineEntryChangeEventArgs (entry, false));
+		}
+		
+		void ConnectChildEntryEvents (CombineEntry entry)
+		{
 			if (entry is Project)
 			{
 				Project project = entry as Project;
@@ -252,9 +316,6 @@ namespace MonoDevelop.Projects
 			}
 			entry.Modified += entryModifiedHandler;
 			entry.Saved += entrySavedHandler;
-			
-			NotifyModified ();
-			OnEntryAdded (new CombineEntryEventArgs (entry));
 		}
 		
 		public override DataCollection Serialize (ITypeSerializer handler)
@@ -288,9 +349,9 @@ namespace MonoDevelop.Projects
 			}
 		}
 
-		public override void Save (IProgressMonitor monitor)
+		protected internal override void OnSave (IProgressMonitor monitor)
 		{
-			base.Save (monitor);
+			base.OnSave (monitor);
 			foreach (CombineEntry entry in Entries)
 			{
 				if (entry is Combine || entry is Project)
@@ -301,12 +362,41 @@ namespace MonoDevelop.Projects
 		public CombineEntry AddEntry (string filename, IProgressMonitor monitor)
 		{
 			if (monitor == null) monitor = new NullProgressMonitor ();
-			CombineEntry entry = Services.ProjectService.ReadFile (filename, monitor);
+			CombineEntry entry = Services.ProjectService.ReadCombineEntry (filename, monitor);
 			Entries.Add (entry);
 			return entry;
 		}
 
 		internal void NotifyEntryRemoved (CombineEntry entry)
+		{
+			DisconnectChildEntryEvents (entry);
+
+			// remove execute definition
+			CombineExecuteDefinition removeExDef = null;
+			foreach (CombineExecuteDefinition exDef in CombineExecuteDefinitions) {
+				if (exDef.Entry == entry) {
+					removeExDef = exDef;
+					break;
+				}
+			}
+			CombineExecuteDefinitions.Remove (removeExDef);
+
+			// remove configuration
+			foreach (CombineConfiguration dentry in Configurations)
+				dentry.RemoveEntry (entry);
+
+			if (StartupEntry == entry) {
+				if (Entries.Count > 0)
+					StartupEntry = Entries [0];
+				else
+					StartupEntry = null;
+			}
+			
+			NotifyModified ();
+			OnEntryRemoved (new CombineEntryChangeEventArgs (entry, false));
+		}
+		
+		void DisconnectChildEntryEvents (CombineEntry entry)
 		{
 			Project pce = entry as Project;
 			if (pce != null) {
@@ -332,23 +422,6 @@ namespace MonoDevelop.Projects
 			}
 			entry.Modified -= entryModifiedHandler;
 			entry.Saved -= entrySavedHandler;
-
-			// remove execute definition
-			CombineExecuteDefinition removeExDef = null;
-			foreach (CombineExecuteDefinition exDef in CombineExecuteDefinitions) {
-				if (exDef.Entry == entry) {
-					removeExDef = exDef;
-					break;
-				}
-			}
-			CombineExecuteDefinitions.Remove (removeExDef);
-
-			// remove configuration
-			foreach (CombineConfiguration dentry in Configurations)
-				dentry.RemoveEntry (entry);
-
-			NotifyModified ();
-			OnEntryRemoved (new CombineEntryEventArgs (entry));
 		}
 		
 		private void RemoveReferencesToProject(Project projectToRemove)
@@ -396,7 +469,7 @@ namespace MonoDevelop.Projects
 			Entries.Remove (entry);
 		}
 		
-		public override void Execute (IProgressMonitor monitor, ExecutionContext context)
+		protected internal override void OnExecute (IProgressMonitor monitor, ExecutionContext context)
 		{
 			if (singleStartup) {
 				if (StartupEntry != null)
@@ -486,26 +559,47 @@ namespace MonoDevelop.Projects
 			return GetAllProjects (false);
 		}
 		
+		/// <remarks>
+		/// Returns a collection containing all entries in this combine and 
+		/// undercombines
+		/// </remarks>
+		public CombineEntryCollection GetAllEntries ()
+		{
+			return GetAllEntries (null);
+		}
+		
+		/// <remarks>
+		/// Returns a collection containing all entries of the given type in this combine and 
+		/// undercombines
+		/// </remarks>
+		public CombineEntryCollection GetAllEntries (Type type)
+		{
+			CombineEntryCollection list = new CombineEntryCollection();
+			GetAllEntries (list, type, this);
+			return list;
+		}
+		
 		// When topologicalSort=true, the projects are returned in the order
 		// they should be compiled, acording to their references.
 		public CombineEntryCollection GetAllProjects (bool topologicalSort)
 		{
 			CombineEntryCollection list = new CombineEntryCollection();
-			GetAllProjects (list);
+			GetAllEntries (list, typeof(Project), this);
 			if (topologicalSort)
 				return TopologicalSort (list);
 			else
 				return list;
 		}
 		
-		void GetAllProjects (CombineEntryCollection list)
+		void GetAllEntries (CombineEntryCollection list, Type type, CombineEntry entry)
 		{
-			foreach (CombineEntry entry in Entries) {
-				if (entry is Project) {
-					list.Add (entry);
-				} else if (entry is Combine) {
-					((Combine)entry).GetAllProjects (list);
-				}
+			if (type == null || type.IsInstanceOfType (entry)) {
+				list.Add (entry);
+			}
+		
+			if (entry is Combine) {
+				foreach (CombineEntry ce in ((Combine)entry).Entries)
+					GetAllEntries (list, type, ce);
 			}
 		}
 		
@@ -601,22 +695,25 @@ namespace MonoDevelop.Projects
 			inserted[index] = true;
 		}
 		
-		public override void Clean ()
+		protected internal override void OnClean (IProgressMonitor monitor)
 		{
-			if (ActiveConfiguration == null)
+			CombineConfiguration config = ActiveConfiguration as CombineConfiguration;
+			if (config == null)
 				return;
-			foreach (CombineConfigurationEntry cce in ((CombineConfiguration)ActiveConfiguration).Entries)
-				cce.Entry.Clean ();
+
+			foreach (CombineConfigurationEntry cce in config.Entries)
+				cce.Entry.Clean (monitor);
 		}
 		
-		public override ICompilerResult Build (IProgressMonitor monitor)
+		protected internal override ICompilerResult OnBuild (IProgressMonitor monitor)
 		{
 			if (ActiveConfiguration == null) {
 				monitor.ReportError (GettextCatalog.GetString ("The solution does not have an active configuration."), null);
 				return new DefaultCompilerResult (new CompilerResults (null), "", 0, 1);
 			}
+			
 			CombineEntryCollection allProjects = GetAllBuildableEntries (ActiveConfiguration.Name);
-			monitor.BeginTask (string.Format (GettextCatalog.GetString ("Building Solution {0}"), Name), allProjects.Count);
+			monitor.BeginTask (GettextCatalog.GetString ("Building Solution {0}", Name), allProjects.Count);
 			try {
 				CompilerResults cres = new CompilerResults (null);
 				
@@ -776,6 +873,22 @@ namespace MonoDevelop.Projects
 			OnEntrySaved (e);
 		}
 		
+		internal void NotifyEntryAddedToCombine (object sender, CombineEntryChangeEventArgs e)
+		{
+			if (EntryAddedToCombine != null)
+				EntryAddedToCombine (sender, e);
+			if (RootCombine != null && RootCombine != this)
+				RootCombine.NotifyEntryAddedToCombine (sender, e);
+		}
+		
+		internal void NotifyEntryRemovedFromCombine (object sender, CombineEntryChangeEventArgs e)
+		{
+			if (EntryRemovedFromCombine != null)
+				EntryRemovedFromCombine (sender, e);
+			if (RootCombine != null && RootCombine != this)
+				RootCombine.NotifyEntryRemovedFromCombine (sender, e);
+		}
+		
 		protected virtual void OnStartupPropertyChanged(EventArgs e)
 		{
 			if (StartupPropertyChanged != null) {
@@ -783,18 +896,20 @@ namespace MonoDevelop.Projects
 			}
 		}
 			
-		protected virtual void OnEntryAdded(CombineEntryEventArgs e)
+		protected virtual void OnEntryAdded (CombineEntryChangeEventArgs e)
 		{
 			if (EntryAdded != null) {
 				EntryAdded (this, e);
 			}
+			NotifyEntryAddedToCombine (this, e);
 		}
 		
-		protected virtual void OnEntryRemoved(CombineEntryEventArgs e)
+		protected virtual void OnEntryRemoved (CombineEntryChangeEventArgs e)
 		{
 			if (EntryRemoved != null) {
 				EntryRemoved (this, e);
 			}
+			NotifyEntryRemovedFromCombine (this, e);
 		}
 		
 		protected virtual void OnFileRemovedFromProject (ProjectFileEventArgs e)
@@ -859,8 +974,10 @@ namespace MonoDevelop.Projects
 		}
 		
 		public event EventHandler StartupPropertyChanged;
-		public event CombineEntryEventHandler EntryAdded;
-		public event CombineEntryEventHandler EntryRemoved;
+		public event CombineEntryChangeEventHandler EntryAdded;
+		public event CombineEntryChangeEventHandler EntryRemoved;
+		public event CombineEntryChangeEventHandler EntryAddedToCombine;     // Fires for child combines
+		public event CombineEntryChangeEventHandler EntryRemovedFromCombine; // Fires for child combines
 		public event ProjectFileEventHandler FileAddedToProject;
 		public event ProjectFileEventHandler FileRemovedFromProject;
 		public event ProjectFileEventHandler FileChangedInProject;
