@@ -31,10 +31,11 @@ namespace MonoDevelop.Projects
 			get;
 		}
 		
-		CombineEntry ReadFile (string file, IProgressMonitor monitor);
+		CombineEntry ReadCombineEntry (string file, IProgressMonitor monitor);
+
+		string Export (IProgressMonitor monitor, string rootSourceFile, string targetPath, IFileFormat format);
+		string Export (IProgressMonitor monitor, string rootSourceFile, string[] childEnryFiles, string targetPath, IFileFormat format);
 
-		void WriteFile (string file, CombineEntry entry, IProgressMonitor monitor);
-		
 		bool CanCreateSingleFileProject (string file);
 		Project CreateSingleFileProject (string file);
 		
