@@ -34,28 +34,9 @@ using MonoDevelop.Core;
 
 namespace MonoDevelop.Projects
 {
-	internal class PrjxFileFormat: IFileFormat
+	internal class PrjxFileFormat
 	{
-		public string Name {
-			get { return "SharpDevelop Project"; }
-		}
-		
-		public string GetValidFormatName (string fileName)
-		{
-			return Path.ChangeExtension (fileName, ".prjx");
-		}
-		
-		public bool CanReadFile (string file)
-		{
-			return string.Compare (Path.GetExtension (file), ".prjx", true) == 0;
-		}
-		
-		public bool CanWriteFile (object obj)
-		{
-			return obj is Project;
-		}
-		
-		public void WriteFile (string file, object node, IProgressMonitor monitor)
+		public static void WriteFile (string file, object node, IProgressMonitor monitor)
 		{
 			Project project = node as Project;
 			if (project == null)
@@ -63,19 +44,19 @@ namespace MonoDevelop.Projects
 
 			StreamWriter sw = new StreamWriter (file);
 			try {
-				monitor.BeginTask (string.Format (GettextCatalog.GetString("Saving project: {0}"), file), 1);
+				monitor.BeginTask (GettextCatalog.GetString ("Saving project: {0}", file), 1);
 				XmlDataSerializer ser = new XmlDataSerializer (Services.ProjectService.DataContext);
 				ser.SerializationContext.BaseFile = file;
 				ser.Serialize (sw, project, typeof(Project));
 			} catch (Exception ex) {
-				monitor.ReportError (string.Format (GettextCatalog.GetString ("Could not save project: {0}"), file), ex);
+				monitor.ReportError (GettextCatalog.GetString ("Could not save project: {0}", file), ex);
 			} finally {
 				monitor.EndTask ();
 				sw.Close ();
 			}
 		}
 		
-		public object ReadFile (string fileName, IProgressMonitor monitor)
+		public static object ReadFile (string fileName, IProgressMonitor monitor)
 		{
 			XmlTextReader reader = new XmlTextReader (new StreamReader (fileName));
 			reader.MoveToContent ();
@@ -110,16 +91,16 @@ namespace MonoDevelop.Projects
 			}
 			
 			if (version != "2.0")
-				monitor.ReportWarning (string.Format (GettextCatalog.GetString ("The file '{0}' is using an old project file format. It will be automatically converted to the current format."), fileName));
+				monitor.ReportWarning (GettextCatalog.GetString ("The file '{0}' is using an old project file format. It will be automatically converted to the current format.", fileName));
 			
 			try {
-				monitor.BeginTask (string.Format (GettextCatalog.GetString ("Loading project: {0}"), fileName), 1);
+				monitor.BeginTask (GettextCatalog.GetString ("Loading project: {0}", fileName), 1);
 				if (projectReader != null) 
 					return projectReader.ReadProject (reader);
 				else
 					throw new UnknownProjectVersionException (fileName, version);
 			} catch (Exception ex) {
-				monitor.ReportError (string.Format (GettextCatalog.GetString ("Could not load project: {0}"), fileName), ex);
+				monitor.ReportError (GettextCatalog.GetString ("Could not load project: {0}", fileName), ex);
 				throw;
 			} finally {
 				monitor.EndTask ();
@@ -163,7 +144,7 @@ namespace MonoDevelop.Projects
 			project.FileName = file;
 			DataItem data = (DataItem) Read (reader);
 			serializer.Deserialize (project, data);
-			project.FileFormat = new MdpFileFormat ();
+			project.FileFormat = new MonoDevelopFileFormat ();
 			return project;
 		}
 		
@@ -221,7 +202,6 @@ namespace MonoDevelop.Projects
 			DataNode data = Read (reader);
 			Project project = (Project) serializer.Deserialize (typeof(Project), data);
 			project.FileName = serializer.SerializationContext.BaseFile;
-			project.FileFormat = new MdpFileFormat ();
 			return project;
 		}
 	}
