@@ -11,6 +11,7 @@ using System.Diagnostics;
 using System.Globalization;
 using System.IO;
 using System.Collections;
+using System.Collections.Specialized;
 using System.Reflection;
 using System.Xml;
 using System.CodeDom.Compiler;
@@ -18,7 +19,7 @@ using System.CodeDom.Compiler;
 using MonoDevelop.Core;
 using MonoDevelop.Core.Execution;
 using MonoDevelop.Core.Properties;
-using MonoDevelop.Core.AddIns;
+using Mono.Addins;
 using MonoDevelop.Projects.Ambience;
 using MonoDevelop.Projects.Serialization;
 
@@ -40,6 +41,9 @@ namespace MonoDevelop.Projects
 		[ItemProperty ("Description", DefaultValue="")]
 		protected string description     = "";
 
+		[ItemProperty ("DefaultNamespace", DefaultValue="")]
+		protected string defaultNamespace = String.Empty;
+
 		[ItemProperty ("newfilesearch", DefaultValue = NewFileSearch.None)]
 		protected NewFileSearch newFileSearch  = NewFileSearch.None;
 
@@ -74,6 +78,11 @@ namespace MonoDevelop.Projects
 				NotifyModified ();
 			}
 		}
+
+		public string DefaultNamespace {
+			get { return defaultNamespace; }
+		        set { defaultNamespace = value; }
+		}
 		
 		[Browsable(false)]
 		[ItemProperty ("Contents")]
@@ -148,7 +157,7 @@ namespace MonoDevelop.Projects
 				
 		public static Project LoadProject (string filename, IProgressMonitor monitor)
 		{
-			Project prj = Services.ProjectService.ReadFile (filename, monitor) as Project;
+			Project prj = Services.ProjectService.ReadCombineEntry (filename, monitor) as Project;
 			if (prj == null)
 				throw new InvalidOperationException ("Invalid project file: " + filename);
 			
@@ -159,6 +168,7 @@ namespace MonoDevelop.Projects
 		{
 			base.Deserialize (handler, data);
 			projectReferences.SetProject (this);
+			isDirty = false;
 		}
 
 		public virtual string GetParseableFileContent(string fileName)
@@ -200,48 +210,60 @@ namespace MonoDevelop.Projects
 		
 		void CopyReferencesToOutputPath (string destPath, bool force)
 		{
-			foreach (ProjectReference projectReference in ProjectReferences) {
-				if ((projectReference.LocalCopy || force) && projectReference.ReferenceType != ReferenceType.Gac) {
-					foreach (string referenceFileName in projectReference.GetReferencedFileNames ()) {
-						string destinationFileName = Path.Combine (destPath, Path.GetFileName (referenceFileName));
-						try {
-							if (destinationFileName != referenceFileName) {
-								File.Copy(referenceFileName, destinationFileName, true);
-								if (File.Exists (referenceFileName + ".mdb"))
-									File.Copy (referenceFileName + ".mdb", destinationFileName + ".mdb", true);
-							}
-						} catch (Exception e) {
-							Runtime.LoggingService.ErrorFormat ("Can't copy reference file from {0} to {1}: {2}", referenceFileName, destinationFileName, e);
-						}
+			string[] deployFiles = GetReferenceDeployFiles (force);
+			
+			foreach (string sourcePath in deployFiles) {
+				string destinationFileName = Path.Combine (destPath, Path.GetFileName (sourcePath));
+				try {
+					if (destinationFileName != sourcePath) {
+						// Make sure the target directory exists
+						if (!Directory.Exists (Path.GetDirectoryName (destinationFileName)))
+							Directory.CreateDirectory (Path.GetDirectoryName (destinationFileName));
+						// Copy the file
+						Runtime.FileService.CopyFile (sourcePath, destinationFileName);
 					}
-				}
-				if (projectReference.ReferenceType == ReferenceType.Project && RootCombine != null) {
-					Project p = RootCombine.FindProject (projectReference.Reference);
-					p.CopyReferencesToOutputPath (destPath, force);
+				} catch (Exception e) {
+					Runtime.LoggingService.ErrorFormat ("Can't copy reference file from {0} to {1}: {2}", (object)sourcePath, (object)destinationFileName, (object)e);
 				}
 			}
 		}
 		
-		void CleanReferencesInOutputPath (string destPath)
+		public string[] GetReferenceDeployFiles (bool force)
 		{
+			ArrayList deployFiles = new ArrayList ();
+
 			foreach (ProjectReference projectReference in ProjectReferences) {
-				if (projectReference.ReferenceType != ReferenceType.Gac) {
+				if ((projectReference.LocalCopy || force) && projectReference.ReferenceType != ReferenceType.Gac) {
 					foreach (string referenceFileName in projectReference.GetReferencedFileNames ()) {
-						string destinationFileName = Path.Combine (destPath, Path.GetFileName (referenceFileName));
-						try {
-							if (destinationFileName != referenceFileName) {
-								File.Delete (destinationFileName);
-								if (File.Exists (destinationFileName + ".mdb"))
-									File.Delete (destinationFileName + ".mdb");
-							}
-						} catch (Exception e) {
-							Runtime.LoggingService.ErrorFormat ("Can't delete reference file {0}: {2}", destinationFileName, e);
-						}
+						deployFiles.Add (referenceFileName);
+						if (File.Exists (referenceFileName + ".mdb"))
+							deployFiles.Add (referenceFileName + ".mdb");
+						if (File.Exists (referenceFileName + ".config"))
+							deployFiles.Add (referenceFileName + ".config");
 					}
 				}
-				if (projectReference.ReferenceType == ReferenceType.Project && RootCombine != null) {
+				if (projectReference.ReferenceType == ReferenceType.Project && projectReference.LocalCopy && RootCombine != null) {
 					Project p = RootCombine.FindProject (projectReference.Reference);
-					p.CleanReferencesInOutputPath (destPath);
+					if (p != null)
+						deployFiles.AddRange (p.GetReferenceDeployFiles (force));
+				}
+			}
+			return (string[]) deployFiles.ToArray (typeof(string));
+		}
+		
+		void CleanReferencesInOutputPath (string destPath)
+		{
+			string[] deployFiles = GetReferenceDeployFiles (true);
+			
+			foreach (string sourcePath in deployFiles) {
+				string destinationFileName = Path.Combine (destPath, Path.GetFileName (sourcePath));
+				try {
+					if (destinationFileName != sourcePath) {
+						if (File.Exists (destinationFileName))
+							Runtime.FileService.DeleteFile (destinationFileName);
+					}
+				} catch (Exception e) {
+					Runtime.LoggingService.ErrorFormat ("Can't delete reference file {0}: {2}", destinationFileName, e);
 				}
 			}
 		}
@@ -249,6 +271,7 @@ namespace MonoDevelop.Projects
 		public override void Dispose()
 		{
 			base.Dispose ();
+			projectFileWatcher.Changed -= new FileSystemEventHandler (OnFileChanged);
 			projectFileWatcher.Dispose ();
 			foreach (ProjectFile file in ProjectFiles) {
 				file.Dispose ();
@@ -282,8 +305,100 @@ namespace MonoDevelop.Projects
 		public void AddFile (ProjectFile projectFile) {
 			ProjectFiles.Add (projectFile);
 		}
-
-		public override void Clean ()
+		
+		public ICompilerResult Build (IProgressMonitor monitor, bool buildReferences)
+		{
+			return InternalBuild (monitor, buildReferences);
+		}
+		
+		internal override ICompilerResult InternalBuild (IProgressMonitor monitor)
+		{
+			return InternalBuild (monitor, true);
+		}
+		
+		ICompilerResult InternalBuild (IProgressMonitor monitor, bool buildReferences)
+		{
+			if (!buildReferences) {
+				if (!NeedsBuilding)
+					return new DefaultCompilerResult (new CompilerResults (null), "");
+					
+				try {
+					monitor.BeginTask (GettextCatalog.GetString ("Building Project: {0} ({1})", Name, ActiveConfiguration.Name), 1);
+					
+					// This will end calling OnBuild ()
+					return Services.ProjectService.ExtensionChain.Build (monitor, this);
+					
+				} finally {
+					monitor.EndTask ();
+				}
+			}
+				
+			// Get a list of all projects that need to be built (including this),
+			// and build them in the correct order
+			
+			CombineEntryCollection referenced = new CombineEntryCollection ();
+			GetReferencedProjects (referenced, this);
+			
+			referenced = Combine.TopologicalSort (referenced);
+			
+			CompilerResults cres = new CompilerResults (null);
+			
+			int builds = 0;
+			int failedBuilds = 0;
+				
+			monitor.BeginTask (null, referenced.Count);
+			foreach (Project p in referenced) {
+				if (p.NeedsBuilding) {
+					ICompilerResult res = p.Build (monitor, false);
+					cres.Errors.AddRange (res.CompilerResults.Errors);
+					builds++;
+					if (res.ErrorCount > 0) {
+						failedBuilds = 1;
+						break;
+					}
+				}
+				monitor.Step (1);
+				if (monitor.IsCancelRequested)
+					break;
+			}
+			monitor.EndTask ();
+			return new DefaultCompilerResult (cres, "", builds, failedBuilds);
+		}
+		
+		protected internal override ICompilerResult OnBuild (IProgressMonitor monitor)
+		{
+			// create output directory, if not exists
+			AbstractProjectConfiguration conf = ActiveConfiguration as AbstractProjectConfiguration;
+			if (conf == null)
+				return null;
+			string outputDir = conf.OutputDirectory;
+			try {
+				DirectoryInfo directoryInfo = new DirectoryInfo(outputDir);
+				if (!directoryInfo.Exists) {
+					directoryInfo.Create();
+				}
+			} catch (Exception e) {
+				throw new ApplicationException("Can't create project output directory " + outputDir + " original exception:\n" + e.ToString());
+			}
+		
+			Runtime.StringParserService.Properties["Project"] = Name;
+			
+			monitor.Log.WriteLine (GettextCatalog.GetString ("Performing main compilation..."));
+			ICompilerResult res = DoBuild (monitor);
+			
+			isDirty = false;
+			
+			if (res != null) {
+				string errorString = GettextCatalog.GetPluralString("{0} error", "{0} errors", res.ErrorCount, res.ErrorCount);
+				string warningString = GettextCatalog.GetPluralString("{0} warning", "{0} warnings", res.WarningCount, res.WarningCount);
+			
+				monitor.Log.WriteLine(GettextCatalog.GetString("Build complete -- ") + errorString + ", " + warningString);
+			}
+			
+			return res;
+		}
+		
+		protected internal override void OnClean (IProgressMonitor monitor)
 		{
 			isDirty = true;
 			
@@ -291,9 +406,9 @@ namespace MonoDevelop.Projects
 			string file = GetOutputFileName ();
 			if (file != null) {
 				if (File.Exists (file))
-					File.Delete (file);
+					Runtime.FileService.DeleteFile (file);
 				if (File.Exists (file + ".mdb"))
-					File.Delete (file + ".mdb");
+					Runtime.FileService.DeleteFile (file + ".mdb");
 			}
 
 			// Delete referenced assemblies
@@ -302,94 +417,12 @@ namespace MonoDevelop.Projects
 				CleanReferencesInOutputPath (config.OutputDirectory);
 		}
 		
-		public override ICompilerResult Build (IProgressMonitor monitor)
-		{
-			return Build (monitor, true);
-		}
-		
-		public virtual ICompilerResult Build (IProgressMonitor monitor, bool buildReferences)
-		{
-			if (buildReferences)
-			{
-				CombineEntryCollection referenced = new CombineEntryCollection ();
-				GetReferencedProjects (referenced, this);
-				
-				referenced = Combine.TopologicalSort (referenced);
-				
-				CompilerResults cres = new CompilerResults (null);
-				
-				int builds = 0;
-				int failedBuilds = 0;
-					
-				monitor.BeginTask (null, referenced.Count);
-				foreach (Project p in referenced) {
-					ICompilerResult res = p.Build (monitor, false);
-					cres.Errors.AddRange (res.CompilerResults.Errors);
-					monitor.Step (1);
-					builds++;
-					if (res.ErrorCount > 0) {
-						failedBuilds = 1;
-						break;
-					}
-				}
-				monitor.EndTask ();
-				return new DefaultCompilerResult (cres, "", builds, failedBuilds);
-			}
-			
-			if (!NeedsBuilding)
-				return new DefaultCompilerResult (new CompilerResults (null), "");
-			
-			try {
-			
-				IBuildStep[] steps = BuildPipeline;
-				
-				monitor.BeginTask (String.Format (GettextCatalog.GetString ("Building Project: {0} Configuration: {1}"), Name, ActiveConfiguration.Name), steps.Length);
-				
-				Runtime.StringParserService.Properties["Project"] = Name;
-				
-				ICompilerResult res = null;
-				
-				foreach (IBuildStep step in steps) {
-					ICompilerResult sres = step.Build (monitor, this);
-					if (sres != null) {
-						if (res != null) {
-							CompilerResults cres = new CompilerResults (null);
-							cres.Errors.AddRange (res.CompilerResults.Errors);
-							cres.Errors.AddRange (sres.CompilerResults.Errors);
-							res = new DefaultCompilerResult (cres, res.CompilerOutput + "\n" + sres.CompilerOutput);
-						} else
-							res = sres;
-					}
-					monitor.Step (1);
-				}
-				
-				isDirty = false;
-				
-				if (res != null) {
-					string errorString = GettextCatalog.GetPluralString("{0} error", "{0} errors", res.ErrorCount, res.ErrorCount);
-					string warningString = GettextCatalog.GetPluralString("{0} warning", "{0} warnings", res.WarningCount, res.WarningCount);
-				
-					monitor.Log.WriteLine(GettextCatalog.GetString("Build complete -- ") + errorString + ", " + warningString);
-				}
-				
-				return res;
-			} finally {
-				monitor.EndTask ();
-			}
-		}
-		
-		protected virtual IBuildStep[] BuildPipeline {
-			get {
-				return (IBuildStep[]) Runtime.AddInService.GetTreeItems ("/SharpDevelop/Workbench/BuildPipeline", typeof(IBuildStep));
-			}
-		}
-		
 		
 		void GetReferencedProjects (CombineEntryCollection referenced, Project project)
 		{
 			if (referenced.Contains (project)) return;
 			
-			if (NeedsBuilding)
+			if (project.NeedsBuilding)
 				referenced.Add (project);
 
 			foreach (ProjectReference pref in project.ProjectReferences) {
@@ -404,78 +437,14 @@ namespace MonoDevelop.Projects
 			}
 		}
 		
-		protected virtual void DoPreBuild (IProgressMonitor monitor)
-		{
-			AbstractProjectConfiguration conf = ActiveConfiguration as AbstractProjectConfiguration;
-				
-			// create output directory, if not exists
-			string outputDir = conf.OutputDirectory;
-			try {
-				DirectoryInfo directoryInfo = new DirectoryInfo(outputDir);
-				if (!directoryInfo.Exists) {
-					directoryInfo.Create();
-				}
-			} catch (Exception e) {
-				throw new ApplicationException("Can't create project output directory " + outputDir + " original exception:\n" + e.ToString());
-			}
-			
-			if (conf != null && conf.ExecuteBeforeBuild != "" && File.Exists(conf.ExecuteBeforeBuild)) {
-				monitor.Log.WriteLine (String.Format (GettextCatalog.GetString ("Executing: {0}"), conf.ExecuteBeforeBuild));
-				ProcessStartInfo ps = GetBuildTaskStartInfo(conf.ExecuteBeforeBuild);
-				Process process = new Process();
-				process.StartInfo = ps;
-				process.Start();
-				monitor.Log.Write (process.StandardOutput.ReadToEnd());
-				monitor.Log.WriteLine ();
-			}
-		}
-		
 		protected virtual ICompilerResult DoBuild (IProgressMonitor monitor)
 		{
 			return new DefaultCompilerResult (new CompilerResults (null), "");
 		}
 		
-		protected virtual void DoPostBuild (IProgressMonitor monitor)
+		protected internal override void OnExecute (IProgressMonitor monitor, ExecutionContext context)
 		{
-			AbstractProjectConfiguration conf = ActiveConfiguration as AbstractProjectConfiguration;
-
-			if (conf != null && conf.ExecuteAfterBuild != "" && File.Exists(conf.ExecuteAfterBuild)) {
-				monitor.Log.WriteLine ();
-				monitor.Log.WriteLine (String.Format (GettextCatalog.GetString ("Executing: {0}"), conf.ExecuteAfterBuild));
-				ProcessStartInfo ps = GetBuildTaskStartInfo(conf.ExecuteAfterBuild);
-				Process process = new Process();
-				process.StartInfo = ps;
-				process.Start();
-				monitor.Log.Write (process.StandardOutput.ReadToEnd());
-			}
-		}
-		
-		private ProcessStartInfo GetBuildTaskStartInfo(string file) {
-			ProcessStartInfo ps = new ProcessStartInfo(file);
-			ps.UseShellExecute = false;
-			ps.RedirectStandardOutput = true;
-			ps.WorkingDirectory = BaseDirectory;
-			return ps;
-		}
-		
-		public override void Execute (IProgressMonitor monitor, ExecutionContext context)
-		{
-			AbstractProjectConfiguration configuration = (AbstractProjectConfiguration) ActiveConfiguration;
-				
-			string args = configuration.CommandLineParameters;
-			
-			if (configuration.ExecuteScript != null && configuration.ExecuteScript.Length > 0) {
-				IConsole console;
-				if (configuration.ExternalConsole)
-					console = context.ExternalConsoleFactory.CreateConsole (!configuration.PauseConsoleOutput);
-				else
-					console = context.ConsoleFactory.CreateConsole (!configuration.PauseConsoleOutput);
-
-				ProcessWrapper p = Runtime.ProcessService.StartConsoleProcess (configuration.ExecuteScript, args, BaseDirectory, console, null);
-				p.WaitForOutput ();
-			} else {
-				DoExecute (monitor, context);
-			}
+			DoExecute (monitor, context);
 		}
 		
 		protected virtual void DoExecute (IProgressMonitor monitor, ExecutionContext context)
@@ -487,14 +456,15 @@ namespace MonoDevelop.Projects
 			return null;
 		}
 		
-		public override bool NeedsBuilding {
-			get {
-				if (!isDirty) CheckNeedsBuild ();
-				return isDirty;
-			}
-			set {
-				isDirty = value;
-			}
+		protected internal override bool OnGetNeedsBuilding ()
+		{
+			if (!isDirty) CheckNeedsBuild ();
+			return isDirty;
+		}
+		
+		protected internal override void OnSetNeedsBuilding (bool value)
+		{
+			isDirty = value;
 		}
 		
 		public override string FileName {
@@ -517,7 +487,8 @@ namespace MonoDevelop.Projects
 			}
 			
 			foreach (ProjectFile file in ProjectFiles) {
-				if (file.BuildAction == BuildAction.Exclude) continue;
+				if (file.BuildAction == BuildAction.Exclude || file.BuildAction == BuildAction.Nothing)
+					continue;
 				FileInfo finfo = new FileInfo (file.FilePath);
 				if (finfo.Exists && finfo.LastWriteTime > tim) {
 					isDirty = true;
@@ -532,24 +503,41 @@ namespace MonoDevelop.Projects
 						isDirty = true;
 						return;
 					}
+					DateTime rptime = GetLastWriteTime (rp.GetOutputFileName ());
+					if (rptime == DateTime.MinValue || rptime > tim) {
+						isDirty = true;
+						return;
+					}
 				}
 			}
 		}
 		
 		protected virtual DateTime GetLastBuildTime ()
 		{
-			string file = GetOutputFileName ();
+			return GetLastWriteTime (GetOutputFileName ());
+		}
+
+		DateTime GetLastWriteTime (string file)
+		{
+			if (file == null)
+				return DateTime.MinValue;
+
 			FileInfo finfo = new FileInfo (file);
 			if (!finfo.Exists) return DateTime.MinValue;
 			else return finfo.LastWriteTime;
 		}
-		
+
 		private void UpdateFileWatch()
 		{
 			projectFileWatcher.EnableRaisingEvents = false;
 			projectFileWatcher.Path = BaseDirectory;
 			projectFileWatcher.EnableRaisingEvents = true;
 		}
+
+		internal protected virtual string GetDefaultResourceId (ProjectFile pf)
+		{
+			return Path.GetFileName (pf.Name);
+		}
 		
 		void OnFileChanged (object source, FileSystemEventArgs e)
 		{
@@ -566,6 +554,19 @@ namespace MonoDevelop.Projects
 			}
 
 		}
+		
+		internal protected override StringCollection OnGetExportFiles ()
+		{
+			StringCollection col = base.OnGetExportFiles ();
+			foreach (ProjectFile pf in ProjectFiles) {
+				if (pf.Subtype != Subtype.Directory)
+					col.Add (pf.FilePath);
+			}
+			foreach (ProjectReference pref in ProjectReferences)
+				if (pref.ReferenceType == ReferenceType.Assembly)
+					col.Add (pref.Reference);
+			return col;
+		}
 
  		internal void NotifyFileChangedInProject (ProjectFile file)
 		{
@@ -674,7 +675,7 @@ namespace MonoDevelop.Projects
 		{
 			public ICompilerResult Build (IProgressMonitor monitor, Project project)
 			{
-				monitor.Log.WriteLine (String.Format (GettextCatalog.GetString ("Performing main compilation...")));
+				monitor.Log.WriteLine (GettextCatalog.GetString ("Performing main compilation..."));
 				return project.DoBuild (monitor);
 			}
 			
@@ -684,34 +685,6 @@ namespace MonoDevelop.Projects
 				return project.isDirty;
 			}
 		}
-		
-		class PreBuildStep: IBuildStep
-		{
-			public ICompilerResult Build (IProgressMonitor monitor, Project project)
-			{
-				project.DoPreBuild (monitor);
-				return null;
-			}
-			
-			public bool NeedsBuilding (Project project)
-			{
-				return false;
-			}
-		}
-		
-		class PostBuildStep: IBuildStep
-		{
-			public ICompilerResult Build (IProgressMonitor monitor, Project project)
-			{
-				project.DoPostBuild (monitor);
-				return null;
-			}
-			
-			public bool NeedsBuilding (Project project)
-			{
-				return false;
-			}
-		}
 	}
 	
 	public class UnknownProject: Project
