@@ -147,10 +147,13 @@ namespace AspNetEdit.Editor.UI
 		void changeService_ComponentChanged (object sender, ComponentChangedEventArgs e)
 		{
 			if (!active) return;
-			
-			Control control = e.Component as Control;
+			UpdateRender (e.Component as Control);			
+		}
+		
+		public void UpdateRender (Control control)
+		{
 			if (control == null)
-				throw new InvalidOperationException ("The changed component is not a System.UI.WebControl");
+				throw new InvalidOperationException ("The updated component is not a System.UI.WebControl");
 			
 			string ctext = Document.RenderDesignerControl (control);
 			comm.JSCall (GeckoFunctions.UpdateControl, null, control.UniqueID, ctext);
@@ -406,18 +409,18 @@ namespace AspNetEdit.Editor.UI
 		
 		///<summary>
 		/// Name:	DeserializeAndAdd
-		///			Deserialises a fragment of ASP.NET code into a Gecko designer document fragment
-		///			and adds the controls, directives etc to the host.
+		///			Handles any ASP.NET code that gets pasted into the designer.
 		/// Arguments:
 		///		string designerDocumentFragment:	the ASP.NET document fragment
-		/// Returns:	the deserialised document
+		/// Returns: none
 		///</summary>
 		private string JSDeserializeAndAdd (string[] args)
 		{
 			if (args.Length != 1)
 				throw new InvalidJSArgumentException ("DeserializeAndAdd", -1);
 						
-			return host.RootDocument.DeserializeAndAdd (args [0]);
+			host.RootDocument.InsertFragment (args [0]);
+			return string.Empty;
 		}
 
 		#endregion
@@ -433,7 +436,7 @@ namespace AspNetEdit.Editor.UI
 			/// 	string content:	The HTML content of the control
 			/// Returns: none
 			///</summary>
-			public static readonly string AddControl = "AddControl";
+			public static readonly string AddControl = "JSCall_AddControl";
 			
 			///<summary>
 			/// Updates the design-time HTML of a control to the document
@@ -442,7 +445,7 @@ namespace AspNetEdit.Editor.UI
 			/// 	string content:	The HTML content of the control
 			/// Returns: none
 			///</summary>
-			public static readonly string UpdateControl = "UpdateControl";
+			public static readonly string UpdateControl = "JSCall_UpdateControl";
 
 			///<summary>
 			/// Removes a control from the document
@@ -450,7 +453,7 @@ namespace AspNetEdit.Editor.UI
 			/// 	string id:		the unique ID of the control.
 			/// Returns: none
 			///</summary>
-			public static readonly string RemoveControl = "RemoveControl";
+			public static readonly string RemoveControl = "JSCall_RemoveControl";
 			
 			///<summary>
 			/// Selects a control
@@ -458,7 +461,7 @@ namespace AspNetEdit.Editor.UI
 			/// 	string id:		the unique ID of the control, or empty to clear selection.
 			/// Returns: none
 			///</summary>
-			public static readonly string SelectControl = "SelectControl";
+			public static readonly string SelectControl = "JSCall_SelectControl";
 			
 			///<summary>
 			/// Replaces the currently loaded document
@@ -466,14 +469,14 @@ namespace AspNetEdit.Editor.UI
 			/// 	string document:	the document text, with placeholder'd controls.
 			/// Returns: none
 			///</summary>
-			public static readonly string LoadPage = "LoadPage";
+			public static readonly string LoadPage = "JSCall_LoadPage";
 			
 			///<summary>
 			/// Replaces the currently loaded document
 			/// Args: none
 			/// Returns: none
 			///</summary>
-			public static readonly string GetPage = "GetPage";
+			public static readonly string GetPage = "JSCall_GetPage";
 			
 			///<summary>
 			/// Passes a simple command to Gecko
@@ -481,7 +484,7 @@ namespace AspNetEdit.Editor.UI
 			///		string command:		Use the enum EditorCommand
 			/// Returns: none
 			///</summary>
-			public static readonly string DoCommand = "DoCommand";
+			public static readonly string DoCommand = "JSCall_DoCommand";
 			
 			///<summary>
 			/// Inserts a document fragment. Host should have deserialised it.
@@ -489,7 +492,7 @@ namespace AspNetEdit.Editor.UI
 			///		string fragment:		The document fragment
 			/// Returns: none
 			///</summary>
-			public static readonly string InsertFragment = "InsertFragment";
+			public static readonly string InsertFragment = "JSCall_InsertFragment";
 		}
 		
 		
