@@ -8,6 +8,7 @@
 using System;
 using System.IO;
 using System.Collections;
+using System.Collections.Specialized;
 using System.Drawing;
 
 using MonoDevelop.Core;
@@ -53,8 +54,6 @@ namespace CSharpBinding.Parser
 		
 		bool showStatic = false;
 		
-		bool inNew = false;
-		
 		public bool ShowStatic {
 			get {
 				return showStatic;
@@ -157,10 +156,35 @@ namespace CSharpBinding.Parser
 			if (type.ArrayDimensions != null && type.ArrayDimensions.Length > 0) {
 				type = new ReturnType("System.Array");
 			}
-			Console.WriteLine("Here: Type is " + type.FullyQualifiedName);
+			//Console.WriteLine("Here: Type is " + type.FullyQualifiedName);
 			return type;
 		}
 		
+		public IClass GetCallingClass (int line, int col, string fileName, bool onlyClassDeclaration)
+		{
+			IParseInformation parseInfo = parserContext.GetParseInformation (fileName);
+			ICSharpCode.NRefactory.Parser.AST.CompilationUnit fileCompilationUnit = parseInfo.MostRecentCompilationUnit.Tag as ICSharpCode.NRefactory.Parser.AST.CompilationUnit;
+			if (fileCompilationUnit == null)
+				return null;
+
+			CSharpVisitor cSharpVisitor = new CSharpVisitor();
+			currentUnit = (ICompilationUnit)cSharpVisitor.Visit(fileCompilationUnit, null);
+			
+			currentFile = fileName;
+		
+			this.caretLine = line;
+			this.caretColumn = col;
+			
+			callingClass = GetInnermostClass();
+			if (callingClass == null)
+				return null;
+				
+			if (onlyClassDeclaration && GetMethod () != null)
+				return null;
+			
+			return callingClass;
+		}
+
 		public IClass ResolveExpressionType (ICSharpCode.NRefactory.Parser.AST.CompilationUnit fileCompilationUnit, Expression expr, int line, int col)
 		{
 			CSharpVisitor cSharpVisitor = new CSharpVisitor();
@@ -175,9 +199,10 @@ namespace CSharpBinding.Parser
 			lookupTableVisitor = new LookupTableVisitor (StringComparer.InvariantCulture);
 			lookupTableVisitor.Visit (fileCompilationUnit, null);
 			TypeVisitor typeVisitor = new TypeVisitor (this);
+			
 			IReturnType type = expr.AcceptVisitor (typeVisitor, null) as IReturnType;
 			if (type != null)
-				return SearchType (type.FullyQualifiedName, currentUnit);
+				return SearchType (type, currentUnit);
 			else
 				return null;
 		}
@@ -212,6 +237,13 @@ namespace CSharpBinding.Parser
 			
 			LanguageItemVisitor itemVisitor = new LanguageItemVisitor (this);
 			ILanguageItem item = expr.AcceptVisitor (itemVisitor, null) as ILanguageItem;
+			
+			if (item == null && expr is BinaryOperatorExpression && !id.EndsWith ("()")) {
+				// The expression parser does not correctly parse individual generic type names.
+				// Try resolving again but using a more complex expression.
+				return ResolveIdentifier (fileCompilationUnit, id + "()", line, col);
+			}
+			
 			return item;
 		}
 
@@ -231,12 +263,7 @@ namespace CSharpBinding.Parser
 				return null;
 			} catch (Exception) {
 			}
-			if (expression.StartsWith("new ")) {
-				inNew = true;
-				expression = expression.Substring(4);
-			} else {
-				inNew = false;
-			}
+			
 			if (expression.StartsWith("using ")) {
 				// expression[expression.Length - 1] != '.'
 				// the period that causes this Resove() is not part of the expression
@@ -267,7 +294,7 @@ namespace CSharpBinding.Parser
 			if (type == null)
 				return null;
 
-			IClass returnClass = SearchType (type.FullyQualifiedName, currentUnit);
+			IClass returnClass = SearchType (type, currentUnit);
 			if (returnClass == null) {
 				// Try if type is Namespace:
 				string n = SearchNamespace(type.FullyQualifiedName, currentUnit);
@@ -287,10 +314,10 @@ namespace CSharpBinding.Parser
 			//Console.WriteLine("Returning Result!");
 			if (returnClass.FullyQualifiedName == "System.Void")
 				return null;
-			return new ResolveResult(returnClass, ListMembers(new LanguageItemCollection(), returnClass));
+			return new ResolveResult(returnClass, ListMembers(new LanguageItemCollection(), returnClass, returnClass));
 		}
 		
-		LanguageItemCollection ListMembers (LanguageItemCollection members, IClass curType)
+		LanguageItemCollection ListMembers (LanguageItemCollection members, IClass qualifierClass, IClass curType)
 		{
 //			Console.WriteLine("LIST MEMBERS!!!");
 //			Console.WriteLine("showStatic = " + showStatic);
@@ -301,58 +328,45 @@ namespace CSharpBinding.Parser
 //			Console.WriteLine(curType.Fields.Count + " fields");
 			if (showStatic) {
 				foreach (IClass c in curType.InnerClasses) {
-					if (IsAccessible(curType, c)) {
+					if (IsAccessible(qualifierClass, curType, c)) {
 						members.Add(c);
-//						Console.WriteLine("Member added");
 					}
 				}
 			}
 			foreach (IProperty p in curType.Properties) {
-				if (MustBeShowen(curType, p)) {
+				if (MustBeShowen (qualifierClass, curType, p)) {
 					members.Add(p);
-//					Console.WriteLine("Member added");
 				}
 			}
-//			Console.WriteLine("ADDING METHODS!!!");
 			foreach (IMethod m in curType.Methods) {
-//				Console.WriteLine("Method : " + m);
-				if (MustBeShowen(curType, m)) {
+				if (MustBeShowen (qualifierClass, curType, m)) {
 					members.Add(m);
-//					Console.WriteLine("Member added");
 				}
 			}
 			
 			foreach (IEvent e in curType.Events) {
-				if (MustBeShowen(curType, e)) {
+				if (MustBeShowen (qualifierClass, curType, e)) {
 					members.Add(e);
-//					Console.WriteLine("Member added");
 				}
 			}
 			foreach (IField f in curType.Fields) {
-				if (MustBeShowen(curType, f)) {
+				if (MustBeShowen (qualifierClass, curType, f)) {
 					members.Add(f);
-//					Console.WriteLine("Member added");
-				} else {
-					//// enum fields must be shown here if present
-					if (curType.ClassType == ClassType.Enum) {
-						if (IsAccessible(curType,f)) members.Add(f);
-//						Console.WriteLine("Member {0} added", f.FullyQualifiedName);
-					}
 				}
 			}
 //			Console.WriteLine("ClassType = " + curType.ClassType);
 			if (curType.ClassType == ClassType.Interface && !showStatic) {
 				foreach (IReturnType s in curType.BaseTypes) {
-					IClass baseClass = parserContext.GetClass (s.FullyQualifiedName, true, true);
+					IClass baseClass = parserContext.GetClass (s.FullyQualifiedName, s.GenericArguments, true, true);
 					if (baseClass != null && baseClass.ClassType == ClassType.Interface) {
-						ListMembers(members, baseClass);
+						ListMembers (members, qualifierClass, baseClass);
 					}
 				}
 			} else {
 				IClass baseClass = BaseClass(curType);
 				if (baseClass != null) {
 //					Console.WriteLine("Base Class = " + baseClass.FullyQualifiedName);
-					ListMembers(members, baseClass);
+					ListMembers (members, qualifierClass, baseClass);
 				}
 			}
 //			Console.WriteLine("listing finished");
@@ -362,7 +376,7 @@ namespace CSharpBinding.Parser
 		public IClass BaseClass(IClass curClass)
 		{
 			foreach (IReturnType s in curClass.BaseTypes) {
-				IClass baseClass = parserContext.GetClass (s.FullyQualifiedName, true, true);
+				IClass baseClass = parserContext.GetClass (s.FullyQualifiedName, s.GenericArguments, true, true);
 				if (baseClass != null && baseClass.ClassType != ClassType.Interface) {
 					return baseClass;
 				}
@@ -370,20 +384,7 @@ namespace CSharpBinding.Parser
 			return null;
 		}
 		
-		bool InStatic()
-		{
-			IProperty property = GetProperty();
-			if (property != null) {
-				return property.IsStatic;
-			}
-			IMethod method = GetMethod();
-			if (method != null) {
-				return method.IsStatic;
-			}
-			return false;
-		}
-		
-		bool IsAccessible(IClass c, IDecoration member)
+		bool IsAccessible (IClass qualifier, IClass c, IDecoration member)
 		{
 //			Console.WriteLine("member.Modifiers = " + member.Modifiers);
 			if ((member.Modifiers & ModifierEnum.Internal) == ModifierEnum.Internal) {
@@ -393,7 +394,7 @@ namespace CSharpBinding.Parser
 //				Console.WriteLine("IsAccessible");
 				return true;
 			}
-			if ((member.Modifiers & ModifierEnum.Protected) == ModifierEnum.Protected && IsClassInInheritanceTree(c, callingClass)) {
+			if ((member.Modifiers & ModifierEnum.Protected) == ModifierEnum.Protected && IsClassInInheritanceTree (callingClass, qualifier)) {
 //				Console.WriteLine("IsAccessible");
 				return true;
 			}
@@ -403,16 +404,18 @@ namespace CSharpBinding.Parser
 			return c.FullyQualifiedName == callingClass.FullyQualifiedName;
 		}
 		
-		bool MustBeShowen(IClass c, IDecoration member)
+		bool MustBeShowen (IClass qualifierClass, IClass c, IDecoration member)
 		{
-//			Console.WriteLine("member:" + member.Modifiers);
-			if ((!showStatic &&  ((member.Modifiers & ModifierEnum.Static) == ModifierEnum.Static)) ||
-			    ( showStatic && !((member.Modifiers & ModifierEnum.Static) == ModifierEnum.Static))) {
+			if (c.ClassType == ClassType.Enum && (member is IField))
+				return showStatic;
+			bool memStatic = member.IsStatic || ((member is IField) && member.IsLiteral);
+			if ((showStatic != memStatic) ||
+			    (showStatic && member.IsStatic && member.IsSpecialName && member.Name.StartsWith ("op_"))
+			    ) {
 				//// enum type fields are not shown here - there is no info in member about enum field
 				return false;
 			}
-//			Console.WriteLine("Testing Accessibility");
-			return IsAccessible(c, member);
+			return IsAccessible (qualifierClass, c, member);
 		}
 		
 		public ArrayList SearchMethod(IReturnType type, string memberName)
@@ -422,9 +425,9 @@ namespace CSharpBinding.Parser
 			}
 			IClass curType;
 			if (type.ArrayDimensions != null && type.ArrayDimensions.Length > 0) {
-				curType = SearchType("System.Array", null);
+				curType = SearchType ("System.Array", null, null);
 			} else {
-				curType = SearchType(type.FullyQualifiedName, null);
+				curType = SearchType (type, null);
 				if (curType == null) {
 					return new ArrayList();
 				}
@@ -432,18 +435,23 @@ namespace CSharpBinding.Parser
 			return SearchMethod(new ArrayList(), curType, memberName);
 		}
 		
-		ArrayList SearchMethod(ArrayList methods, IClass curType, string memberName)
+		ArrayList SearchMethod (ArrayList methods, IClass curType, string memberName)
+		{
+			return SearchMethod (methods, curType, curType, memberName);
+		}
+		
+		ArrayList SearchMethod (ArrayList methods, IClass qualifierClass, IClass curType, string memberName)
 		{
 			foreach (IMethod m in curType.Methods) {
 				if (m.Name == memberName &&
-				    MustBeShowen(curType, m) &&
+				    MustBeShowen (qualifierClass, curType, m) &&
 				    !((m.Modifiers & ModifierEnum.Override) == ModifierEnum.Override)) {
 					methods.Add(m);
 				}
 			}
 			IClass baseClass = BaseClass(curType);
 			if (baseClass != null) {
-				return SearchMethod(methods, baseClass, memberName);
+				return SearchMethod(methods, qualifierClass, baseClass, memberName);
 			}
 			showStatic = false;
 			return methods;
@@ -451,23 +459,23 @@ namespace CSharpBinding.Parser
 		
 		public ArrayList SearchIndexer(IReturnType type)
 		{
-			IClass curType = SearchType(type.FullyQualifiedName, null);
+			IClass curType = SearchType (type, null);
 			if (curType != null) {
-				return SearchIndexer(new ArrayList(), curType);
+				return SearchIndexer(new ArrayList(), curType, curType);
 			}
 			return new ArrayList();
 		}
 		
-		public ArrayList SearchIndexer(ArrayList indexer, IClass curType)
+		public ArrayList SearchIndexer (ArrayList indexer, IClass qualifierClass, IClass curType)
 		{
 			foreach (IIndexer i in curType.Indexer) {
-				if (MustBeShowen(curType, i) && !((i.Modifiers & ModifierEnum.Override) == ModifierEnum.Override)) {
+				if (MustBeShowen(qualifierClass, curType, i) && !((i.Modifiers & ModifierEnum.Override) == ModifierEnum.Override)) {
 					indexer.Add(i);
 				}
 			}
 			IClass baseClass = BaseClass(curType);
 			if (baseClass != null) {
-				return SearchIndexer(indexer, baseClass);
+				return SearchIndexer (indexer, qualifierClass, baseClass);
 			}
 			showStatic = false;
 			return indexer;
@@ -519,7 +527,7 @@ namespace CSharpBinding.Parser
 			if (type == null || memberName == null || memberName == "")
 				return false;
 			
-			curType = SearchType (type.FullyQualifiedName, currentUnit);
+			curType = SearchType (type, currentUnit);
 			if (curType == null)
 				return false;
 
@@ -527,11 +535,18 @@ namespace CSharpBinding.Parser
 				return false;
 
 			if (type.ArrayDimensions != null && type.ArrayDimensions.Length > 0)
-				curType = SearchType("System.Array", null);
-
+				curType = SearchType ("System.Array", null, null);
+				
+			return SearchClassMember (curType, curType, memberName, includeMethods, out curType, out member);
+		}
+		
+		bool SearchClassMember (IClass qualifierClass, IClass curType, string memberName, bool includeMethods, out IClass resultType, out IDecoration member)
+		{
+			resultType = curType;
+			
 			if (curType.ClassType == ClassType.Enum) {
 				foreach (IField f in curType.Fields) {
-					if (f.Name == memberName && MustBeShowen(curType, f)) {
+					if (f.Name == memberName && MustBeShowen (qualifierClass, curType, f)) {
 						showStatic = false;
 						member = f; // enum members have the type of the enum
 						return true;
@@ -540,28 +555,28 @@ namespace CSharpBinding.Parser
 			}
 			if (showStatic) {
 				foreach (IClass c in curType.InnerClasses) {
-					if (c.Name == memberName && IsAccessible(curType, c)) {
+					if (c.Name == memberName && IsAccessible (qualifierClass, curType, c)) {
 						member = c;
 						return true;
 					}
 				}
 			}
 			foreach (IProperty p in curType.Properties) {
-				if (p.Name == memberName && MustBeShowen(curType, p)) {
+				if (p.Name == memberName && MustBeShowen (qualifierClass, curType, p)) {
 					showStatic = false;
 					member = p;
 					return true;
 				}
 			}
 			foreach (IField f in curType.Fields) {
-				if (f.Name == memberName && MustBeShowen(curType, f)) {
+				if (f.Name == memberName && MustBeShowen (qualifierClass, curType, f)) {
 					showStatic = false;
 					member = f;
 					return true;
 				}
 			}
 			foreach (IEvent e in curType.Events) {
-				if (e.Name == memberName && MustBeShowen(curType, e)) {
+				if (e.Name == memberName && MustBeShowen (qualifierClass, curType, e)) {
 					showStatic = false;
 					member = e;
 					return true;
@@ -569,18 +584,25 @@ namespace CSharpBinding.Parser
 			}
 			if (includeMethods) {
 				foreach (IMethod m in curType.Methods) {
-					if (m.Name == memberName && MustBeShowen(curType, m)) {
+					if (m.Name == memberName && MustBeShowen (qualifierClass, curType, m)) {
 						showStatic = false;
 						member = m;
 						return true;
 					}
 				}
 			}
+			
+			// Don't look in interfaces, unless the base type is already an interface.
+			
 			foreach (IReturnType baseType in curType.BaseTypes) {
-				IClass c = parserContext.GetClass (baseType.FullyQualifiedName, true, true);
-				if (c != null)
-					return SearchClassMember (new ReturnType(c.FullyQualifiedName), memberName, includeMethods, out curType, out member);
+				IClass c = parserContext.GetClass (baseType.FullyQualifiedName, baseType.GenericArguments, true, true);
+				if (c != null && (c.ClassType != ClassType.Interface || curType.ClassType == ClassType.Interface)) {
+					if (SearchClassMember (qualifierClass, c, memberName, includeMethods, out resultType, out member))
+						return true;
+				}
 			}
+			
+			member = null;
 			return false;
 		}
 		
@@ -617,7 +639,8 @@ namespace CSharpBinding.Parser
 			
 			foreach (LocalLookupVariable v in variables) {
 				if (IsInside(new Point(caretColumn, caretLine), v.StartPos, v.EndPos)) {
-					IClass c = SearchType (v.TypeRef.SystemType, CompilationUnit);
+					// The call to GetFullTypeName will return a type name with generics decoration
+					IClass c = SearchType (ReturnType.GetFullTypeName (v.TypeRef), null, CompilationUnit);
 					DefaultRegion reg = new DefaultRegion (v.StartPos, v.EndPos);
 					reg.FileName = currentFile;
 					return new LocalVariable (name, new ReturnType (v.TypeRef, c), "", reg);
@@ -777,10 +800,13 @@ namespace CSharpBinding.Parser
 		IMethod GetMethod()
 		{
 			foreach (IMethod method in callingClass.Methods) {
-				if (method.BodyRegion != null && method.BodyRegion.IsInside(caretLine, caretColumn)) {
+				if (method.Region != null && method.Region.IsInside (caretLine, caretColumn))
+					return method;
+				
+				if (method.BodyRegion != null && method.BodyRegion.IsInside(caretLine, caretColumn))
 					return method;
-				}
 			}
+			
 			return null;
 		}
 		
@@ -799,14 +825,15 @@ namespace CSharpBinding.Parser
 		IParameter SearchMethodParameter(string parameter)
 		{
 			IMethod method = GetMethod();
-			if (method == null) {
+			if (method == null)
 				return null;
-			}
+			
 			foreach (IParameter p in method.Parameters) {
 				if (p.Name == parameter) {
 					return p;
 				}
 			}
+			
 			return null;
 		}
 		
@@ -815,6 +842,18 @@ namespace CSharpBinding.Parser
 		/// </remarks>
 		public string SearchNamespace(string name, ICompilationUnit unit)
 		{
+			// If the name matches an alias, try using the alias first.
+			if (unit != null) {
+				string aliasResult = FindAlias (name, unit);
+				if (aliasResult != null) {
+					// Don't provide the compilation unit when trying to resolve the alias,
+					// since aliases are not affected by other 'using' directives.
+					string ns = SearchNamespace (aliasResult, null);
+					if (ns != null)
+						return ns;
+				}
+			}
+			
 			if (parserContext.NamespaceExists (name)) {
 				return name;
 			}
@@ -832,52 +871,91 @@ namespace CSharpBinding.Parser
 			return null;
 		}
 		
+		public IClass SearchType (IReturnType type, ICompilationUnit unit)
+		{
+			return SearchType (type.FullyQualifiedName, type.GenericArguments, unit);
+		}
+		
 		/// <remarks>
 		/// use the usings and the name of the namespace to find a class
 		/// </remarks>
-		public IClass SearchType(string name, ICompilationUnit unit)
+		public IClass SearchType (string name, ReturnTypeList genericArguments, ICompilationUnit unit)
 		{
 //			Console.WriteLine("Searching Type " + name);
-			if (name == null || name == String.Empty) {
-//				Console.WriteLine("No Name!");
+			if (name == null || name == String.Empty)
 				return null;
-			}
+			
 			IClass c;
-			c = parserContext.GetClass (name);
-			if (c != null) {
-//				Console.WriteLine("Found!");
-				return c;
+			
+			// If the name matches an alias, try using the alias first.
+			if (unit != null) {
+				string aliasResult = FindAlias (name, unit);
+				if (aliasResult != null) {
+					// Don't provide the compilation unit when trying to resolve the alias,
+					// since aliases are not affected by other 'using' directives.
+					c = SearchType (aliasResult, genericArguments, null);
+					if (c != null)
+						return c;
+				}
 			}
+			
+			// Look for an exact match
+			
+			c = parserContext.GetClass (name, genericArguments);
+			if (c != null)
+				return c;
+				
+
+			// The enclosing namespace has preference over the using directives.
+			// Check it now.
+
+			if (callingClass != null)
+			{
+				string fullname = callingClass.FullyQualifiedName;
+				string[] namespaces = fullname.Split(new char[] {'.'});
+				string curnamespace = "";
+				int i = 0;
+				
+				do {
+					curnamespace += namespaces[i] + '.';
+					c = parserContext.GetClass (curnamespace + name, genericArguments);
+					if (c != null) {
+						return c;
+					}
+					i++;
+				}
+				while (i < namespaces.Length);
+			}
+			
+			// Now try to find the class using the included namespaces
+			
 			if (unit != null) {
 				foreach (IUsing u in unit.Usings) {
 					if (u != null && (u.Region == null || u.Region.IsInside(caretLine, caretColumn))) {
-//						Console.WriteLine("In UsingRegion");
-						c = parserContext.SearchType (u, name);
-						if (c != null) {
-//							Console.WriteLine("SearchType Successfull!!!");
+						c = parserContext.SearchType (u, name, genericArguments);
+						if (c != null)
 							return c;
-						}
 					}
 				}
 			}
-			if (callingClass == null) {
+			
+			return null;
+		}
+		
+		string FindAlias (string name, ICompilationUnit unit)
+		{
+			// If the name matches an alias, try using the alias first.
+			if (unit == null)
 				return null;
-			}
-			string fullname = callingClass.FullyQualifiedName;
-			string[] namespaces = fullname.Split(new char[] {'.'});
-			string curnamespace = "";
-			int i = 0;
-			
-			do {
-				curnamespace += namespaces[i] + '.';
-				c = parserContext.GetClass (curnamespace + name);
-				if (c != null) {
-					return c;
+				
+			foreach (IUsing u in unit.Usings) {
+				if (u != null && (u.Region == null || u.Region.IsInside(caretLine, caretColumn))) {
+					foreach (DictionaryEntry e in u.Aliases) {
+						if ((string)e.Value == name)
+							return (string)e.Key;
+					}
 				}
-				i++;
 			}
-			while (i < namespaces.Length);
-			
 			return null;
 		}
 		
@@ -893,7 +971,7 @@ namespace CSharpBinding.Parser
 				return true;
 			}
 			foreach (IReturnType baseClass in c.BaseTypes) {
-				IClass bc = parserContext.GetClass (baseClass.FullyQualifiedName, true, true);
+				IClass bc = parserContext.GetClass (baseClass.FullyQualifiedName, baseClass.GenericArguments, true, true);
 				if (IsClassInInheritanceTree(possibleBaseClass, bc)) {
 					return true;
 				}
@@ -946,7 +1024,7 @@ namespace CSharpBinding.Parser
 			ClassCollection classes = new ClassCollection();
 			if (currentUnit != null) {
 				foreach (IClass c in currentUnit.Classes) {
-					if (c != null && c.Region != null && c.BodyRegion.IsInside(caretLine, caretColumn)) {
+					if (c != null && c.BodyRegion != null && c.BodyRegion.IsInside(caretLine, caretColumn)) {
 						if (c != GetInnermostClass()) {
 							GetOuterClasses(classes, c);
 							classes.Add(GetResolvedClass (c));
@@ -963,7 +1041,7 @@ namespace CSharpBinding.Parser
 		{
 			if (curClass != null) {
 				foreach (IClass c in curClass.InnerClasses) {
-					if (c != null && c.Region != null && c.BodyRegion.IsInside(caretLine, caretColumn)) {
+					if (c != null && c.BodyRegion != null && c.BodyRegion.IsInside(caretLine, caretColumn)) {
 						if (c != GetInnermostClass()) {
 							GetOuterClasses(classes, c);
 							classes.Add(GetResolvedClass (c));
@@ -980,7 +1058,7 @@ namespace CSharpBinding.Parser
 			return parserContext.GetClass (cls.FullyQualifiedName);
 		}
 
-		public LanguageItemCollection IsAsResolve (string expression, int caretLine, int caretColumn, string fileName, string fileContent)
+		public LanguageItemCollection IsAsResolve (string expression, int caretLine, int caretColumn, string fileName, string fileContent, bool excludeInterfaces)
 		{
 			LanguageItemCollection result = new LanguageItemCollection ();
 			this.caretLine = caretLine;
@@ -1023,15 +1101,39 @@ namespace CSharpBinding.Parser
 			if (type.ArrayDimensions != null && type.ArrayDimensions.Length > 0)
 				type = new ReturnType ("System.Array");
 
-			IClass returnClass = SearchType (type.FullyQualifiedName, currentUnit);
+			IClass returnClass = SearchType (type, currentUnit);
 //			IClass returnClass = parserContext.SearchType (type.FullyQualifiedName, null, currentUnit);
 			if (returnClass == null)
 				return null;
+				
+			// Get the list of namespaces where subclasses have to be searched.
+			// Include all namespaces for which there is an "using".
+			ArrayList ns = new ArrayList ();
+			if (currentUnit != null && currentUnit.Usings != null) {
+				foreach (IUsing us in currentUnit.Usings)
+					ns.AddRange (us.Usings);
+			}
+			// Include the calling class namesapce and all its parent namespaces
+			if (callingClass != null) {
+				string[] namespaceParts = callingClass.Namespace.Split ('.');
+				string cns = "";
+				foreach (string s in namespaceParts) {
+					if (cns.Length > 0)
+						cns += ".";
+					cns += s;
+					ns.Add (cns);
+				}
+			}
 
-			foreach (IClass iclass in parserContext.GetClassInheritanceTree (returnClass)) {
-				if (!result.Contains (iclass))
+			foreach (IClass iclass in parserContext.GetSubclassesTree (returnClass, (string[]) ns.ToArray (typeof(string)))) {
+				if (!result.Contains (iclass) && !(excludeInterfaces && (iclass.ClassType == ClassType.Interface || iclass.IsAbstract)))
 					result.Add (iclass);
 			}
+			
+			// Include all namespaces as well
+			foreach (string nss in parserContext.GetNamespaceList ("", true, true))
+				result.Add (new Namespace (nss));
+			
 			return result;
 		}
 		
@@ -1063,14 +1165,14 @@ namespace CSharpBinding.Parser
 				if (variables != null && variables.Count > 0) {
 					foreach (LocalLookupVariable v in variables) {
 						if (IsInside(new Point(caretColumn, caretLine), v.StartPos, v.EndPos)) {
-							result.Add(new Parameter (null, name, new ReturnType (v.TypeRef.SystemType)));
+							result.Add(new DefaultParameter (null, name, new ReturnType (ReturnType.GetSystemType (v.TypeRef.Type))));
 							break;
 						}
 					}
 				}
 			}
 			if (callingClass != null) {
-				ListMembers(result, callingClass);
+				ListMembers(result, callingClass, callingClass);
 			}
 			string n = "";
 			result.AddRange(parserContext.GetNamespaceContents (n, true));
