@@ -6,20 +6,14 @@ using ModifierFlags = ICSharpCode.NRefactory.Parser.AST.Modifier;
 
 namespace CSharpBinding.Parser.SharpDevelopTree
 {
-	public class Method : AbstractMethod
+	public class Method : DefaultMethod
 	{
-		public void AddModifier(ModifierEnum m)
+		public Method (string name, ReturnType type, ModifierFlags m, IRegion region, IRegion bodyRegion)
 		{
-			modifiers = modifiers | m;
-		}
-		
-		public Method (IClass declaringType, string name, ReturnType type, ModifierFlags m, IRegion region, IRegion bodyRegion)
-		{
-			FullyQualifiedName = name;
+			Name = name;
 			returnType = type;
 			this.region     = region;
 			this.bodyRegion = bodyRegion;
-			this.declaringType = declaringType;
 			modifiers = (ModifierEnum)m;
 		}
 	}
