@@ -72,24 +72,48 @@ namespace MonoDevelop.GtkCore.NodeBuilders
 			AddNewWindow ("DialogFileTemplate");
 		}
 		
+		[CommandUpdateHandler (MonoDevelop.GtkCore.GtkCommands.AddNewDialog)]
+		public void UpdateAddNewDialogToProject (CommandInfo cinfo)
+		{
+			cinfo.Visible = CanAddWindow ();
+		}
+		
 		[CommandHandler (MonoDevelop.GtkCore.GtkCommands.AddNewWindow)]
 		public void AddNewWindowToProject()
 		{
 			AddNewWindow ("WindowFileTemplate");
 		}
 		
+		[CommandUpdateHandler (MonoDevelop.GtkCore.GtkCommands.AddNewWindow)]
+		public void UpdateAddNewWindowToProject (CommandInfo cinfo)
+		{
+			cinfo.Visible = CanAddWindow ();
+		}
+		
 		[CommandHandler (MonoDevelop.GtkCore.GtkCommands.AddNewWidget)]
 		public void AddNewWidgetToProject()
 		{
 			AddNewWindow ("WidgetFileTemplate");
 		}
 		
+		[CommandUpdateHandler (MonoDevelop.GtkCore.GtkCommands.AddNewWidget)]
+		public void UpdateAddNewWidgetToProject (CommandInfo cinfo)
+		{
+			cinfo.Visible = CanAddWindow ();
+		}
+		
 		[CommandHandler (MonoDevelop.GtkCore.GtkCommands.AddNewActionGroup)]
 		public void AddNewActionGroupToProject()
 		{
 			AddNewWindow ("ActionGroupFileTemplate");
 		}
 		
+		[CommandUpdateHandler (MonoDevelop.GtkCore.GtkCommands.AddNewActionGroup)]
+		public void UpdateAddNewActionGroupToProject(CommandInfo cinfo)
+		{
+			cinfo.Visible = CanAddWindow ();
+		}
+		
 		[CommandHandler (GtkCommands.ImportGladeFile)]
 		protected void OnImportGladeFile ()
 		{
@@ -97,23 +121,62 @@ namespace MonoDevelop.GtkCore.NodeBuilders
 			GuiBuilderService.ImportGladeFile (project);
 		}
 		
+		[CommandUpdateHandler (GtkCommands.ImportGladeFile)]
+		protected void UpdateImportGladeFile (CommandInfo cinfo)
+		{
+			cinfo.Visible = CanAddWindow ();
+		}
+		
 		[CommandHandler (GtkCommands.EditIcons)]
 		protected void OnEditIcons ()
 		{
 			Project project = CurrentNode.GetParentDataItem (typeof(Project), true) as Project;
 			GuiBuilderProject gp = GtkCoreService.GetGtkInfo (project).GuiBuilderProject;
 			Stetic.Project sp = gp.SteticProject;
-			using (Stetic.Editor.EditIconFactoryDialog dlg = new Stetic.Editor.EditIconFactoryDialog (null, sp, sp.IconFactory)) {
-				dlg.Run ();
+			sp.EditIcons ();
+			gp.Save (true);
+		}
+		
+		[CommandUpdateHandler (GtkCommands.EditIcons)]
+		protected void UpdateEditIcons (CommandInfo cinfo)
+		{
+			cinfo.Visible = CanAddWindow ();
+		}
+		
+		[CommandHandler (GtkCommands.GtkSettings)]
+		protected void OnGtkSettings ()
+		{
+			Project project = CurrentNode.GetParentDataItem (typeof(Project), true) as Project;
+			IdeApp.ProjectOperations.ShowOptions (project, "SteticOptionsPanel");
+		}
+		
+		[CommandUpdateHandler (GtkCommands.EditIcons)]
+		protected void UpdateGtkSettings (CommandInfo cinfo)
+		{
+			cinfo.Visible = CanAddWindow ();
+		}
+		
+		bool CanAddWindow ()
+		{
+			DotNetProject project = CurrentNode.GetParentDataItem (typeof(Project), true) as DotNetProject;
+			if (project != null) {
+				GtkDesignInfo info = GtkCoreService.GetGtkInfo (project);
+				return (info == null || !info.GuiBuilderProject.HasError);
 			}
-			gp.Save ();
+			return false;
 		}
 		
 		public void AddNewWindow (string id)
 		{
-			Project project = CurrentNode.GetParentDataItem (typeof(Project), true) as Project;
+			DotNetProject project = CurrentNode.GetParentDataItem (typeof(Project), true) as DotNetProject;
+			if (project == null)
+				return;
+			
 			ProjectFolder folder = CurrentNode.GetParentDataItem (typeof(ProjectFolder), true) as ProjectFolder;
 			
+			if (GtkCoreService.SupportsPartialTypes (project))
+				id = "Partial" + id;
+			
 			string path;
 			if (folder != null)
 				path = folder.Path;
@@ -122,9 +185,7 @@ namespace MonoDevelop.GtkCore.NodeBuilders
 
 			IdeApp.ProjectOperations.CreateProjectFile (project, path, id);
 			
-			using (IProgressMonitor m = IdeApp.Workbench.ProgressMonitors.GetSaveProgressMonitor ()) {
-				project.Save (m);
-			}
+			IdeApp.ProjectOperations.SaveProject (project);
 			CurrentNode.Expanded = true;
 		}
 	}
