@@ -41,7 +41,6 @@ using MonoDevelop.Projects.Parser;
 using MonoDevelop.Projects.Serialization;
 
 using MonoDevelop.GtkCore.GuiBuilder;
-using MonoDevelop.GtkCore.WidgetLibrary;
 
 namespace MonoDevelop.GtkCore
 {
@@ -49,17 +48,29 @@ namespace MonoDevelop.GtkCore
 	{
 		ArrayList exportedWidgets = new ArrayList ();
 		DotNetProject project;
-		ProjectWidgetLibrary widgetLibrary;
 		GuiBuilderProject builderProject;
 		IDotNetLanguageBinding binding;
 		ProjectResourceProvider resourceProvider;
 		
+		[ItemProperty (DefaultValue=false)]
+		bool partialTypes;
+		
+		[ItemProperty (DefaultValue=true)]
+		bool generateGettext = true;
+		
+		[ItemProperty (DefaultValue=false)]
+		bool isWidgetLibrary = false;
+		
+		[ItemProperty (DefaultValue="Mono.Unix.Catalog")]
+		string gettextClass = "Mono.Unix.Catalog";
+		
 		public GtkDesignInfo ()
 		{
 		}
 		
 		public GtkDesignInfo (DotNetProject project)
 		{
+			partialTypes = GtkCoreService.SupportsPartialTypes (project);
 			IExtendedDataItem item = (IExtendedDataItem) project;
 			item.ExtendedProperties ["GtkDesignInfo"] = this;
 			Bind (project);
@@ -73,26 +84,32 @@ namespace MonoDevelop.GtkCore
 		
 		public void Dispose ()
 		{
-			if (widgetLibrary != null)
-				widgetLibrary.Dispose ();
+			if (resourceProvider != null)
+				System.Runtime.Remoting.RemotingServices.Disconnect (resourceProvider);
 			if (builderProject != null)
 				builderProject.Dispose ();
 		}
 		
 		public GuiBuilderProject GuiBuilderProject {
 			get {
-				if (builderProject == null) {
-					UpdateGtkFolder ();
+				if (builderProject == null)
 					builderProject = new GuiBuilderProject (project, SteticFile);
-				}
 				return builderProject;
 			}
 		}
 		
+		public void ReloadGuiBuilderProject ()
+		{
+			if (builderProject != null)
+				builderProject.Reload ();
+		}
+		
 		public ProjectResourceProvider ResourceProvider {
 			get {
-				if (resourceProvider == null)
+				if (resourceProvider == null) {
 					resourceProvider = new ProjectResourceProvider (project);
+					System.Runtime.Remoting.RemotingServices.Marshal (resourceProvider, null, typeof(Stetic.IResourceProvider));
+				}
 				return resourceProvider;
 			}
 		}
@@ -114,48 +131,36 @@ namespace MonoDevelop.GtkCore
 		}
 		
 		public bool IsWidgetLibrary {
-			get { return exportedWidgets.Count > 0; }
-		}
-		
-		public ProjectWidgetLibrary ProjectWidgetLibrary {
-			get {
-				if (!IsWidgetLibrary)
-					return null;
-				if (widgetLibrary == null)
-					widgetLibrary = new ProjectWidgetLibrary (project);
-				return widgetLibrary;
+			get { return isWidgetLibrary || exportedWidgets.Count > 0; }
+			set {
+				isWidgetLibrary = value;
+				if (!isWidgetLibrary)
+					exportedWidgets.Clear ();
 			}
 		}
 		
-		public Stetic.WidgetLibrary[] GetReferencedWidgetLibraries ()
-		{
-			ArrayList list = new ArrayList ();
-
-			foreach (ProjectReference pref in project.ProjectReferences) {
-				if (pref.ReferenceType == ReferenceType.Project) {
-					Project p = project.RootCombine.FindProject (pref.Reference);
-					if (p != null) {
-						GtkDesignInfo info = GtkCoreService.GetGtkInfo (p);
-						if (info != null && info.IsWidgetLibrary)
-							list.Add (info.ProjectWidgetLibrary);
-					}
-				} else if (pref.ReferenceType == ReferenceType.Gac || 
-						pref.ReferenceType == ReferenceType.Assembly) {
-					
-					Stetic.WidgetLibrary lib = GuiBuilderService.GetAssemblyLibrary (pref.Reference);
-					if (lib != null)
-						list.Add (lib);
-				}
-			}
-			if (IsWidgetLibrary)
-				list.Add (ProjectWidgetLibrary);
-
-			return (Stetic.WidgetLibrary[]) list.ToArray (typeof(Stetic.WidgetLibrary));
+		public bool GeneratePartialClasses {
+			get { return partialTypes; }
 		}
 		
-		public bool IsExported (IClass cls)
+		public bool GenerateGettext {
+			get { return generateGettext; }
+			set {
+				generateGettext = value;
+				// Set to default value if gettext is not enabled
+				if (!generateGettext) 
+					gettextClass = "Mono.Unix.Catalog";
+			}
+		}
+		
+		public string GettextClass {
+			get { return gettextClass; }
+			set { gettextClass = value; }
+		}
+		
+		public bool IsExported (string name)
 		{
-			return exportedWidgets.Contains (cls.FullyQualifiedName);
+			return exportedWidgets.Contains (name);
 		}
 		
 		public void AddExportedWidget (string name)
@@ -164,6 +169,11 @@ namespace MonoDevelop.GtkCore
 				exportedWidgets.Add (name);
 		}
 
+		public void RemoveExportedWidget (string name)
+		{
+			exportedWidgets.Remove (name);
+		}
+		
 		public IClass[] GetExportedClasses ()
 		{
 			IParserContext pctx = GuiBuilderProject.GetParserContext ();
@@ -188,15 +198,28 @@ namespace MonoDevelop.GtkCore
 			}
 		}
 		
-		public void UpdateGtkFolder ()
+		public void ForceCodeGenerationOnBuild ()
+		{
+			try {
+				FileInfo fi = new FileInfo (SteticFile);
+				fi.LastWriteTime = DateTime.Now;
+				fi = new FileInfo (SteticGeneratedFile);
+				fi.LastWriteTime = DateTime.Now;
+			} catch {
+				// Ignore errors here
+			}
+		}
+		
+		public bool UpdateGtkFolder ()
 		{
 			if (project == null)	// This happens when deserializing
-				return;
+				return false;
 
 			// This method synchronizes the current gtk project configuration info
 			// with the needed support files in the gtk-gui folder.
 
-			Directory.CreateDirectory (GtkGuiFolder);
+			Runtime.FileService.CreateDirectory (GtkGuiFolder);
+			bool projectModified = false;
 				
 			// Create the stetic file if not found
 			if (!File.Exists (SteticFile)) {
@@ -207,21 +230,58 @@ namespace MonoDevelop.GtkCore
 			}
 			
 			// Add the stetic file to the project
-			if (!project.IsFileInProject (SteticFile))
-				project.AddFile (SteticFile, BuildAction.Exclude);
-			
+			if (!project.IsFileInProject (SteticFile)) {
+				project.AddFile (SteticFile, BuildAction.EmbedAsResource);
+				projectModified = true;
+			}
+		
 			if (!File.Exists (SteticGeneratedFile)) {
 				// Generate an empty build class
 				CodeDomProvider provider = GetCodeDomProvider ();
-				Stetic.CodeGenerator.GenerateProjectCode (SteticGeneratedFile, "Stetic", provider);
+				GuiBuilderService.SteticApp.GenerateProjectCode (SteticGeneratedFile, "Stetic", provider, null);
 			}
 
 			// Add the generated file to the project, if not already there
-			if (!project.IsFileInProject (SteticGeneratedFile))
+			if (!project.IsFileInProject (SteticGeneratedFile)) {
 				project.AddFile (SteticGeneratedFile, BuildAction.Compile);
+				projectModified = true;
+			}
 
+			if (!GuiBuilderProject.HasError)
+			{
+				// Create files for all widgets
+				ArrayList partialFiles = new ArrayList ();
+				
+				foreach (GuiBuilderWindow win in GuiBuilderProject.Windows) {
+					string fn = GuiBuilderService.GenerateSteticCodeStructure (project, win.RootWidget, true, false);
+					partialFiles.Add (fn);
+					if (!project.IsFileInProject (fn)) {
+						project.AddFile (fn, BuildAction.Compile);
+						projectModified = true;
+					}
+				}
+				
+				foreach (Stetic.ActionGroupInfo ag in GuiBuilderProject.SteticProject.ActionGroups) {
+					string fn = GuiBuilderService.GenerateSteticCodeStructure (project, ag, true, false);
+					partialFiles.Add (fn);
+					if (!project.IsFileInProject (fn)) {
+						project.AddFile (fn, BuildAction.Compile);
+						projectModified = true;
+					}
+				}
+				
+				// Remove all project files which are not in the generated list
+				foreach (ProjectFile pf in project.ProjectFiles.GetFilesInPath (GtkGuiFolder)) {
+					if (pf.FilePath != SteticGeneratedFile && pf.FilePath != ObjectsFile && pf.FilePath != SteticFile && !partialFiles.Contains (pf.FilePath)) {
+						project.ProjectFiles.Remove (pf);
+						Runtime.FileService.DeleteFile (pf.FilePath);
+						projectModified = true;
+					}
+				}
+			}
+			
 			// If the project is exporting widgets, make sure the objects.xml file exists
-			if (exportedWidgets.Count > 0) {
+			if (IsWidgetLibrary) {
 				if (!File.Exists (ObjectsFile)) {
 					XmlDocument doc = new XmlDocument ();
 					doc.AppendChild (doc.CreateElement ("objects"));
@@ -229,25 +289,40 @@ namespace MonoDevelop.GtkCore
 				}
 				
 				ProjectFile file = project.GetProjectFile (ObjectsFile);
-				if (file == null)
+				if (file == null) {
 					project.AddFile (ObjectsFile, BuildAction.EmbedAsResource);
-				else if (file.BuildAction != BuildAction.EmbedAsResource)
+					projectModified = true;
+				}
+				else if (file.BuildAction != BuildAction.EmbedAsResource) {
 					file.BuildAction = BuildAction.EmbedAsResource;
+					projectModified = true;
+				}
 			}
 			
 			// Add gtk-sharp and gdk-sharp references, if not already added.
 			
-			bool gtk=false, gdk=false;
+			bool gtk=false, gdk=false, posix=false;
 			foreach (ProjectReference r in project.ProjectReferences) {
 				if (r.Reference.StartsWith ("gtk-sharp") && r.ReferenceType == ReferenceType.Gac)
 					gtk = true;
 				else if (r.Reference.StartsWith ("gdk-sharp") && r.ReferenceType == ReferenceType.Gac)
 					gdk = true;
+				else if (r.Reference.StartsWith ("Mono.Posix") && r.ReferenceType == ReferenceType.Gac)
+					posix = true;
 			}
 			if (!gtk)
 				project.ProjectReferences.Add (new ProjectReference (ReferenceType.Gac, typeof(Gtk.Widget).Assembly.FullName));
 			if (!gdk)
 				project.ProjectReferences.Add (new ProjectReference (ReferenceType.Gac, typeof(Gdk.Window).Assembly.FullName));
+				
+			if (!posix && GenerateGettext && GettextClass == "Mono.Unix.Catalog") {
+				// Add a reference to Mono.Posix. Use the version for the selected project's runtime version.
+				string aname = Runtime.SystemAssemblyService.FindInstalledAssembly ("Mono.Posix, Version=1.0.5000.0, Culture=neutral, PublicKeyToken=0738eb9f132ed756");
+				aname = Runtime.SystemAssemblyService.GetAssemblyNameForVersion (aname, project.ClrVersion);
+				project.ProjectReferences.Add (new ProjectReference (ReferenceType.Gac, aname));
+			}
+			
+			return projectModified || gtk || gdk || posix;
 		}
 		
 		CodeDomProvider GetCodeDomProvider ()
