@@ -1,26 +1,33 @@
 using System;
 using System.IO;
+using System.Text;
+using System.Collections;
+using System.Collections.Specialized;
 using System.Runtime.InteropServices;
 
-using MonoDevelop.Core.Execution;
-using MonoDevelop.Core.Gui;
-using MonoDevelop.Ide.Gui.Content;
-using MonoDevelop.Projects;
-using MonoDevelop.Core.Properties;
-using MonoDevelop.Core.AddIns;
-using MonoDevelop.Core;
-using MonoDevelop.Core.Gui.Utils;
-using MonoDevelop.SourceEditor.Properties;
-using MonoDevelop.SourceEditor.FormattingStrategy;
-using MonoDevelop.Ide.Gui;
-using MonoDevelop.Ide.Gui.Search;
-using MonoDevelop.Ide.Codons;
-using MonoDevelop.SourceEditor.Document;
-using MonoDevelop.Projects.Text;
-
 using Gtk;
 using GtkSourceView;
 
+using Mono.Addins;
+
+using MonoDevelop.Core;
+using MonoDevelop.Core.Gui;
+using MonoDevelop.Core.Gui.Utils;
+using MonoDevelop.Core.Execution;
+using MonoDevelop.Core.Properties;
+
+using MonoDevelop.Ide.Gui;
+using MonoDevelop.Ide.Gui.Search;
+using MonoDevelop.Ide.Gui.Content;
+using MonoDevelop.Ide.Codons;
+
+using MonoDevelop.Projects;
+using MonoDevelop.Projects.Text;
+using MonoDevelop.Projects.Parser;
+using MonoDevelop.Projects.Gui.Completion;
+
+using MonoDevelop.SourceEditor.FormattingStrategy;
+
 namespace MonoDevelop.SourceEditor.Gui
 {
 	public class SourceEditorDisplayBinding : IDisplayBinding
@@ -53,12 +60,14 @@ namespace MonoDevelop.SourceEditor.Gui
 				return true;
 			if (mimetype == "application/x-aspx")
 				return true;
+			if (mimetype == "application/x-ascx")
+				return true;
 
 			// If gedit can open the file, this editor also can do it
 			foreach (DesktopApplication app in DesktopApplication.GetApplications (mimetype))
 				if (app.Command == "gedit")
 					return true;
-				
+			
 			return false;
 		}
 		
@@ -81,14 +90,24 @@ namespace MonoDevelop.SourceEditor.Gui
 	}
 	
 	public class SourceEditorDisplayBindingWrapper : AbstractViewContent,
-		IEditableTextBuffer, IPositionable, IBookmarkBuffer, IDebuggableEditor, ICodeStyleOperations,
-		IDocumentInformation, IEncodedTextContent
+		IExtensibleTextEditor, IPositionable, IBookmarkBuffer, IDebuggableEditor, ICodeStyleOperations,
+		IDocumentInformation, IEncodedTextContent, IViewHistory
 	{
 		VBox mainBox;
-		HBox editorBar;
+		VBox editorBar;
 		HBox reloadBar;
+		HBox classBrowser;
+		Gtk.ComboBox classCombo;
+		Gtk.ComboBox membersCombo;
+		bool loadingMembers;
+		ListStore classStore;
+		ListStore memberStore;
+		bool classBrowserVisible = true;
 		internal FileSystemWatcher fsw;
 		IProperties properties;
+		IParseInformation memberParseInfo;
+		bool handlingParseEvent = false;
+		bool disposed;
 		
 		BreakpointEventHandler breakpointAddedHandler;
 		BreakpointEventHandler breakpointRemovedHandler;
@@ -101,7 +120,7 @@ namespace MonoDevelop.SourceEditor.Gui
 		DateTime lastSaveTime;
 		bool warnOverwrite = false;
 		
-		PropertyEventHandler propertyHanlder;
+		PropertyEventHandler propertyHandler;
 		
 		void UpdateFSW (object o, EventArgs e)
 		{
@@ -155,25 +174,86 @@ namespace MonoDevelop.SourceEditor.Gui
 			}
 		}
 		
+		public bool ClassBrowserVisible {
+			get {
+				return classBrowserVisible;
+			}
+			set {
+				classBrowser.Visible = value;
+				classBrowserVisible = value;
+				if (classBrowserVisible)
+					BindClassCombo ();
+			}
+		}
+		
 		public SourceEditorDisplayBindingWrapper ()
 		{
 			mainBox = new VBox ();
-			editorBar = new HBox ();
-			mainBox.PackStart (editorBar, false, false, 0);
+			mainBox.Spacing = 3;
+			editorBar = new VBox ();
+			mainBox.PackStart (editorBar, false, true, 0);
+			
+			classBrowser = new HBox(true, 2);
+			classCombo = new Gtk.ComboBox();
+			classCombo.WidthRequest = 1;
+			membersCombo = new Gtk.ComboBox();
+			membersCombo.WidthRequest = 1;
+			
+			// Setup the columns and column renders for the comboboxes
+			CellRendererPixbuf pixr = new CellRendererPixbuf();
+			pixr.Ypad = 0;
+			classCombo.PackStart(pixr, false);
+			classCombo.AddAttribute(pixr, "pixbuf", 0);
+			CellRenderer colr = new CellRendererText();
+			colr.Ypad = 0;
+			classCombo.PackStart(colr, true);
+			classCombo.AddAttribute(colr, "text", 1);
+			
+			pixr = new CellRendererPixbuf();
+			pixr.Ypad = 0;
+			membersCombo.PackStart(pixr, false);
+			membersCombo.AddAttribute(pixr, "pixbuf", 0);
+			colr = new CellRendererText();
+			colr.Ypad = 0;
+			membersCombo.PackStart(colr, true);
+			membersCombo.AddAttribute(colr, "text", 1);
+			
+			// Pack the controls into the editorbar just below the file name tabs.
+			classBrowser.PackStart(classCombo, true, true, 0);
+			classBrowser.PackStart(membersCombo, true, true, 0);
+			editorBar.PackEnd (classBrowser, false, true, 0);
+			
+			// Set up the data stores for the comboboxes
+			classStore = new ListStore(typeof(Gdk.Pixbuf), typeof(string), typeof(IClass));
+			classCombo.Model = classStore;	
+			memberStore = new ListStore(typeof(Gdk.Pixbuf), typeof(string), typeof(IMember));
+			membersCombo.Model = memberStore;
+   			membersCombo.Changed += new EventHandler (MemberChanged);
+			classCombo.Changed += new EventHandler (ClassChanged);
+			
 			se = new SourceEditor (this);
 			se.Buffer.ModifiedChanged += new EventHandler (OnModifiedChanged);
 			se.Buffer.MarkSet += new MarkSetHandler (OnMarkSet);
 			se.Buffer.Changed += new EventHandler (OnChanged);
 			se.View.ToggleOverwrite += new EventHandler (CaretModeChanged);
+		  	se.View.MoveCursor += new MoveCursorHandler (OnMoveCursorEvent);
+		  	se.View.ButtonReleaseEvent += new ButtonReleaseEventHandler (OnButtonReleaseEvent);
 			ContentNameChanged += new EventHandler (UpdateFSW);
 			
-			CaretModeChanged (null, null);
-			SetInitialValues ();
+			// setup a focus chain so that the editor widget gets focus when
+			// switching tabs rather than the classCombo, by default
+			Widget [] chain = new Widget [3];
+			chain[0] = se;
+			chain[1] = classCombo;
+			chain[2] = membersCombo;
+			((Container) mainBox).FocusChain = chain;
 			
-			propertyHanlder = (PropertyEventHandler) Services.DispatchService.GuiDispatch (new PropertyEventHandler (PropertiesChanged));
+			CaretModeChanged (null, null);
+			
+			propertyHandler = (PropertyEventHandler) Services.DispatchService.GuiDispatch (new PropertyEventHandler (PropertiesChanged));
 			PropertyService propertyService = (PropertyService) ServiceManager.GetService (typeof (PropertyService));
 			properties = ((IProperties) propertyService.GetProperty("MonoDevelop.TextEditor.Document.Document.DefaultDocumentAggregatorProperties", new DefaultProperties()));
-			properties.PropertyChanged += propertyHanlder;
+			properties.PropertyChanged += propertyHandler;
 			fsw = new FileSystemWatcher ();
 			fsw.Changed += (FileSystemEventHandler) Services.DispatchService.GuiDispatch (new FileSystemEventHandler (OnFileChanged));
 			UpdateFSW (null, null);
@@ -188,7 +268,23 @@ namespace MonoDevelop.SourceEditor.Gui
 				Services.DebuggingService.BreakpointRemoved += breakpointRemovedHandler;
 				Services.DebuggingService.ExecutionLocationChanged += executionChangedHandler;
 			}
+			
+			IdeApp.ProjectOperations.ParserDatabase.ParseInformationChanged += new ParseInformationEventHandler(UpdateClassBrowser);
+			
 			mainBox.ShowAll ();
+			
+			SetInitialValues ();
+		}
+		
+		public override object GetContent (Type type)
+		{
+			if (type == typeof(ICompletionWidget)) {
+				if (se.View.EnableCodeCompletion)
+					return se.View;
+				else
+					return null;
+			} else
+				return base.GetContent (type);
 		}
 		
 		public void JumpTo (int line, int column)
@@ -210,6 +306,8 @@ namespace MonoDevelop.SourceEditor.Gui
 		// http://bugzilla.gnome.org/show_bug.cgi?id=138458
 		bool changeFocus ()
 		{
+			if (disposed)
+				return false;
 			se.View.GrabFocus ();
 			se.View.ScrollToMark (se.Buffer.InsertMark, 0.3, false, 0, 0);
 			return false;
@@ -221,6 +319,8 @@ namespace MonoDevelop.SourceEditor.Gui
 		
 		public override void Dispose()
 		{
+			disposed = true;
+			
 			if (Services.DebuggingService != null) {
 				Services.DebuggingService.BreakpointAdded -= breakpointAddedHandler;
 				Services.DebuggingService.BreakpointRemoved -= breakpointRemovedHandler;
@@ -228,12 +328,24 @@ namespace MonoDevelop.SourceEditor.Gui
 			}
 
 			mainBox.Remove (se);
-			properties.PropertyChanged -= propertyHanlder;
+			properties.PropertyChanged -= propertyHandler;
 			se.Buffer.ModifiedChanged -= new EventHandler (OnModifiedChanged);
 			se.Buffer.MarkSet -= new MarkSetHandler (OnMarkSet);
 			se.Buffer.Changed -= new EventHandler (OnChanged);
 			se.View.ToggleOverwrite -= new EventHandler (CaretModeChanged);
+		  	se.View.MoveCursor -= new MoveCursorHandler (OnMoveCursorEvent);
+		  	se.View.ButtonReleaseEvent -= new ButtonReleaseEventHandler (OnButtonReleaseEvent);
 			ContentNameChanged -= new EventHandler (UpdateFSW);
+   			membersCombo.Changed -= new EventHandler (MemberChanged);
+			classCombo.Changed -= new EventHandler (ClassChanged);
+			
+			classStore.Dispose ();
+			memberStore.Dispose ();
+			
+			membersCombo.Model = null;
+			classCombo.Model = null;
+			
+			IdeApp.ProjectOperations.ParserDatabase.ParseInformationChanged -= new ParseInformationEventHandler(UpdateClassBrowser);
 			se.Dispose ();
 			fsw.Dispose ();
 			se = null;
@@ -314,6 +426,314 @@ namespace MonoDevelop.SourceEditor.Gui
 					
 				UpdateExecutionLocation ();
 			}
+			
+			IFileParserContext context = IdeApp.ProjectOperations.ParserDatabase.GetFileParserContext(fileName);
+			memberParseInfo = context.ParseFile(fileName);
+			BindClassCombo();
+		}
+		
+		public INavigationPoint BuildNavPoint ()
+		{
+			int line, column;
+			string content;
+			
+			GetLineColumnFromPosition (CursorPosition, out line, out column);
+			content = GetLineTextAtOffset (CursorPosition);
+			
+			return new TextNavigationPoint (ContentName, line, column, content);
+		}
+		
+		private void UpdateClassBrowser(object sender, ParseInformationEventArgs args)
+		{
+			// This event handler can get called when files other than the current content are updated. eg.
+			// when loading a new document. If we didn't do this check the member combo for this tab would have
+			// methods for a different class in it!
+			
+			if (ContentName == args.FileName && !handlingParseEvent) {
+				handlingParseEvent = true;
+				memberParseInfo = args.ParseInformation;
+				GLib.Timeout.Add (1000, new GLib.TimeoutHandler (BindClassCombo));
+			}
+		}
+		
+		[GLib.ConnectBefore]
+		private void OnButtonReleaseEvent (object o, ButtonReleaseEventArgs args)
+		{
+			UpdateMethodBrowser();
+		}
+		
+		private void OnMoveCursorEvent (object sender, Gtk.MoveCursorArgs args)
+		{
+			UpdateMethodBrowser();
+		}
+		
+		private void UpdateMethodBrowser()
+		{
+			if (!ClassBrowserVisible)
+				return;
+
+			if (memberParseInfo == null) {
+				classBrowser.Visible = false;
+				return;
+			}
+			
+			loadingMembers = true;
+			
+			// find out where the current cursor position is and set the combos.
+			int line;
+			int column;
+			this.GetLineColumnFromPosition (this.CursorPosition, out line, out column);
+			
+			int index = 0;
+			Gtk.TreeIter iter;
+			IClass currentClass = null;
+			
+			// check we are in the right class
+			if (classCombo.GetActiveIter (out iter))
+				currentClass = classCombo.Model.GetValue (iter, 2) as IClass;
+			
+			if (currentClass != null && currentClass.BodyRegion != null && currentClass.BodyRegion.BeginLine <= line && line <= currentClass.BodyRegion.EndLine) {
+				// we are still in the active class, just need to check method now
+				bool more = memberStore.GetIterFirst (out iter);
+			    while (more)
+				{
+			    	object member = memberStore.GetValue (iter, 2);
+			    	if (member is IField) {
+			    		if (((IField) member).Region.BeginLine <= line && line <= ((IField) member).Region.EndLine) {
+			    			membersCombo.Active = index;
+							break;
+			    		}
+			    	}
+			    	if (member is IProperty) {
+			    		if (((IProperty) member).BodyRegion.BeginLine <= line && line <= ((IProperty) member).BodyRegion.EndLine) {
+			    			membersCombo.Active = index;
+							break;
+			    		}
+			    	}
+			    	if (member is IMethod) {
+			    		if (((IMethod) member).BodyRegion.BeginLine <= line && line <= ((IMethod) member).BodyRegion.EndLine) {
+			    			membersCombo.Active = index;
+							break;
+			    		}
+			    	}
+			    	index++;
+					more = memberStore.IterNext (ref iter);
+				}
+			} else {
+				// Changed class, so rebind
+				BindClassCombo();
+			}
+			loadingMembers = false;
+		}
+		
+		
+		private bool BindClassCombo()
+		{
+			if (disposed)
+				return false;
+			
+			if (!ClassBrowserVisible)
+				return false;
+			
+			loadingMembers = true;
+			
+			try {
+				// Clear down all our local stores.
+				classStore.Clear();				
+				
+				// check the IParseInformation member variable to see if we could get ParseInformation for the 
+				// current docuement. If not we can't display class and member info so hide the browser bar.
+				if (memberParseInfo == null) {
+					classBrowser.Visible = false;
+					return false;
+				}
+				
+				ClassCollection cls = ((ICompilationUnit)memberParseInfo.BestCompilationUnit).Classes;
+				// if we've got this far then we have valid parse info - but if we have not classes the not much point
+				// in displaying the browser bar
+				if (cls.Count == 0) {
+					classBrowser.Visible = false;
+					return false;
+				}
+				
+				classBrowser.Visible = true;
+					
+				foreach (IClass c in cls) {
+					// Get the appropriate icon from the Icon service for the current IClass.
+					Gdk.Pixbuf pix = IdeApp.Services.Resources.GetIcon (IdeApp.Services.Icons.GetIcon (c), IconSize.Menu);
+					classStore.AppendValues (pix, c.Name, c);
+				}
+				
+				// find out where the current cursor position is and set the combos.
+				int line;
+				int column;
+				this.GetLineColumnFromPosition(this.CursorPosition, out line, out column);
+				for(int i = 0; i < cls.Count; i++) {
+					IClass c = cls[i];
+					if (c.BodyRegion != null && c.BodyRegion.BeginLine <= line && line <= c.BodyRegion.EndLine)	{
+						// found the right class. Now need right method
+						classCombo.Active = i;
+						BindMemberCombo(c);
+						handlingParseEvent = false;
+						
+						// return false to stop the GLib.Timeout
+						return false;
+					}
+				}
+				// Sometimes there might be no classes e.g. AssemblyInfo.cs
+				if (cls.Count > 0) {
+					for (int i = 0; i < cls.Count; i++) {
+						// If the first "class" is a delegate it will have no members 
+						if (cls[i].ClassType != ClassType.Delegate) {
+							BindMemberCombo(cls[i]);
+							classCombo.Active = i;
+							break;
+						}
+					}
+				}
+				handlingParseEvent = false;
+			}
+			finally {
+				loadingMembers = false;
+			}
+			
+			// return false to stop the GLib.Timeout
+			return false;
+		}
+		
+		
+		private void BindMemberCombo(IClass c)
+		{
+			if (!ClassBrowserVisible)
+				return;
+
+			int position = 0;
+			int activeIndex = 0;
+			// find out where the current cursor position is and set the combos.
+			int line;
+			int column;
+			this.GetLineColumnFromPosition(this.CursorPosition, out line, out column);
+			
+			membersCombo.Changed -= new EventHandler (MemberChanged);
+			// Clear down all our local stores.
+			memberStore.Clear();
+				
+			HybridDictionary methodMap = new HybridDictionary();
+			
+			Gdk.Pixbuf pix;
+			// Add items to the member drop down 
+			MethodCollection sortedMethods = c.Methods;
+			ArrayList.Adapter (sortedMethods).Sort (new CaseInsensitiveComparer ());
+			
+			foreach (IMethod method in sortedMethods) {
+				pix = IdeApp.Services.Resources.GetIcon(IdeApp.Services.Icons.GetIcon(method), IconSize.Menu); 
+				// For methods we append their parameter types too. This is a nice feature,
+				// and it is also necessay to avoid problems with overloaded methods having
+				// the same name
+				StringBuilder methodName = new StringBuilder();
+				methodName.Append(method.Name + " (");
+				for (int i = 0; i < method.Parameters.Count; i++) {
+					methodName.Append(method.Parameters [i].ReturnType.Name);
+					if (i < method.Parameters.Count - 1) methodName.Append (", ");
+				}
+				methodName.Append(")"); 
+				
+				memberStore.AppendValues(pix, methodName.ToString (), method);
+			
+				// Check if the current cursor position in inside this method
+				if (method.BodyRegion != null && method.BodyRegion.BeginLine <= line && line <= method.BodyRegion.EndLine) {
+					activeIndex = position;
+				}
+				++position;
+			}
+			foreach (IProperty property in c.Properties) {
+				pix = IdeApp.Services.Resources.GetIcon (IdeApp.Services.Icons.GetIcon (property), IconSize.Menu);
+				if (methodMap [property.Name] != null) {
+					// The unlikely case that two properties have the same name has occured.
+					// We use the fully qualified name for each subsequent property to avoid
+					// problems.
+					memberStore.AppendValues (pix, property.FullyQualifiedName, property);
+				}
+				else {
+					memberStore.AppendValues (pix, property.Name, property);
+				}
+				
+				// Check if the current cursor position in inside this property
+				if (property.BodyRegion.BeginLine <= line && line <= property.BodyRegion.EndLine) {
+					activeIndex = position;
+				}
+				++position;
+			}
+			foreach(IField member in c.Fields) {
+				pix = IdeApp.Services.Resources.GetIcon (IdeApp.Services.Icons.GetIcon (member), IconSize.Menu);
+				memberStore.AppendValues (pix, member.Name, member);
+				
+				// Check if the current cursor position in inside this field
+				if (member.Region.BeginLine <= line && line <= member.Region.EndLine) {
+					activeIndex = position;
+				}
+				++position;
+			}
+			// don't need method map anymore
+			methodMap.Clear ();
+			
+			// set active the method the cursor is in
+			membersCombo.Active = activeIndex;
+			membersCombo.Changed += new EventHandler (MemberChanged);
+		}
+		
+		private void MemberChanged(object sender, EventArgs e)
+		{
+			if (loadingMembers)
+				return;
+
+			Gtk.TreeIter iter;
+			if (membersCombo.GetActiveIter (out iter)) {	    
+				// Find the IMember object in our list store by name from the member combo
+				IMember member = (IMember) memberStore.GetValue (iter, 2);
+				int line = member.Region.BeginLine;
+				
+				// Get a handle to the current document
+				if (IdeApp.Workbench.ActiveDocument == null) {
+					return;
+				}
+				
+				// If we can we navigate to the line location of the IMember.
+				IViewContent content = (IViewContent) IdeApp.Workbench.ActiveDocument.GetContent(typeof(IViewContent));
+				if (content is IPositionable) {
+					((IPositionable)content).JumpTo (Math.Max (1, line), 1);
+				}
+			}
+		}
+		
+		private void ClassChanged(object sender, EventArgs e)
+		{
+			if (loadingMembers)
+				return;
+			
+			Gtk.TreeIter iter;
+			if (classCombo.GetActiveIter(out iter)) 	{
+				IClass selectedClass = (IClass)classStore.GetValue(iter, 2);
+				int line = selectedClass.Region.BeginLine;
+				
+				// Get a handle to the current document
+				if (IdeApp.Workbench.ActiveDocument == null) {
+					return;
+				}
+				
+				// If we can we navigate to the line location of the IMember.
+				IViewContent content = (IViewContent)IdeApp.Workbench.ActiveDocument.GetContent(typeof(IViewContent));
+				if (content is IPositionable) {
+					((IPositionable)content).JumpTo (Math.Max (1, line), 1);
+				}
+				
+				// check that selected "class" isn't a delegate
+				if (selectedClass.ClassType == ClassType.Delegate) {
+					memberStore.Clear();
+				} else {
+					BindMemberCombo(selectedClass);
+				}
+			}
 		}
 		
 		void OnBreakpointAdded (object sender, BreakpointEventArgs args)
@@ -374,7 +794,7 @@ namespace MonoDevelop.SourceEditor.Gui
 				reloadBar.ShowAll ();
 			}
 			warnOverwrite = true;
-			editorBar.PackStart (reloadBar);
+			editorBar.PackStart (reloadBar, false, true, 0);
 			reloadBar.ShowAll ();
 			WorkbenchWindow.ShowNotification = true;
 		}
@@ -398,30 +818,14 @@ namespace MonoDevelop.SourceEditor.Gui
 		
 		public void InitializeFormatter()
 		{
-			string ext = Path.GetExtension (ContentName).ToUpper ();
+			string ext = Path.GetExtension (ContentName).ToLower ();
 
 			if (ext.Length > 0) {
-				string path;
-				switch (ext) {
-					case ".CS":
-						path = "/AddIns/DefaultTextEditor/Formatter/C#";
-						break;
-					case ".VB":
-						path = "/AddIns/DefaultTextEditor/Formatter/VBNET";
-						break;
-					default:
-						// we fall back to the uppercase extension without the .
-						// ex. BOO, XML
-						path = String.Format ("/AddIns/DefaultTextEditor/Formatter/{0}", ext.Substring (1));
-						break;
-				}
-				
-				if (AddInTreeSingleton.AddInTree.TreeNodeExists (path)) {
-					IFormattingStrategy[] formatter = (IFormattingStrategy[])(AddInTreeSingleton.AddInTree.GetTreeNode(path).BuildChildItems(this)).ToArray(typeof(IFormattingStrategy));
-					if (formatter != null && formatter.Length > 0) {
-						se.View.fmtr = formatter[0];
-						return;
-					}
+				string id = ext.Substring (1);
+				TypeExtensionNode node = AddinManager.GetExtensionNode ("/AddIns/DefaultTextEditor/Formatters/" + id) as TypeExtensionNode;
+				if (node != null) {
+					se.View.fmtr = (IFormattingStrategy) node.CreateInstance (typeof(IFormattingStrategy));
+					return;
 				}
 			}
 			
@@ -441,6 +845,13 @@ namespace MonoDevelop.SourceEditor.Gui
 			se.Buffer.LoadText (val, mime);
 		}
 		
+#region IExtensibleTextEditor
+		ITextEditorExtension IExtensibleTextEditor.AttachExtension (ITextEditorExtension extension)
+		{
+			return se.View.AttachExtension (extension);
+		}
+#endregion
+
 #region IEditableTextBuffer
 		public IClipboardHandler ClipboardHandler {
 			get { return se.Buffer; }
@@ -498,6 +909,16 @@ namespace MonoDevelop.SourceEditor.Gui
 			}
 		}
 		
+		public void BeginAtomicUndo ()
+		{
+			Editor.Buffer.BeginUserAction ();
+		}
+		
+		public void EndAtomicUndo ()
+		{
+			Editor.Buffer.EndUserAction ();
+		}
+		
 		public string SelectedText {
 			get {
 				return se.Buffer.GetSelectedText ();
@@ -511,10 +932,22 @@ namespace MonoDevelop.SourceEditor.Gui
 			}
 		}
 		
+		public int GetLineLength (int line)
+		{
+			TextIter begin = Editor.Buffer.GetIterAtLine (line);
+			return begin.CharsInLine;
+		}
+
 		public int GetPositionFromLineColumn (int line, int column)
 		{
+			if (line > Editor.Buffer.LineCount)
+				return -1;
+
 			TextIter itr = se.Buffer.GetIterAtLine (line - 1);
-			itr.LineOffset = column - 1 > itr.CharsInLine ? itr.CharsInLine - 1 : column - 1;
+			if (column - 1 > itr.CharsInLine)
+				itr.LineOffset = itr.CharsInLine > 0 ? itr.CharsInLine - 1 : 0;
+			else
+				itr.LineOffset = column - 1;
 			return itr.Offset;
 		}
 		
@@ -538,7 +971,7 @@ namespace MonoDevelop.SourceEditor.Gui
 		}
 		
 #endregion
-
+		
 #region Status Bar Handling
 		IStatusBarService statusBarService = (IStatusBarService) ServiceManager.GetService (typeof (IStatusBarService));
 		
@@ -612,12 +1045,12 @@ namespace MonoDevelop.SourceEditor.Gui
 		
 		void ICodeStyleOperations.IndentSelection ()
 		{
-			se.View.IndentSelection (false);
+			se.View.IndentSelection (false, false);
 		}
 		
 		void ICodeStyleOperations.UnIndentSelection ()
 		{
-			se.View.IndentSelection (true);
+			se.View.IndentSelection (true, false);
 		}
 #endregion 
 
@@ -654,7 +1087,7 @@ namespace MonoDevelop.SourceEditor.Gui
 		{
 			TextIter it = se.Buffer.GetIterAtOffset (position);
 			line = it.Line + 1;
-			column = it.LineOffset;
+			column = it.LineOffset + 1;
 		}
 		
 		public void ShowPosition (int position)
@@ -670,6 +1103,14 @@ namespace MonoDevelop.SourceEditor.Gui
 		int ITextFile.Length {
 			get { return se.Buffer.Length; }
 		}
+		
+		char ITextFile.GetCharAt (int offset)
+		{
+			if (offset < (se.Buffer.Length - 1))
+				return se.Buffer.GetIterAtOffset (offset).Char[0];
+			else
+				return (char) 0;
+		}
 
 		public void SetBookmarked (int position, bool mark)
 		{
@@ -698,7 +1139,7 @@ namespace MonoDevelop.SourceEditor.Gui
 		{
 			se.ClearBookmarks ();
 		}
-
+		
 #region IDocumentInformation
 		string IDocumentInformation.FileName {
 			get { return ContentName != null ? ContentName : UntitledName; }
@@ -717,11 +1158,9 @@ namespace MonoDevelop.SourceEditor.Gui
 			start_line.LineOffset = 0;
 			end_line.ForwardToLineEnd ();
 			return se.Buffer.GetText (start_line.Offset, end_line.Offset - start_line.Offset);
-		}
-		
+		}		
 #endregion
 
-
 		void SetInitialValues ()
 		{
 			se.View.ModifyFont (TextEditorProperties.Font);
@@ -732,8 +1171,10 @@ namespace MonoDevelop.SourceEditor.Gui
 			se.View.InsertSpacesInsteadOfTabs = TextEditorProperties.ConvertTabsToSpaces;
 			se.View.AutoIndent = (TextEditorProperties.IndentStyle == IndentStyle.Auto);
 			se.View.AutoInsertTemplates = TextEditorProperties.AutoInsertTemplates;
+			se.View.HighlightCurrentLine = TextEditorProperties.HighlightCurrentLine;
 			se.Buffer.UnderlineErrors = TextEditorProperties.UnderlineErrors;
 			se.Buffer.Highlight = TextEditorProperties.SyntaxHighlight;
+			se.DisplayBinding.ClassBrowserVisible = TextEditorProperties.ShowClassBrowser;
 
 			if (TextEditorProperties.VerticalRulerRow > -1)
 				se.View.Margin = (uint) TextEditorProperties.VerticalRulerRow;
@@ -744,6 +1185,8 @@ namespace MonoDevelop.SourceEditor.Gui
 				se.View.TabsWidth = (uint) TextEditorProperties.TabIndent;
 			else
 				se.View.TabsWidth = (uint) 4;
+
+			se.View.WrapMode = TextEditorProperties.WrapMode;
 		}
 		
 		void PropertiesChanged (object sender, PropertyEventArgs e)
@@ -751,6 +1194,7 @@ namespace MonoDevelop.SourceEditor.Gui
 			switch (e.Key) {
 				case "DefaultFont":
 					se.View.ModifyFont (TextEditorProperties.Font);
+					se.UpdateMarkerSize ();
 					break;
 				case "ShowLineNumbers":
 					se.View.ShowLineNumbers = TextEditorProperties.ShowLineNumbers;
@@ -794,6 +1238,19 @@ namespace MonoDevelop.SourceEditor.Gui
 				case "EnableFolding":
 					// TODO
 					break;
+				case "WrapMode":
+					se.View.WrapMode = TextEditorProperties.WrapMode;
+					break;
+				case "ShowClassBrowser":
+					se.DisplayBinding.ClassBrowserVisible = TextEditorProperties.ShowClassBrowser;
+					break;
+				case "HighlightCurrentLine":
+					se.View.HighlightCurrentLine = TextEditorProperties.HighlightCurrentLine;
+					break;
+				case "ShowControlCharacters":
+					SourceEditorView.DrawWhiteSpacesEnabled = TextEditorProperties.ShowControlCharacters;
+				se.View.QueueDraw ();
+					break;
 				default:
 					Console.WriteLine ("unhandled property change: {0}", e.Key);
 					break;
