@@ -1,5 +1,6 @@
 using System;
 using System.IO;
+using System.Collections.Generic;
 using System.Runtime.InteropServices;
 
 using Gtk;
@@ -7,37 +8,45 @@ using Gdk;
 using Global = Gtk.Global;
 using GtkSourceView;
 
+using Mono.Addins;
+
 using MonoDevelop.Components.Commands;
+
 using MonoDevelop.Core;
-using MonoDevelop.Core.AddIns;
 using MonoDevelop.Core.Gui;
 using MonoDevelop.Core.Gui.Utils;
-using MonoDevelop.Ide.CodeTemplates;
+
 using MonoDevelop.Ide.Commands;
+using MonoDevelop.Ide.CodeTemplates;
 using MonoDevelop.Ide.Gui;
+using MonoDevelop.Ide.Gui.Content;
+
 using MonoDevelop.Projects;
 using MonoDevelop.Projects.Gui.Completion;
 using MonoDevelop.Projects.Parser;
-using MonoDevelop.SourceEditor.Actions;
-using MonoDevelop.SourceEditor.Document;
-using MonoDevelop.SourceEditor.InsightWindow;
-using MonoDevelop.SourceEditor.Properties;
-using MonoDevelop.SourceEditor.FormattingStrategy;
+
 using MonoDevelop.SourceEditor;
+using MonoDevelop.SourceEditor.Actions;
+using MonoDevelop.SourceEditor.FormattingStrategy;
 
 namespace MonoDevelop.SourceEditor.Gui
 {
-	public class SourceEditorView : SourceView, IFormattableDocument, ICompletionWidget
-	{	
+	public class SourceEditorView : SourceView, ICompletionWidget, ITextEditorExtension
+	{
+		public static readonly bool HighlightCurrentLineSupported;
+		public static bool DrawWhiteSpacesEnabled = TextEditorProperties.ShowControlCharacters;
+		
 		public readonly SourceEditor ParentEditor;
 		internal IFormattingStrategy fmtr = new DefaultFormattingStrategy ();
 		public SourceEditorBuffer buf;
 		bool codeCompleteEnabled;
-		bool autoHideCompletionWindow = true;
 		bool autoInsertTemplates;
 		EditActionCollection editactions = new EditActionCollection ();
 		LanguageItemWindow languageItemWindow;
-		CodeCompletionDataProvider currentCompletionProvider;
+		ITextEditorExtension extension;
+		TextEditor thisEditor;
+		DrawControlCharacterImp controlsDrawer;
+		event EventHandler completionContextChanged;
 		
 		const int LanguageItemTipTimer = 800;
 		ILanguageItem tipItem;
@@ -55,6 +64,25 @@ namespace MonoDevelop.SourceEditor.Gui
 			set { autoInsertTemplates = value; }
 		}
 		
+		static SourceEditorView ()
+		{
+			SourceView view = new SourceView ();
+			try
+			{
+				GetHighlightCurrentLine (view.Handle);
+				HighlightCurrentLineSupported = true;
+			}
+			catch
+			{
+				HighlightCurrentLineSupported = false;
+			}
+			finally
+			{
+				view.Destroy ();
+				view = null;
+			}
+		}
+		
 		protected SourceEditorView (IntPtr p): base (p)
 		{
 		}
@@ -68,26 +96,39 @@ namespace MonoDevelop.SourceEditor.Gui
 			SmartHomeEnd = true;
 			ShowLineNumbers = true;
 			ShowLineMarkers = true;
+			controlsDrawer = new DrawControlCharacterImp (this);
+			HighlightCurrentLine = true;
 			buf.PlaceCursor (buf.StartIter);
 			GrabFocus ();
 			buf.MarkSet += new MarkSetHandler (BufferMarkSet);
 			buf.Changed += new EventHandler (BufferChanged);
 			LoadEditActions ();
-			this.Events = this.Events | EventMask.PointerMotionMask | EventMask.LeaveNotifyMask;
+			this.Events = this.Events | EventMask.PointerMotionMask | EventMask.LeaveNotifyMask | EventMask.ExposureMask;
 		}
 		
 		public new void Dispose ()
 		{
-			if (currentCompletionProvider != null) {
-				currentCompletionProvider.Dispose ();
-				currentCompletionProvider = null;
-			}
+			controlsDrawer.Detach ();
 			HideLanguageItemWindow ();
 			buf.MarkSet -= new MarkSetHandler (BufferMarkSet);
 			buf.Changed -= new EventHandler (BufferChanged);
 			base.Dispose ();
 		}
 		
+		public ITextEditorExtension AttachExtension (ITextEditorExtension extension)
+		{
+			this.extension = extension;
+			return this;
+		}
+		
+		TextEditor ThisEditor {
+			get {
+				if (thisEditor == null)
+					thisEditor = new TextEditor (ParentEditor.DisplayBinding);
+				return thisEditor;
+			}
+		}
+		
 		protected override bool OnMotionNotifyEvent (Gdk.EventMotion evnt)
 		{
 			bool res = base.OnMotionNotifyEvent (evnt);
@@ -135,8 +176,12 @@ namespace MonoDevelop.SourceEditor.Gui
 				tipItem = item;
 
 				HideLanguageItemWindow ();
+				
+				IParserContext pctx = GetParserContext ();
+				if (pctx == null)
+					return false;
 
-				languageItemWindow = new LanguageItemWindow (tipItem, GetParserContext (), GetAmbience ());
+				languageItemWindow = new LanguageItemWindow (tipItem, pctx, GetAmbience ());
 				
 				int ox, oy;
 				this.GetWindow (TextWindowType.Text).GetOrigin (out ox, out oy);
@@ -177,8 +222,7 @@ namespace MonoDevelop.SourceEditor.Gui
 		void BufferMarkSet (object s, MarkSetArgs a)
 		{
 			if (a.Mark.Name == "insert") {
-				if (autoHideCompletionWindow)
-					CompletionListWindow.HideWindow ();
+				NotifyCompletionContextChanged ();
 				buf.HideHighlightLine ();
 			}
 		}
@@ -186,30 +230,28 @@ namespace MonoDevelop.SourceEditor.Gui
 		void LoadEditActions ()
 		{
 			string editactionsPath = "/AddIns/DefaultTextEditor/EditActions";
-			if (AddInTreeSingleton.AddInTree.TreeNodeExists (editactionsPath)) {
-				IEditAction[] actions = (IEditAction[]) (AddInTreeSingleton.AddInTree.GetTreeNode (editactionsPath).BuildChildItems (this)).ToArray (typeof(IEditAction));
-				foreach (IEditAction action in actions)
-					editactions.Add (action);
-			} else {
-				Console.WriteLine ("/AddIns/DefaultTextEditor/EditActions addin path not found");
-			}
+			IEditAction[] actions = (IEditAction[]) AddinManager.GetExtensionObjects (editactionsPath, typeof(IEditAction));
+			foreach (IEditAction action in actions)
+				editactions.Add (action);
 		}
 		
 		protected override bool OnFocusOutEvent (EventFocus e)
 		{
-			CompletionListWindow.HideWindow ();
+			NotifyCompletionContextChanged ();
 			return base.OnFocusOutEvent (e);
 		}
 		
 		void BufferChanged (object s, EventArgs args)
 		{
-			if (autoHideCompletionWindow)
-				CompletionListWindow.HideWindow ();
+			NotifyCompletionContextChanged ();
 		}
 		
 		protected override bool OnButtonPressEvent (Gdk.EventButton e)
 		{
-			CompletionListWindow.HideWindow ();
+			if (extension != null)
+				extension.CursorPositionChanged ();
+
+			NotifyCompletionContextChanged ();
 			HideLanguageItemWindow ();
 			
 			if (!ShowLineMarkers)
@@ -287,63 +329,23 @@ namespace MonoDevelop.SourceEditor.Gui
 			Global.PropagateEvent (this, evnt);
 		}
 
-		// remove the current line, including the new line chars
 		internal void DeleteLine ()
 		{
 			TextIter start = buf.GetIterAtMark (buf.InsertMark);
-			start.LineOffset = 0;
 			TextIter end = start;
-			if (!end.EndsLine ())
+			
+			if (!end.EndsLine ()) {
+				// Delete up to, but not including, the end-of-line marker
 				end.ForwardToLineEnd ();
-			// delete the end of the line
-			end.Offset++;
+			} else {
+				// Delete the end-of-line marker
+				end.Offset++;
+			}
+			
 			using (AtomicUndo a = new AtomicUndo (buf)) 
 				buf.Delete (ref start, ref end);
 		}
 
-		internal void ShowCodeCompletion (char key)
-		{
-			if (EnableCodeCompletion && PeekCharIsWhitespace ()) {
-				PrepareCompletionDetails (buf.GetIterAtMark (buf.InsertMark));
-				CompletionListWindow.ShowWindow (key, GetCodeCompletionDataProvider (false), this);
-			}
-		}
-
-		// FIXME: this method is broken
-		// and will trigger at various incorrect spots
-		internal void TriggerCodeComplete ()
-		{
-			
-			TextIter iter = buf.GetIterAtMark (buf.InsertMark);
-			char triggerChar = '\0';
-			TextIter triggerIter = TextIter.Zero;
-			do {
-				if (iter.Char == null || iter.Char.Length == 0) {
-					break;
-				}
-				switch (iter.Char[0]) {
-				case ' ':
-				case '\t':
-				case '.':
-				case '(':
-				case '[':
-					triggerIter = iter;
-					triggerChar = iter.Char[0];
-					break;
-				}
-				if (!triggerIter.Equals (TextIter.Zero))
-					break;
-				iter.BackwardChar ();
-			} while (iter.LineOffset != 0);
-
-			if (triggerIter.Equals (TextIter.Zero))
-				return;
-			triggerIter.ForwardChar ();
-			
-			PrepareCompletionDetails(triggerIter);
-			CompletionListWindow.ShowWindow (triggerChar, GetCodeCompletionDataProvider (true), this);
-		}
-
 		IParserContext GetParserContext ()
 		{
 			string file = ParentEditor.DisplayBinding.IsUntitled ? ParentEditor.DisplayBinding.UntitledName : ParentEditor.DisplayBinding.ContentName;
@@ -367,43 +369,10 @@ namespace MonoDevelop.SourceEditor.Gui
 			}
 		}
 
-		CodeCompletionDataProvider GetCodeCompletionDataProvider (bool ctrl)
-		{
-			if (currentCompletionProvider != null)
-				currentCompletionProvider.Dispose ();
-			IParserContext ctx = GetParserContext ();
-			string file = ParentEditor.DisplayBinding.IsUntitled ? ParentEditor.DisplayBinding.UntitledName : ParentEditor.DisplayBinding.ContentName;
-			currentCompletionProvider = new CodeCompletionDataProvider (ctx, GetAmbience (), file, ctrl);
-			return currentCompletionProvider;
-		}
-			
 		internal bool MonodocResolver ()
 		{
 			TextIter insertIter = buf.GetIterAtMark (buf.InsertMark);
-			TextIter triggerIter = TextIter.Zero;
-			try {
-				do {
-					switch (insertIter.Char[0]) {
-					case ' ':
-					case '\t':
-					case '\r':
-					case '\n':
-					case '.':
-					case ';':
-					case '(':
-					case '[':
-						triggerIter = insertIter;
-						break;
-					}
-					if (!triggerIter.Equals (TextIter.Zero))
-						break;
-					insertIter.ForwardChar ();
-				} while (insertIter.LineOffset != 0);
-				triggerIter.ForwardChar ();
-			} catch {
-				return false;
-			}
-			ILanguageItem languageItem = GetLanguageItem (triggerIter);
+			ILanguageItem languageItem = GetLanguageItem (insertIter);
 			
 			if (languageItem == null)
 				return false;
@@ -421,6 +390,9 @@ namespace MonoDevelop.SourceEditor.Gui
 				fileName = ParentEditor.DisplayBinding.UntitledName;
 
 			IParserContext ctx = GetParserContext ();
+			if (ctx == null)
+				return null;
+
 			IExpressionFinder expressionFinder = null;
 			if (fileName != null)
 				expressionFinder = ctx.GetExpressionFinder (fileName);
@@ -456,12 +428,20 @@ namespace MonoDevelop.SourceEditor.Gui
 		
 		protected override bool OnKeyPressEvent (Gdk.EventKey evnt)
 		{
+			evntCopy = evnt;
+			if (extension == null)
+				return ((ITextEditorExtension)this).KeyPress (evnt.Key, evnt.State);
+			else
+				return extension.KeyPress (evnt.Key, evnt.State);
+		}
+		
+		Gdk.EventKey evntCopy;
+
+		bool ITextEditorExtension.KeyPress (Gdk.Key key, Gdk.ModifierType modifier)
+		{
+			Gdk.EventKey evnt = evntCopy;
 			HideLanguageItemWindow ();
 			
-			if (CompletionListWindow.ProcessKeyEvent (evnt))
-				return true;
-			
-			autoHideCompletionWindow = false;
 			bool res = false;
 			IEditAction action = editactions.GetAction (evnt.Key, evnt.State);
 			if (action != null) {
@@ -479,9 +459,20 @@ namespace MonoDevelop.SourceEditor.Gui
 			} else {
 				res = base.OnKeyPressEvent (evnt);
 			}
-			autoHideCompletionWindow = true;
 			return res;
 		}
+		
+		protected override void OnMoveCursor (MovementStep step, int count, bool extend_selection)
+		{
+			if (extension != null)
+				extension.CursorPositionChanged ();
+			base.OnMoveCursor (step, count, extend_selection);
+		}
+		
+		void ITextEditorExtension.CursorPositionChanged ()
+		{
+			NavigationService.Log (ParentEditor.DisplayBinding.BuildNavPoint ());
+		}
 
 		internal bool GotoSelectionEnd ()
 		{
@@ -520,7 +511,7 @@ namespace MonoDevelop.SourceEditor.Gui
 		}
 
 		public string GetLeadingWhiteSpace (int line) {
-			string lineText = ((IFormattableDocument)this).GetLineAsString (line);
+			string lineText = ThisEditor.GetLineText (line + 1);
 			int index = 0;
 			while (index < lineText.Length && System.Char.IsWhiteSpace (lineText[index])) {
     				index++;
@@ -595,37 +586,48 @@ namespace MonoDevelop.SourceEditor.Gui
 
 
 #region Indentation
-		public bool IndentSelection (bool unindent)
+		public bool IndentSelection (bool unindent, bool requireLineSelection)
 		{
 			TextIter begin, end;
-			if (!buf.GetSelectionBounds (out begin, out end))
-				return false;
+			bool hasSelection = buf.GetSelectionBounds (out begin, out end);
+			if (!hasSelection) {
+				if (requireLineSelection)
+					return false;
+				else
+					begin = end = buf.GetIterAtMark (buf.InsertMark);
+			}
 			
 			int y0 = begin.Line, y1 = end.Line;
+			
+			if (requireLineSelection && y0 == y1)
+				return false;
 
 			// If last line isn't selected, it's illogical to indent it.
-			if (end.StartsLine())
+			if (end.StartsLine() && hasSelection && y0 != y1)
 				y1--;
 
-			if (y0 == y1)
-				return false;
-			
 			using (AtomicUndo a = new AtomicUndo (buf)) {
 				if (unindent)
 					UnIndentLines (y0, y1);
 				else
 					IndentLines (y0, y1);
-				SelectLines (y0, y1);
+				if (hasSelection)
+					SelectLines (y0, y1);
 			}
 			
 			return true;
 		}
 
+		string IndentString
+		{
+			get { return !InsertSpacesInsteadOfTabs ? "\t" : new string (' ', (int) TabsWidth); }
+		}
+		
 		public void FormatLine ()
 		{
-			if (((IFormattableDocument)this).IndentStyle == IndentStyle.Smart) {
+			if (TextEditorProperties.IndentStyle == IndentStyle.Smart) {
 				TextIter iter = buf.GetIterAtMark (buf.InsertMark);
-				fmtr.FormatLine (this, iter.Line, iter.Offset, '\n');
+				fmtr.FormatLine (ThisEditor, iter.Line + 1, iter.Offset, '\n', IndentString, TextEditorProperties.AutoInsertCurlyBracket);
 			}
 			IndentLine ();
 		}
@@ -636,7 +638,7 @@ namespace MonoDevelop.SourceEditor.Gui
 
 			// preserve offset in line
 			int offset = 0;
-			int chars = fmtr.IndentLine (this, iter.Line);
+			int chars = fmtr.IndentLine (ThisEditor, iter.Line + 1, IndentString);
 			offset += chars;
 
 			// FIXME: not quite right yet
@@ -693,148 +695,58 @@ namespace MonoDevelop.SourceEditor.Gui
 		void SelectLines (int y0, int y1)
 		{
 			Buffer.PlaceCursor (Buffer.GetIterAtLine (y0));
-			
-			TextIter end = Buffer.GetIterAtLine (y1);
-			end.ForwardToLineEnd ();
-			Buffer.MoveMark ("selection_bound", end);
+
+			if (y1 < Buffer.EndIter.Line) {
+				TextIter end = Buffer.GetIterAtLine (y1 + 1);
+				end.LineOffset = 0;
+				Buffer.MoveMark ("selection_bound", end);
+			} else
+				Buffer.MoveMark ("selection_bound", Buffer.EndIter);
 		}
 
-		void PrepareCompletionDetails(TextIter iter)
+#endregion
+
+
+#region ICompletionWidget
+
+		void NotifyCompletionContextChanged ()
 		{
-			Gdk.Rectangle rect = GetIterLocation (Buffer.GetIterAtMark (Buffer.InsertMark));
+			if (completionContextChanged != null)
+				completionContextChanged (this, EventArgs.Empty);
+		}
+
+		event EventHandler ICompletionWidget.CompletionContextChanged {
+			add { completionContextChanged += value; }
+			remove { completionContextChanged -= value; }
+		}
+
+		ICodeCompletionContext ICompletionWidget.CreateCodeCompletionContext (int triggerOffset)
+		{
+			TextIter iter = Buffer.GetIterAtOffset (triggerOffset);
+			Gdk.Rectangle rect = GetIterLocation (iter);
 			int wx, wy;
 			BufferToWindowCoords (Gtk.TextWindowType.Widget, rect.X, rect.Y + rect.Height, out wx, out wy);
 			int tx, ty;
 			GdkWindow.GetOrigin (out tx, out ty);
 
-			this.completionX = tx + wx;
-			this.completionY = ty + wy;
-			this.textHeight = rect.Height;
-			this.triggerMark = buf.CreateMark (null, iter, true);
-		}
-#endregion
-
-#region IFormattableDocument
-		string IFormattableDocument.GetLineAsString (int ln)
-		{
-			TextIter begin = Buffer.GetIterAtLine (ln);
-			TextIter end = begin;
-			if (!end.EndsLine ())
-				end.ForwardToLineEnd ();
-			
-			return begin.GetText (end);
+			CodeCompletionContext ctx = new CodeCompletionContext ();
+			ctx.TriggerOffset = iter.Offset;
+			ctx.TriggerLine = iter.Line;
+			ctx.TriggerLineOffset = iter.LineOffset;
+			ctx.TriggerXCoord = tx + wx;
+			ctx.TriggerYCoord = ty + wy;
+			ctx.TriggerTextHeight = rect.Height;
+			return ctx;
 		}
 		
-		void IFormattableDocument.BeginAtomicUndo ()
+		string ICompletionWidget.GetCompletionText (ICodeCompletionContext ctx)
 		{
-			Buffer.BeginUserAction ();
-		}
-		void IFormattableDocument.EndAtomicUndo ()
-		{
-			Buffer.EndUserAction ();
-		}
-		
-		void IFormattableDocument.ReplaceLine (int ln, string txt)
-		{
-			TextIter begin = Buffer.GetIterAtLine (ln);
-			TextIter end = begin;
-			if (!end.EndsLine ())
-				end.ForwardToLineEnd ();
-			
-			Buffer.Delete (ref begin, ref end);
-			Buffer.Insert (ref begin, txt);
-		}
-		
-		IndentStyle IFormattableDocument.IndentStyle
-		{
-			get { return TextEditorProperties.IndentStyle; }
-		}
-		
-		bool IFormattableDocument.AutoInsertCurlyBracket
-		{
-			get { return TextEditorProperties.AutoInsertCurlyBracket; }
-		}
-		
-		string IFormattableDocument.TextContent
-		{ get { return ParentEditor.DisplayBinding.Text; } }
-		
-		int IFormattableDocument.TextLength
-		{ get { return Buffer.EndIter.Offset; } }
-		
-		char IFormattableDocument.GetCharAt (int offset)
-		{
-			TextIter it = Buffer.GetIterAtOffset (offset);
-			return it.Char[0];
-		}
-		
-		void IFormattableDocument.Insert (int offset, string text)
-		{
-			TextIter insertIter = Buffer.GetIterAtOffset (offset);
-			Buffer.Insert (ref insertIter, text);
-		}
-		
-		string IFormattableDocument.IndentString
-		{
-			get { return !InsertSpacesInsteadOfTabs ? "\t" : new string (' ', (int) TabsWidth); }
-		}
-		
-		int IFormattableDocument.GetClosingBraceForLine (int ln, out int openingLine)
-		{
-			int offset = MonoDevelop.Projects.Gui.Completion.TextUtilities.SearchBracketBackward
-				(this, Buffer.GetIterAtLine (ln).Offset - 1, '{', '}');
-			
-			openingLine = offset == -1 ? -1 : Buffer.GetIterAtOffset (offset).Line;
-			return offset;
-		}
-		
-		void IFormattableDocument.GetLineLengthInfo (int ln, out int offset, out int len)
-		{
-			TextIter begin = Buffer.GetIterAtLine (ln);
-			offset = begin.Offset;
-			len = begin.CharsInLine;
-		}
-#endregion
-
-#region ICompletionWidget
-
-		private int completionX;
-		int ICompletionWidget.TriggerXCoord
-		{
-			get
-			{
-				return completionX;
-			}
+			return Buffer.GetText (Buffer.GetIterAtOffset (ctx.TriggerOffset), Buffer.GetIterAtMark (Buffer.InsertMark), false);
 		}
 
-		private int completionY;
-		int ICompletionWidget.TriggerYCoord
+		void ICompletionWidget.SetCompletionText (ICodeCompletionContext ctx, string partial_word, string complete_word)
 		{
-			get
-			{
-				return completionY;
-			}
-		}
-
-		private int textHeight;
-		int ICompletionWidget.TriggerTextHeight
-		{
-			get
-			{
-				return textHeight;
-			}
-		}
-
-		string ICompletionWidget.CompletionText
-		{
-			get
-			{
-				return Buffer.GetText (Buffer.GetIterAtMark (triggerMark), Buffer.GetIterAtMark (Buffer.InsertMark), false);
-			}
-		}
-
-		void ICompletionWidget.SetCompletionText (string partial_word, string complete_word)
-		{
-			TextIter offsetIter = buf.GetIterAtMark(triggerMark);
+			TextIter offsetIter = buf.GetIterAtOffset (ctx.TriggerOffset);
 			TextIter endIter = buf.GetIterAtOffset (offsetIter.Offset + partial_word.Length);
 			buf.MoveMark (buf.InsertMark, offsetIter);
 			buf.Delete (ref offsetIter, ref endIter);
@@ -847,14 +759,6 @@ namespace MonoDevelop.SourceEditor.Gui
 			buf.InsertAtCursor (text);
 		}
 		
-		string ICompletionWidget.Text
-		{
-			get
-			{
-				return buf.Text;
-			}
-		}
-
 		int ICompletionWidget.TextLength
 		{
 			get
@@ -873,31 +777,6 @@ namespace MonoDevelop.SourceEditor.Gui
 			return buf.GetText(buf.GetIterAtOffset (startOffset), buf.GetIterAtOffset(endOffset), true);
 		}
 
-		private TextMark triggerMark;
-		int ICompletionWidget.TriggerOffset
-		{
-			get
-			{
-				return buf.GetIterAtMark (triggerMark).Offset;
-			}
-		}
-
-		int ICompletionWidget.TriggerLine
-		{
-			get
-			{
-				return buf.GetIterAtMark (triggerMark).Line;
-			}
-		}
-
-		int ICompletionWidget.TriggerLineOffset
-		{
-			get
-			{
-				return buf.GetIterAtMark (triggerMark).LineOffset;
-			}
-		}
-
 		Gtk.Style ICompletionWidget.GtkStyle
 		{
 			get
@@ -905,24 +784,194 @@ namespace MonoDevelop.SourceEditor.Gui
 				return Style.Copy();
 			}
 		}
-
-		bool PeekCharIsWhitespace ()
-		{
-			TextIter start = buf.GetIterAtMark (buf.InsertMark);
-			TextIter end = buf.GetIterAtLine (start.Line);
-			end.Offset = start.Offset + 1;
-			string text = buf.GetText (start, end, true);
-			// if it is not whitespace or the start of a comment
-			// we disable completion except for by ctl+space
-			if (text.Length == 1)
-				return IsSeparator (text[0]);
-			return true;
-		}
+#endregion
 		
-		bool IsSeparator (char c)
+#region HighlightCurrentLine functionality
+		//gboolean gtk_source_view_get_highlight_current_line (GtkSourceView *view);
+		[DllImport ("gtksourceview-1.0", EntryPoint="gtk_source_view_get_highlight_current_line")]
+		static extern bool GetHighlightCurrentLine (IntPtr raw);
+
+		//void gtk_source_view_set_highlight_current_line (GtkSourceView *view, gboolean show);
+		[DllImport("gtksourceview-1.0", EntryPoint="gtk_source_view_set_highlight_current_line")]
+		static extern void SetHighlightCurrentLine (IntPtr raw, bool show);
+
+		[GLib.Property ("highlight-current-line")]
+		public bool HighlightCurrentLine {
+			get  {
+				if (HighlightCurrentLineSupported)
+					return GetHighlightCurrentLine (Handle);
+				return false;
+			}
+			set  {
+				if (HighlightCurrentLineSupported)
+					SetHighlightCurrentLine (Handle, value);
+			}
+		}
+#endregion
+
+#region Drawing control characters functionality
+		class DrawControlCharacterImp
 		{
-			// FIXME: this is language specific.
-			return !System.Char.IsLetterOrDigit (c) && c != '_';
+			SourceView view;
+
+			public DrawControlCharacterImp (SourceView view)
+			{
+				this.view = view;
+				view.WidgetEventAfter += OnWidgetEvent;
+			}
+
+			public void Detach ()
+			{
+				view.WidgetEventAfter -= OnWidgetEvent;
+				view = null;
+			}
+
+			void OnWidgetEvent (object o, WidgetEventAfterArgs args)
+			{
+				if (args.Event.Type == Gdk.EventType.Expose &&
+				    o is TextView &&
+				    args.Event.Window == view.GetWindow (TextWindowType.Text))
+				{
+					int x, y;
+					view.WindowToBufferCoords (TextWindowType.Text,
+					                           args.Event.Window.ClipRegion.Clipbox.X,
+					                           args.Event.Window.ClipRegion.Clipbox.Y,
+					                           out x, out y);
+
+					TextIter start, end;
+					int topLine;
+					view.GetLineAtY (out start, y, out topLine);
+					view.GetLineAtY (out end, y + args.Event.Window.ClipRegion.Clipbox.Height, out topLine);
+					end.ForwardToLineEnd ();
+					Draw (args.Event.Window, view, start, end);
+				}
+			}
+
+			static Cairo.Color GetDrawingColorForIter (TextView view, TextIter iter)
+			{
+				TextIter start, end;
+				Gdk.Color color;
+				Gdk.Color bgColor;
+				
+				if (iter.Buffer.GetSelectionBounds (out start, out end) && iter.InRange (start, end)) {
+					bgColor = view.Style.Base (StateType.Selected);
+					color = view.Style.Text (StateType.Selected);
+				} else {
+					bgColor = view.Style.Base (StateType.Normal);
+					color = view.Style.Text (StateType.Normal);
+				}
+				
+				//simple interpolation 1/4 of way between BG colour and text colour
+				int red   = (bgColor.Red   * 3 + color.Red  ) / 4;
+				int green = (bgColor.Green * 3 + color.Green) / 4;
+				int blue  = (bgColor.Blue  * 3 + color.Blue ) / 4;
+
+				return new Cairo.Color ((double)(red) / UInt16.MaxValue,
+				                        (double)(green) / UInt16.MaxValue,
+				                        (double)(blue) / UInt16.MaxValue);
+			}
+
+			static void DrawSpaceAtIter (Cairo.Context cntx, TextView view, TextIter iter)
+			{
+				Gdk.Rectangle rect = view.GetIterLocation (iter);
+				int x, y;
+				view.BufferToWindowCoords (TextWindowType.Text,
+				                           rect.X + rect.Width / 2,
+				                           rect.Y + rect.Height / 2,
+				                           out x, out y);
+				cntx.Save ();
+				cntx.Color =  GetDrawingColorForIter (view, iter);
+				//no overlap on the circle, even if context is set to LineCap.Square
+				cntx.LineCap = Cairo.LineCap.Butt;
+				
+				cntx.MoveTo (x, y);
+				cntx.Arc (x, y, 0.5, 0, 2 * Math.PI);
+				
+				cntx.Stroke ();
+				cntx.Restore ();
+			}
+
+			static void DrawTabAtIter (Cairo.Context cntx, TextView view, TextIter iter)
+			{
+				Gdk.Rectangle rect = view.GetIterLocation (iter);
+				int x, y;
+				view.BufferToWindowCoords (TextWindowType.Text,
+				                           rect.X,
+				                           rect.Y + rect.Height / 2,
+				                           out x, out y);
+				cntx.Save ();
+				cntx.Color =  GetDrawingColorForIter (view, iter);
+				
+				double arrowSize = 3;
+				cntx.MoveTo (x + 2, y + 0);
+				cntx.RelLineTo (new Cairo.Distance (rect.Width - 4, 0));
+				cntx.RelLineTo (new Cairo.Distance (-arrowSize, -arrowSize));
+				cntx.RelMoveTo (new Cairo.Distance (arrowSize, arrowSize));
+				cntx.RelLineTo (new Cairo.Distance (-arrowSize, arrowSize));
+				
+				cntx.Stroke ();
+				cntx.Restore ();
+			}
+
+			static void DrawLineEndAtIter (Cairo.Context cntx, TextView view, TextIter iter)
+			{
+				Gdk.Rectangle rect = view.GetIterLocation (iter);
+				int x, y;
+				view.BufferToWindowCoords (TextWindowType.Text,
+				                           rect.X,
+				                           rect.Y + rect.Height / 2,
+				                           out x, out y);
+				cntx.Save ();
+				cntx.Color =  GetDrawingColorForIter (view, iter);
+				
+				double arrowSize = 3;
+				cntx.MoveTo (x + 10, y);
+				cntx.RelLineTo (new Cairo.Distance (0, -arrowSize));
+				cntx.RelMoveTo (new Cairo.Distance (0, arrowSize));
+				cntx.RelLineTo (new Cairo.Distance (-8, 0));
+				cntx.RelLineTo (new Cairo.Distance (arrowSize, arrowSize));
+				cntx.RelMoveTo (new Cairo.Distance (-arrowSize, -arrowSize));
+				cntx.RelLineTo (new Cairo.Distance (arrowSize, -arrowSize));
+				
+				cntx.Stroke ();
+				cntx.Restore ();
+			}
+
+			static void Draw (Gdk.Drawable drawable, TextView view, TextIter start, TextIter end)
+			{
+				if (DrawWhiteSpacesEnabled)
+				{
+					Cairo.Context cntx = Gdk.CairoHelper.Create (drawable);
+					
+					//shift to pixel grid to reduce antialiasing
+					cntx.Antialias = Cairo.Antialias.Default;
+					cntx.LineCap = Cairo.LineCap.Square;
+					cntx.LineWidth = 1;
+					cntx.Translate (0.5, 0.5);
+
+					TextIter iter = start;
+					while (iter.Compare (end) <= 0)
+					{
+						switch (iter.Char)
+						{
+							case " ":
+								DrawSpaceAtIter (cntx, view, iter);
+								break;
+							case "\t":
+								DrawTabAtIter (cntx, view, iter);
+								break;
+							case "\n":
+							case "\r":
+								DrawLineEndAtIter (cntx, view, iter);
+								break;
+							default:
+								break;
+						}
+						if (! iter.ForwardChar ())
+							break;
+					}
+				}
+			}
 		}
 #endregion
 	}
