@@ -27,6 +27,7 @@
 //
 
 using System;
+using System.Text;
 using System.Collections;
 using System.Threading;
 using Gtk;
@@ -41,7 +42,7 @@ using MonoDevelop.Ide.Gui;
 
 namespace MonoDevelop.NUnit
 {
-	public class TestResultsPad: GuiSyncObject, IPadContent, ITestProgressMonitor
+	class TestResultsPad: IPadContent, ITestProgressMonitor
 	{
 		NUnitService testService = (NUnitService) ServiceManager.GetService (typeof(NUnitService));
 		
@@ -50,8 +51,8 @@ namespace MonoDevelop.NUnit
 		HPaned book;
 		Gtk.Tooltips tips = new Gtk.Tooltips ();
 		
-		Label infoFailed = new Label (GettextCatalog.GetString ("<b>Failed</b>: ") + "0");
-		Label infoIgnored = new Label (GettextCatalog.GetString ("<b>Ignored</b>: ") + "0");
+		Label infoFailed = new Label (GettextCatalog.GetString ("<b>Failed</b>: {0}", 0));
+		Label infoIgnored = new Label (GettextCatalog.GetString ("<b>Ignored</b>: {0}", 0));
 		Label infoCurrent = new Label ();
 		HBox labels;
 		
@@ -63,7 +64,9 @@ namespace MonoDevelop.NUnit
 		TextView outputView;
 		Widget outputViewScrolled;
 		VSeparator infoSep;
+		
 		ToolButton buttonStop;
+		ToolButton buttonRun;
 		
 		ToggleToolButton buttonSuccess;
 		ToggleToolButton buttonFailures;
@@ -97,7 +100,7 @@ namespace MonoDevelop.NUnit
 			panel = new VBox ();
 			
 			Toolbar toolbar = new Toolbar ();
-			toolbar.IconSize = IconSize.SmallToolbar;
+			toolbar.IconSize = IconSize.Menu;
 			panel.PackStart (toolbar, false, false, 0);
 			
 			buttonSuccess = new ToggleToolButton ();
@@ -130,7 +133,7 @@ namespace MonoDevelop.NUnit
 			buttonOutput = new ToggleToolButton ();
 			buttonOutput.Label = GettextCatalog.GetString ("Output");
 			buttonOutput.Active = false;
-			buttonOutput.IconWidget = Services.Resources.GetImage (MonoDevelop.Core.Gui.Stock.OutputIcon, IconSize.SmallToolbar);
+			buttonOutput.IconWidget = Services.Resources.GetImage (MonoDevelop.Core.Gui.Stock.OutputIcon, IconSize.Menu);
 			buttonOutput.Toggled += new EventHandler (OnShowOutputToggled);
 			buttonOutput.IsImportant = true;
 			buttonOutput.SetTooltip (tips, GettextCatalog.GetString ("Show Output"), GettextCatalog.GetString ("Show Output"));
@@ -138,7 +141,11 @@ namespace MonoDevelop.NUnit
 			
 			toolbar.Insert (new SeparatorToolItem (), -1);
 			
-			buttonStop = new ToolButton ("gtk-stop");
+			buttonRun = new ToolButton (new Gtk.Image (Gtk.Stock.Execute, IconSize.Menu), GettextCatalog.GetString ("Run Test"));
+			buttonRun.IsImportant = true;
+			toolbar.Insert (buttonRun, -1);
+			
+			buttonStop = new ToolButton (Gtk.Stock.Stop);
 			toolbar.Insert (buttonStop, -1);
 			
 			toolbar.ToolbarStyle = ToolbarStyle.BothHoriz;
@@ -186,6 +193,8 @@ namespace MonoDevelop.NUnit
 			infoSep = new VSeparator ();
 			
 			resultLabel.UseMarkup = true;
+			infoCurrent.Ellipsize = Pango.EllipsizeMode.Start;
+			infoCurrent.WidthRequest = 0;
 			runPanel.PackStart (resultLabel, false, false, 0);
 			runPanel.PackStart (progressBar, false, false, 0);
 			runPanel.PackStart (infoCurrent, true, true, 10);	
@@ -205,6 +214,7 @@ namespace MonoDevelop.NUnit
 			progressBar.HeightRequest = infoFailed.SizeRequest().Height;
 			
 			buttonStop.Clicked += new EventHandler (OnStopClicked);
+			buttonRun.Clicked += new EventHandler (OnRunClicked);
 			failuresTreeView.ButtonReleaseEvent += new Gtk.ButtonReleaseEventHandler (OnPopupMenu);
 			
 			Control.ShowAll ();
@@ -236,14 +246,6 @@ namespace MonoDevelop.NUnit
 			UpdateCounters ();
 		}
 		
-		public string Id {
-			get { return "MonoDevelop.NUnit.TestResultsPad"; }
-		}
-		
-		public string DefaultPlacement {
-			get { return "Bottom"; }
-		}
-		
 		bool Running {
 			get { return running; }
 			set {
@@ -268,8 +270,8 @@ namespace MonoDevelop.NUnit
 		
 		void UpdateCounters ()
 		{
-			infoFailed.Markup = GettextCatalog.GetString ("<b>Failed</b>: ") + testsFailed;
-			infoIgnored.Markup = GettextCatalog.GetString ("<b>Ignored</b>: ") + testsIgnored;
+			infoFailed.Markup = GettextCatalog.GetString ("<b>Failed</b>: {0}", testsFailed);
+			infoIgnored.Markup = GettextCatalog.GetString ("<b>Ignored</b>: {0}", testsIgnored);
 		}
 		
 		public void InitializeTestRun (UnitTest test)
@@ -292,6 +294,7 @@ namespace MonoDevelop.NUnit
 			resultLabel.Hide ();
 			labels.Show ();
 			buttonStop.Sensitive = true;
+			buttonRun.Sensitive = false;
 			
 			failuresStore.Clear ();
 			outputView.Buffer.Clear ();
@@ -305,7 +308,7 @@ namespace MonoDevelop.NUnit
 		
 		public void AddStartMessage ()
 		{
-			Gdk.Pixbuf infoIcon = failuresTreeView.RenderIcon (Gtk.Stock.DialogInfo, Gtk.IconSize.SmallToolbar, "");
+			Gdk.Pixbuf infoIcon = failuresTreeView.RenderIcon (Gtk.Stock.DialogInfo, Gtk.IconSize.Menu, "");
 			string msg = string.Format (GettextCatalog.GetString ("Running tests for <b>{0}</b> configuration <b>{1}</b>"), rootTest.Name, configuration);
 			failuresStore.AppendValues (infoIcon, msg, rootTest);
 		}
@@ -323,7 +326,7 @@ namespace MonoDevelop.NUnit
 			if (errorMessage != null)
 				msg += ": " + errorMessage;
 
-			Gdk.Pixbuf stock = failuresTreeView.RenderIcon (Gtk.Stock.DialogError, Gtk.IconSize.SmallToolbar, "");
+			Gdk.Pixbuf stock = failuresTreeView.RenderIcon (Gtk.Stock.DialogError, Gtk.IconSize.Menu, "");
 			TreeIter testRow = failuresStore.AppendValues (stock, msg, null);
 			failuresStore.AppendValues (testRow, null, Escape (error.GetType().Name + ": " + error.Message), null);
 			TreeIter row = failuresStore.AppendValues (testRow, null, GettextCatalog.GetString ("Stack Trace"), null);
@@ -342,8 +345,13 @@ namespace MonoDevelop.NUnit
 			resultLabel.Show ();
 			labels.Hide ();
 			buttonStop.Sensitive = false;
+			buttonRun.Sensitive = true;
 			
-			resultLabel.Markup = GettextCatalog.GetString ("<b>Tests</b>: ") + testsRun + GettextCatalog.GetString ("  <b>Failed</b>: ") + testsFailed + GettextCatalog.GetString ("  <b>Ignored</b>: ") + testsIgnored;
+			StringBuilder sb = new StringBuilder ();
+			sb.Append (GettextCatalog.GetString ("<b>Tests</b>: {0}", testsRun)).Append ("  ");
+			sb.Append (GettextCatalog.GetString ("<b>Failed</b>: {0}", testsFailed)).Append ("  ");
+			sb.Append (GettextCatalog.GetString ("<b>Ignored</b>: {0}", testsIgnored));
+			resultLabel.Markup = sb.ToString ();
 			
 			Running = false;
 		}
@@ -354,10 +362,16 @@ namespace MonoDevelop.NUnit
 			failuresStore.AppendValues (CircleImage.Failure, GettextCatalog.GetString ("Test execution cancelled."), null);
 		}
 		
+		void OnRunClicked (object sender, EventArgs args)
+		{
+			NUnitService testService = (NUnitService) ServiceManager.GetService (typeof(NUnitService));
+			testService.RunTest (rootTest);
+		}
+		
 		void OnPopupMenu (object o, Gtk.ButtonReleaseEventArgs args)
 		{
 			if (args.Event.Button == 3) {
-				IdeApp.CommandService.ShowContextMenu ("/SharpDevelop/Views/TestChart/ContextMenu");
+				IdeApp.CommandService.ShowContextMenu ("/SharpDevelop/Views/TestResultsPad/ContextMenu");
 			}
 		}
 		
@@ -466,7 +480,8 @@ namespace MonoDevelop.NUnit
 				if (!buttonIgnored.Active)
 					return;
 				TreeIter testRow = failuresStore.AppendValues (CircleImage.NotRun, Escape (test.FullName), test);
-				failuresStore.AppendValues (testRow, null, Escape (result.Message), test);
+				if (result.Message != null)
+					failuresStore.AppendValues (testRow, null, Escape (result.Message), test);
 				failuresTreeView.ScrollToCell (failuresStore.GetPath (testRow), null, false, 0, 0);
 			}
 			if (result.ConsoleOutput != null)
@@ -478,7 +493,7 @@ namespace MonoDevelop.NUnit
 		
 		string Escape (string s)
 		{
-			return s.Replace ("<","&lt;").Replace (">","&gt;");
+			return GLib.Markup.EscapeText (s);
 		}
 		
 		void ITestProgressMonitor.EndTest (UnitTest test, UnitTestResult result)
@@ -525,5 +540,45 @@ namespace MonoDevelop.NUnit
 		
 		public event TestHandler CancelRequested;
 	}
+	
+	class TestMonitor: GuiSyncObject, ITestProgressMonitor
+	{
+		// TestResultsPad can't be a GuiSyncObject because there are some
+		// object identity issues. If the pad is registered using the
+		// proxy, it will get different hash codes depending on the caller.
+		
+		ITestProgressMonitor monitor;
+		TestResultsPad pad;
+		
+		public TestMonitor (TestResultsPad pad) {
+			this.pad = pad;
+			this.monitor = pad;
+		}
+		public void InitializeTestRun (UnitTest test) {
+			pad.InitializeTestRun (test);
+		}
+		public void FinishTestRun () {
+			pad.FinishTestRun ();
+		}
+		public void Cancel () {
+			pad.Cancel ();
+		}
+		public void BeginTest (UnitTest test) {
+			monitor.BeginTest (test);
+		}
+		public void EndTest (UnitTest test, UnitTestResult result) {
+			monitor.EndTest (test, result);
+		}
+		public void ReportRuntimeError (string message, Exception exception) {
+			monitor.ReportRuntimeError (message, exception);
+		}
+		public bool IsCancelRequested {
+			get { return monitor.IsCancelRequested; }
+		}
+		public event TestHandler CancelRequested {
+			add { monitor.CancelRequested += value; }
+			remove { monitor.CancelRequested -= value; }
+		}
+	}
 }
 
