@@ -9,6 +9,7 @@ using System.Xml.Serialization;
 using MonoDevelop.Prj2Make.Schema.Prjx;
 using MonoDevelop.Prj2Make.Schema.Csproj;
 using MonoDevelop.Projects;
+using MonoDevelop.Projects.Text;
 using MonoDevelop.Core;
 
 using CSharpBinding;
@@ -346,7 +347,7 @@ namespace MonoDevelop.Prj2Make
 								refs += "-r:$(" + pi2.makename_ext + ")";
 								deps += "$(" + pi2.makename_ext + ")";
 							}
-							catch(System.NullReferenceException exc)
+							catch(System.NullReferenceException)
 							{
 								refs += String.Format("-r:{0}.dll", rf.Name);
 								deps += String.Format("# Missing dependency project {1} ID:{0}?", rf.Project, 
@@ -447,9 +448,15 @@ namespace MonoDevelop.Prj2Make
 		}
 		
 		public string CreatePrjxFromCsproj (string csprojFileName, IProgressMonitor monitor)
+		{
+			DotNetProject project = CreatePrjxFromCsproj (csprojFileName, monitor, true);
+			return project.FileName;
+		}
+
+		public DotNetProject CreatePrjxFromCsproj (string csprojFileName, IProgressMonitor monitor, bool save)
 		{
 			try {
-				FileStream fsIn = null;
+				TextFileReader fsIn = null;
 				XmlSerializer xmlDeSer = null;
 				MonoDevelop.Prj2Make.Schema.Csproj.VisualStudioProject csprojObj = null;
 				
@@ -465,7 +472,7 @@ namespace MonoDevelop.Prj2Make
 				prjxObj.FileName = prjxFileName;
 
 				// Load the csproj
-				fsIn = new FileStream (csprojFileName, FileMode.Open);	    
+				fsIn = new TextFileReader (csprojFileName);
 				xmlDeSer = new XmlSerializer (typeof(VisualStudioProject));
 				csprojObj = (VisualStudioProject) xmlDeSer.Deserialize (fsIn);	    
 				fsIn.Close();
@@ -477,6 +484,7 @@ namespace MonoDevelop.Prj2Make
 				prjxObj.Description = "";
 				prjxObj.NewFileSearch = NewFileSearch.None;
 				prjxObj.EnableViewState = true;
+				prjxObj.DefaultNamespace = csprojObj.CSHARP.Build.Settings.RootNamespace;
 
 				GetContents (prjxObj, csprojObj.CSHARP.Files.Include, prjxObj.ProjectFiles, monitor);
 				monitor.Step (1);
@@ -496,19 +504,26 @@ namespace MonoDevelop.Prj2Make
 						));
 				}
 				monitor.Step (1);
-				prjxObj.Save (monitor);
+				if (save)
+					prjxObj.Save (monitor);
 				monitor.Step (1);
-				return prjxFileName;
+				return prjxObj;
 
 			} catch (Exception ex) {
 				monitor.ReportError (GettextCatalog.GetString ("Could not import project:") + csprojFileName, ex);
-				return null;
+				throw;
 			} finally {
 				monitor.EndTask ();
 			}
 		}
 
 		public string MsSlnToCmbxHelper (string slnFileName, IProgressMonitor monitor)
+		{
+			Combine c = MsSlnToCmbxHelper (slnFileName, monitor, true);
+			return c.FileName;
+		}
+
+		public Combine MsSlnToCmbxHelper (string slnFileName, IProgressMonitor monitor, bool save)
 		{
 			Combine cmbxObj = new Combine();
 			cmbxFileName = String.Format ("{0}.mds",
@@ -533,26 +548,37 @@ namespace MonoDevelop.Prj2Make
 						monitor.ReportWarning (GettextCatalog.GetString ("Project file not found: ") + pi.csprojpath);
 						continue;
 					}
-					string prjName = CreatePrjxFromCsproj (mappedPath, monitor);
-					monitor.Step (1);
-					if (prjName != null)
-						cmbxObj.AddEntry (prjName, monitor);
-					else
+					DotNetProject prj = CreatePrjxFromCsproj (mappedPath, monitor, save);
+					if (prj == null)
 						return null;
+
+					monitor.Step (1);
+					if (save) {
+						string prjName = prj.FileName;
+						if (prjName != null)
+							cmbxObj.AddEntry (prjName, monitor);
+						else
+							return null;
+					} else {
+						cmbxObj.Entries.Add (prj);
+					}
 					monitor.Step (1);
 				}
 				
 				monitor.EndTask ();
 				monitor.Step (1);
 
-				cmbxObj.Save (cmbxFileName, monitor);
+				cmbxObj.FileName = cmbxFileName;
+				if (save)
+					cmbxObj.Save (cmbxFileName, monitor);
+
 				monitor.Step (1);
-				return cmbxFileName;
+				return cmbxObj;
 			}
 			catch (Exception e)
 			{
 				monitor.ReportError (GettextCatalog.GetString ("The solution could not be imported."), e);
-				return null;
+				throw;
 			}
 			finally
 			{
@@ -701,6 +727,8 @@ namespace MonoDevelop.Prj2Make
 					continue;
 				}
 				flOut.Name = name;
+				// Adding here as GetDefaultResourceIdInternal needs flOut.Project
+				files.Add (flOut);
 				
 				switch (fl.SubType)
 				{
@@ -719,6 +747,7 @@ namespace MonoDevelop.Prj2Make
 						break;
 					case MonoDevelop.Prj2Make.Schema.Csproj.FileBuildAction.EmbeddedResource:
 						flOut.BuildAction = BuildAction.EmbedAsResource;
+						flOut.ResourceId = MSBuildProject.GetDefaultResourceIdInternal (flOut);
 						break;
 					case MonoDevelop.Prj2Make.Schema.Csproj.FileBuildAction.None:
 						flOut.BuildAction = BuildAction.Nothing;
@@ -726,8 +755,6 @@ namespace MonoDevelop.Prj2Make
 				}
 				flOut.DependsOn = fl.DependentUpon;
 				flOut.Data = "";
-				
-				files.Add (flOut);
 			}
 		}
 
@@ -761,17 +788,23 @@ namespace MonoDevelop.Prj2Make
 			return confObj;
 		}
 		
-		string MapPath (string basePath, string relPath)
+		internal static string MapPath (string basePath, string relPath)
 		{
 			if (relPath == null || relPath.Length == 0)
 				return null;
 			
-			string path = relPath.Replace ("\\", "/");
+			string path = relPath;
+			if (Path.DirectorySeparatorChar != '\\')
+				path = path.Replace ("\\", "/");
+
 			if (char.IsLetter (path [0]) && path.Length > 1 && path[1] == ':')
 				return null;
 			
 			if (basePath != null)
 				path = Path.Combine (basePath, path);
+
+			if (System.IO.File.Exists (path))
+				return path;
 				
 			if (Path.IsPathRooted (path)) {
 					
